@@ -11,6 +11,8 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -20,13 +22,23 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 from contextlib import contextmanager
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    ContextManager,
+    Iterator,
+)
 
 # External imports
 import pytest
+from typing_extensions import Protocol
 
 # Bokeh imports
 from bokeh.server.server import Server
 
+if TYPE_CHECKING:
+    from bokeh.application import Application
+
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
@@ -41,22 +53,22 @@ __all__ = (
 # General API
 #-----------------------------------------------------------------------------
 
+class MSL(Protocol):
+    def __call__(self, application: Application, port: int | None = None, **server_kwargs: Any) -> ContextManager[Server]: ...
+
 @pytest.fixture
-def ManagedServerLoop(unused_tcp_port):
+def ManagedServerLoop(unused_tcp_port: int) -> MSL:
     @contextmanager
-    def msl(application, port=None, **server_kwargs):
+    def msl(application: Application, port: int | None = None, **server_kwargs: Any) -> Iterator[Server]:
         if port is None:
             port = unused_tcp_port
         server = Server(application, port=port, **server_kwargs)
-        try:
-            server.start()
-            yield server
-        finally:
-            server.unlisten()
-            server.stop()
+        server.start()
+        yield server
+        server.unlisten()
+        server.stop()
     return msl
 
-
 #-----------------------------------------------------------------------------
 # Dev API
 #-----------------------------------------------------------------------------
