@@ -11,6 +11,8 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -23,14 +25,27 @@ import socket
 import time
 from contextlib import closing
 from threading import Thread
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    Tuple,
+)
 
 # External imports
 import pytest
 from selenium.webdriver.common.action_chains import ActionChains
 from selenium.webdriver.support import expected_conditions as EC
-from selenium.webdriver.support.ui import WebDriverWait
+from selenium.webdriver.support.wait import WebDriverWait
 from tornado.ioloop import IOLoop
 from tornado.web import RequestHandler
+from typing_extensions import Protocol
+
+if TYPE_CHECKING:
+    from selenium.webdriver.common.keys import _KeySeq
+    from selenium.webdriver.remote.webdriver import WebDriver
+    from selenium.webdriver.remote.webelement import WebElement
 
 # Bokeh imports
 import bokeh.server.views.ws as ws
@@ -38,6 +53,11 @@ from bokeh._testing.util.selenium import INIT, RESULTS, wait_for_canvas_resize
 from bokeh.io import save
 from bokeh.server.server import Server
 
+if TYPE_CHECKING:
+    from bokeh._testing.plugins.file_server import SimpleWebServer
+    from bokeh.application.handlers.function import ModifyDoc
+    from bokeh.models.layouts import LayoutDOM
+
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
@@ -63,7 +83,7 @@ __all__ = (
 #-----------------------------------------------------------------------------
 
 @pytest.fixture
-def output_file_url(request, file_server):
+def output_file_url(request: pytest.FixtureRequest, file_server: SimpleWebServer) -> str:
     from bokeh.io import output_file
     filename = request.function.__name__ + '.html'
     file_obj = request.fspath.dirpath().join(filename)
@@ -72,7 +92,7 @@ def output_file_url(request, file_server):
 
     output_file(file_path, mode='inline')
 
-    def tear_down():
+    def tear_down() -> None:
         if file_obj.isfile():
             file_obj.remove()
     request.addfinalizer(tear_down)
@@ -80,13 +100,13 @@ def output_file_url(request, file_server):
     return file_server.where_is(url)
 
 @pytest.fixture
-def test_file_path_and_url(request, file_server):
+def test_file_path_and_url(request: pytest.FixtureRequest, file_server: SimpleWebServer) -> Tuple[str, str]:
     filename = request.function.__name__ + '.html'
     file_obj = request.fspath.dirpath().join(filename)
     file_path = file_obj.strpath
     url = file_path.replace('\\', '/')  # Windows-proof
 
-    def tear_down():
+    def tear_down() -> None:
         if file_obj.isfile():
             file_obj.remove()
     request.addfinalizer(tear_down)
@@ -95,38 +115,37 @@ def test_file_path_and_url(request, file_server):
 
 
 class _ExitHandler(RequestHandler):
-    def initialize(self, io_loop):
+    def initialize(self, io_loop: IOLoop) -> None:
         self.io_loop = io_loop
-    async def get(self, *args, **kwargs):
+    async def get(self, *args: Any, **kwargs: Any) -> None:
         self.io_loop.stop()
 
 
-
-def find_free_port():
+def find_free_port() -> int:
     with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as s:
         s.bind(('', 0))
         return s.getsockname()[1]
 
+class BokehAppInfo(Protocol):
+    def __call__(self, modify_doc: ModifyDoc) -> Tuple[str, ws.MessageTestPort]: ...
+
+class HasNoConsoleErrors(Protocol):
+    def __call__(self, webdriver: WebDriver) -> bool: ...
+
 @pytest.fixture
-def bokeh_app_info(request, driver):
+def bokeh_app_info(request: pytest.FixtureRequest, driver: WebDriver) -> BokehAppInfo:
     ''' Start a Bokeh server app and return information needed to test it.
 
-    Returns a tuple (url, message_test_port), where the latter is defined as
-
-        namedtuple('MessageTestPort', ['sent', 'received'])
-
-    and will contain all messages that the Bokeh Server sends/receives while
-    running during the test.
+    Returns a tuple (url, message_test_port), where the latter is an instance of
+    ``MessageTestPort`` dataclass, and will contain all messages that the Bokeh
+    Server sends/receives while running during the test.
 
     '''
 
-    def func(modify_doc):
-
-        from collections import namedtuple
-        MessageTestPort = namedtuple('MessageTestPort', ['sent', 'received'])
-        ws._message_test_port = MessageTestPort([], [])
+    def func(modify_doc: ModifyDoc) -> Tuple[str, ws.MessageTestPort]:
+        ws._message_test_port = ws.MessageTestPort(sent=[], received=[])
         port = find_free_port()
-        def worker():
+        def worker() -> None:
             io_loop = IOLoop()
             server = Server({'/': modify_doc},
                             port=port,
@@ -138,8 +157,8 @@ def bokeh_app_info(request, driver):
         t = Thread(target=worker)
         t.start()
 
-        def cleanup():
-            driver.get("http://localhost:%d/exit" % port)
+        def cleanup() -> None:
+            driver.get(f"http://localhost:{port}/exit")
 
             # XXX (bev) this line is a workaround for https://github.com/bokeh/bokeh/issues/7970
             # and should be removed when that issue is resolved
@@ -150,40 +169,27 @@ def bokeh_app_info(request, driver):
 
         request.addfinalizer(cleanup)
 
-        return "http://localhost:%d/" % port, ws._message_test_port
+        return f"http://localhost:{port}/", ws._message_test_port
 
     return func
 
+class _ElementMixin:
+    _driver: WebDriver
 
-class _BokehPageMixin:
-    @property
-    def results(self):
-        WebDriverWait(self._driver, 10).until(EC.staleness_of(self.test_div))
-        self.test_div = self._driver.find_element_by_class_name("bokeh-test-div")
-        return self._driver.execute_script(RESULTS)
-
-    @property
-    def driver(self):
-        return self._driver
-
-    def init_results(self):
-        self._driver.execute_script(INIT)
-        self.test_div = self._driver.find_element_by_class_name("bokeh-test-div")
-
-    def click_element_at_position(self, element, x, y):
+    def click_element_at_position(self, element: WebElement, x: int, y: int) -> None:
         actions = ActionChains(self._driver)
         actions.move_to_element_with_offset(element, x, y)
         actions.click()
         actions.perform()
 
-    def double_click_element_at_position(self, element, x, y):
+    def double_click_element_at_position(self, element: WebElement, x: int, y: int) -> None:
         actions = ActionChains(self._driver)
         actions.move_to_element_with_offset(element, x, y)
         actions.click()
         actions.click()
         actions.perform()
 
-    def drag_element_at_position(self, element, x, y, dx, dy, mod=None):
+    def drag_element_at_position(self, element: WebElement, x: int, y: int, dx: int, dy: int, mod: _KeySeq | None = None) -> None:
         actions = ActionChains(self._driver)
         if mod:
             actions.key_down(mod)
@@ -195,69 +201,90 @@ class _BokehPageMixin:
             actions.key_up(mod)
         actions.perform()
 
-    def send_keys(self, *keys):
+    def send_keys(self, *keys: _KeySeq) -> None:
         actions = ActionChains(self._driver)
         actions.send_keys(*keys)
         actions.perform()
 
-    def has_no_console_errors(self):
+class _CanvasMixin(_ElementMixin):
+    canvas: WebElement
+
+    def click_canvas_at_position(self, x: int, y: int) -> None:
+        self.click_element_at_position(self.canvas, x, y)
+
+    def double_click_canvas_at_position(self, x: int, y: int) -> None:
+        self.double_click_element_at_position(self.canvas, x, y)
+
+    def click_custom_action(self) -> None:
+        button = self._driver.find_element_by_class_name("bk-toolbar-button-custom-action")
+        button.click()
+
+    def drag_canvas_at_position(self, x: int, y: int, dx: int, dy: int, mod: _KeySeq | None = None) -> None:
+        self.drag_element_at_position(self.canvas, x, y, dx, dy, mod)
+
+    def get_toolbar_button(self, name: str) -> WebElement:
+        return self._driver.find_element_by_class_name(f"bk-tool-icon-{name}")
+
+class _BokehPageMixin(_ElementMixin):
+
+    test_div: WebElement
+    _driver: WebDriver
+    _has_no_console_errors: HasNoConsoleErrors
+
+    @property
+    def results(self) -> Dict[str, Any]:
+        WebDriverWait(self._driver, 10).until(EC.staleness_of(self.test_div))
+        self.test_div = self._driver.find_element_by_class_name("bokeh-test-div")
+        return self._driver.execute_script(RESULTS)
+
+    @property
+    def driver(self) -> WebDriver:
+        return self._driver
+
+    def init_results(self) -> None:
+        self._driver.execute_script(INIT)
+        self.test_div = self._driver.find_element_by_class_name("bokeh-test-div")
+
+    def has_no_console_errors(self) -> bool:
         return self._has_no_console_errors(self._driver)
 
 class _BokehModelPage(_BokehPageMixin):
 
-    def __init__(self, model, driver, output_file_url, has_no_console_errors):
+    def __init__(self, model: LayoutDOM, driver: WebDriver, output_file_url: str, has_no_console_errors: HasNoConsoleErrors) -> None:
         self._driver = driver
         self._model = model
         self._has_no_console_errors = has_no_console_errors
 
         save(self._model)
-
         self._driver.get(output_file_url)
-
         self.init_results()
 
-
-class _CanvasMixin:
-    def click_canvas_at_position(self, x, y):
-        self.click_element_at_position(self.canvas, x, y)
-
-    def double_click_canvas_at_position(self, x, y):
-        self.double_click_element_at_position(self.canvas, x, y)
-
-    def click_custom_action(self):
-        button = self._driver.find_element_by_class_name("bk-toolbar-button-custom-action")
-        button.click()
-
-    def drag_canvas_at_position(self, x, y, dx, dy, mod=None):
-        self.drag_element_at_position(self.canvas, x, y, dx, dy, mod)
-
-    def get_toolbar_button(self, name):
-        return self.driver.find_element_by_class_name('bk-tool-icon-' + name)
-
 @pytest.fixture()
-def bokeh_model_page(driver, output_file_url, has_no_console_errors):
-    def func(model):
+def bokeh_model_page(driver: WebDriver, output_file_url: str,
+        has_no_console_errors: HasNoConsoleErrors) -> Callable[[LayoutDOM], _BokehModelPage]:
+    def func(model: LayoutDOM) -> _BokehModelPage:
         return _BokehModelPage(model, driver, output_file_url, has_no_console_errors)
     return func
 
 class _SinglePlotPage(_BokehModelPage, _CanvasMixin):
 
     # model may be a layout, but should only contain a single plot
-    def __init__(self, model, driver, output_file_url, has_no_console_errors):
+    def __init__(self, model: LayoutDOM, driver: WebDriver, output_file_url: str, has_no_console_errors: HasNoConsoleErrors) -> None:
         super().__init__(model, driver, output_file_url, has_no_console_errors)
 
         self.canvas = self._driver.find_element_by_tag_name('canvas')
         wait_for_canvas_resize(self.canvas, self._driver)
 
 @pytest.fixture()
-def single_plot_page(driver, output_file_url, has_no_console_errors):
-    def func(model):
+def single_plot_page(driver: WebDriver, output_file_url: str,
+        has_no_console_errors: HasNoConsoleErrors) -> Callable[[LayoutDOM], _SinglePlotPage]:
+    def func(model: LayoutDOM) -> _SinglePlotPage:
         return _SinglePlotPage(model, driver, output_file_url, has_no_console_errors)
     return func
 
 class _BokehServerPage(_BokehPageMixin, _CanvasMixin):
 
-    def __init__(self, modify_doc, driver, bokeh_app_info, has_no_console_errors):
+    def __init__(self, modify_doc: ModifyDoc, driver: WebDriver, bokeh_app_info: BokehAppInfo, has_no_console_errors: HasNoConsoleErrors) -> None:
         self._driver = driver
         self._has_no_console_errors = has_no_console_errors
 
@@ -271,8 +298,9 @@ class _BokehServerPage(_BokehPageMixin, _CanvasMixin):
         wait_for_canvas_resize(self.canvas, self._driver)
 
 @pytest.fixture()
-def bokeh_server_page(driver, bokeh_app_info, has_no_console_errors):
-    def func(modify_doc):
+def bokeh_server_page(driver: WebDriver, bokeh_app_info: BokehAppInfo,
+        has_no_console_errors: HasNoConsoleErrors) -> Callable[[ModifyDoc], _BokehServerPage]:
+    def func(modify_doc: ModifyDoc) -> _BokehServerPage:
         return _BokehServerPage(modify_doc, driver, bokeh_app_info, has_no_console_errors)
     return func
 
