@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -12,20 +12,17 @@ on the Document.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
+from __future__ import annotations
 
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
-# Standard library imports
-
-# External imports
-
 # Bokeh imports
+from ..application import SessionContext
 from .lifecycle import LifecycleHandler
 
 #-----------------------------------------------------------------------------
@@ -43,10 +40,12 @@ __all__ = (
 class DocumentLifecycleHandler(LifecycleHandler):
     ''' Calls on_session_destroyed callbacks defined on the Document.
 
+    .. autoclasstoc::
+
     '''
 
-    def __init__(self, *args, **kwargs):
-        super(DocumentLifecycleHandler, self).__init__(*args, **kwargs)
+    def __init__(self) -> None:
+        super().__init__()
         self._on_session_destroyed = _on_session_destroyed
 
 #-----------------------------------------------------------------------------
@@ -57,7 +56,7 @@ class DocumentLifecycleHandler(LifecycleHandler):
 # Private API
 #-----------------------------------------------------------------------------
 
-def _on_session_destroyed(session_context):
+def _on_session_destroyed(session_context: SessionContext) -> None:
     '''
     Calls any on_session_destroyed callbacks defined on the Document
     '''
@@ -67,9 +66,8 @@ def _on_session_destroyed(session_context):
         try:
             callback(session_context)
         except Exception as e:
-            log.warning('DocumentLifeCycleHandler on_session_destroyed '
-                        'callback %s failed with following error: %s'
-                        % (callback, e))
+            log.warning("DocumentLifeCycleHandler on_session_destroyed "
+                        f"callback {callback} failed with following error: {e}")
     if callbacks:
         # If any session callbacks were defined garbage collect after deleting all references
         del callback
