@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -24,6 +24,8 @@ A client session has two primary uses:
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -32,16 +34,32 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
+from typing import TYPE_CHECKING, Any, Dict
 from urllib.parse import quote_plus
 
+# External imports
+from typing_extensions import Literal
+
+if TYPE_CHECKING:
+    from tornado.ioloop import IOLoop
+
 # Bokeh imports
+from ..core.types import ID
 from ..document import Document
-from ..resources import DEFAULT_SERVER_HTTP_URL, _SessionCoordinates
-from ..util.browser import NEW_PARAM
+from ..resources import DEFAULT_SERVER_HTTP_URL, SessionCoordinates
+from ..util.browser import NEW_PARAM, BrowserLike, BrowserTarget
 from ..util.token import generate_jwt_token, generate_session_id
 from .states import ErrorReason
 from .util import server_url_for_websocket_url, websocket_url_for_server_url
 
+if TYPE_CHECKING:
+    from ..document.events import DocumentPatchedEvent, SessionCallbackAdded, SessionCallbackRemoved
+    from ..models.layouts import LayoutDOM
+    from ..protocol.messages.patch_doc import patch_doc
+    from ..protocol.messages.server_info_reply import ServerInfo
+    from ..server.callbacks import DocumentCallbackGroup
+    from .connection import ClientConnection
+
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
@@ -61,7 +79,8 @@ __all__ = (
 # General API
 #-----------------------------------------------------------------------------
 
-def pull_session(session_id=None, url='default', io_loop=None, arguments=None):
+def pull_session(session_id: ID | None = None, url: str = "default", io_loop: IOLoop | None = None,
+        arguments: Dict[str, str] | None = None, max_message_size: int = 20*1024*1024) -> ClientSession:
     ''' Create a session by loading the current server-side document.
 
     ``session.document`` will be a fresh document loaded from
@@ -109,18 +128,24 @@ def pull_session(session_id=None, url='default', io_loop=None, arguments=None):
             If ``session_id`` is not None, or a session with ``session_id``
             already exists, these arguments will have no effect.
 
+        max_message_size (int, optional) :
+            Configure the Tornado max websocket message size.
+            (default: 20 MB)
+
     Returns:
         ClientSession :
             A new ``ClientSession`` connected to the server
 
     '''
 
-    coords = _SessionCoordinates(session_id=session_id, url=url)
-    session = ClientSession(session_id=session_id, websocket_url=websocket_url_for_server_url(coords.url), io_loop=io_loop, arguments=arguments)
+    coords = SessionCoordinates(session_id=session_id, url=url)
+    session = ClientSession(
+        session_id=session_id, websocket_url=websocket_url_for_server_url(coords.url), io_loop=io_loop, arguments=arguments, max_message_size=max_message_size)
     session.pull()
     return session
 
-def push_session(document, session_id=None, url='default', io_loop=None):
+def push_session(document: Document, session_id: ID | None = None, url: str = "default",
+        io_loop: IOLoop | None = None, max_message_size: int = 20*1024*1024) -> ClientSession:
     ''' Create a session by pushing the given document to the server,
     overwriting any existing server-side document.
 
@@ -154,17 +179,27 @@ def push_session(document, session_id=None, url='default', io_loop=None):
         io_loop : (tornado.ioloop.IOLoop, optional)
             The IOLoop to use for the websocket
 
+        max_message_size (int, optional) :
+            Configure the Tornado max websocket message size.
+            (default: 20 MB)
+
     Returns:
         ClientSession
             A new ClientSession connected to the server
 
     '''
-    coords = _SessionCoordinates(session_id=session_id, url=url)
-    session = ClientSession(session_id=coords.session_id, websocket_url=websocket_url_for_server_url(coords.url), io_loop=io_loop)
+    coords = SessionCoordinates(session_id=session_id, url=url)
+    session = ClientSession(session_id=coords.session_id, websocket_url=websocket_url_for_server_url(coords.url), io_loop=io_loop, max_message_size=max_message_size)
     session.push(document)
     return session
 
-def show_session(session_id=None, url='default', session=None, browser=None, new="tab", controller=None):
+def show_session(
+            session_id: ID | None = None,
+            url: str = "default",
+            session: ClientSession | None = None,
+            browser: str | None = None,
+            new: BrowserTarget = "tab",
+            controller: BrowserLike | None = None) -> None:
         ''' Open a browser displaying a session document.
 
         If you have a session from ``pull_session()`` or ``push_session`` you
@@ -185,8 +220,8 @@ def show_session(session_id=None, url='default', session=None, browser=None, new
             browser (str, optional) : browser to show with (default: None)
                 For systems that support it, the **browser** argument allows
                 specifying which browser to display in, e.g. "safari", "firefox",
-                "opera", "windows-default" (see the ``webbrowser`` module
-                documentation in the standard lib for more details).
+                "opera", "windows-default" (see the :doc:`webbrowser <python:library/webbrowser>`
+                module documentation in the standard lib for more details).
 
             new (str, optional) : new file output mode (default: "tab")
                 For file-based output, opens or raises the browser window
@@ -198,7 +233,7 @@ def show_session(session_id=None, url='default', session=None, browser=None, new
             server_url = server_url_for_websocket_url(session._connection.url)
             session_id = session.id
         else:
-            coords = _SessionCoordinates(session_id=session_id, url=url)
+            coords = SessionCoordinates(session_id=session_id, url=url)
             server_url = coords.url
             session_id = coords.session_id
 
@@ -230,9 +265,17 @@ class ClientSession:
     If you do not use ``ClientSession`` in this way, it is up to you to ensure
     that ``mysession.close()`` is called.
 
+    .. autoclasstoc::
+
     '''
 
-    def __init__(self, session_id=None, websocket_url=DEFAULT_SERVER_WEBSOCKET_URL, io_loop=None, arguments=None):
+    _document: Document | None
+    _id: ID
+    _connection: ClientConnection
+    _callbacks: DocumentCallbackGroup
+
+    def __init__(self, session_id: ID | None = None, websocket_url: str = DEFAULT_SERVER_WEBSOCKET_URL,
+            io_loop: IOLoop | None = None, arguments: Dict[str, str] | None = None, max_message_size: int = 20*1024*1024):
         ''' A connection which attaches to a particular named session on the
         server.
 
@@ -262,23 +305,27 @@ class ClientSession:
                 If ``session_id`` is not None, or a session with ``session_id``
                 already exists, these arguments will have no effect.
 
+            max_message_size (int, optional) :
+                Configure the Tornado max websocket message size.
+                (default: 20 MB)
+
         '''
         self._document = None
         self._id = self._ensure_session_id(session_id)
 
         from .connection import ClientConnection
-        self._connection = ClientConnection(session=self, io_loop=io_loop, websocket_url=websocket_url, arguments=arguments)
+        self._connection = ClientConnection(session=self, io_loop=io_loop, websocket_url=websocket_url, arguments=arguments, max_message_size=max_message_size)
 
-        from ..server.callbacks import _DocumentCallbackGroup
-        self._callbacks = _DocumentCallbackGroup(self._connection.io_loop)
+        from ..server.callbacks import DocumentCallbackGroup
+        self._callbacks = DocumentCallbackGroup(self._connection.io_loop)
 
-    def __enter__(self):
+    def __enter__(self) -> ClientSession:
         '''
 
         '''
         return self
 
-    def __exit__(self, exc_type, exc_value, exc_traceback):
+    def __exit__(self, exc_type: Any, exc_value: Any, exc_traceback: Any) -> None:
         '''
 
         '''
@@ -287,30 +334,30 @@ class ClientSession:
     # Properties --------------------------------------------------------------
 
     @property
-    def connected(self):
+    def connected(self) -> bool:
         ''' Whether this session is currently connected. '''
         return self._connection.connected
 
     @property
-    def error_reason(self):
+    def error_reason(self) -> ErrorReason | None:
         return self._connection.error_reason
 
     @property
-    def error_code(self):
+    def error_code(self) -> int | None:
         return self._connection.error_code
 
     @property
-    def error_detail(self):
+    def error_detail(self) -> str:
         return self._connection.error_detail
 
     @property
-    def url(self):
+    def url(self) -> str:
         return self._connection.url
 
     @property
-    def document(self):
-        ''' A :class:`~bokeh.document.Document` that will be kept in sync with
-        the corresponding Document on the server.
+    def document(self) -> Document:
+        ''' A |Document| that will be kept in sync with the corresponding
+        ``Document`` on the server.
 
         This value is initialized when :func:`pull` or :func:`push` succeeds.
         It will be ``None`` until then.
@@ -319,26 +366,26 @@ class ClientSession:
         return self._document
 
     @property
-    def id(self):
+    def id(self) -> ID:
         ''' A unique ID for this session. '''
         return self._id
 
     @property
-    def token(self):
+    def token(self) -> str:
         ''' A JWT token to authenticate the session. '''
         return generate_jwt_token(self.id)
 
     # Public methods ----------------------------------------------------------
 
-    def connect(self):
+    def connect(self) -> None:
         ''' Connect to a Bokeh server at the configured URL. '''
         self._connection.connect()
 
-    def close(self, why="closed"):
+    def close(self, why: str = "closed") -> None:
         ''' Close the connection to the server. '''
         self._connection.close(why)
 
-    def force_roundtrip(self):
+    def force_roundtrip(self) -> None:
         ''' Force a round-trip request/reply to the server, sometimes needed to
         avoid race conditions. Mostly useful for testing.
 
@@ -351,7 +398,7 @@ class ClientSession:
         '''
         self._connection.force_roundtrip()
 
-    def check_connection_errors(self):
+    def check_connection_errors(self) -> None:
         ''' Raises an error, when the connection could not have been
         established.
 
@@ -365,10 +412,10 @@ class ClientSession:
             if self.error_reason is ErrorReason.HTTP_ERROR:
                 if self.error_code == 404:
                     raise OSError(f"Check your application path! The given Path is not valid: {self.url}")
-                raise OSError(f"We received an HTTP-Error. Disconnected with error code: {self.error_id}, given message: {self.error_message}")
+                raise OSError(f"We received an HTTP-Error. Disconnected with error code: {self.error_code}, given message: {self.error_detail}")
             raise OSError("We failed to connect to the server (to start the server, try the 'bokeh serve' command)")
 
-    def pull(self):
+    def pull(self) -> None:
         ''' Pull the server's state and set it as session.document.
 
         If this is called more than once, session.document will be the same
@@ -388,19 +435,19 @@ class ClientSession:
         if self.document is None:
             self._attach_document(doc)
 
-    def push(self, document=None):
-        ''' Push the given document to the server and record it as session.document.
+    def push(self, document: Document | None = None) -> None:
+        ''' Push the given document to the server and record it as ``session.document``.
 
         If this is called more than once, the Document has to be the same (or None
-        to mean "session.document").
+        to mean ``session.document``).
 
         .. note::
             Automatically calls :func:`~connect` before pushing.
 
         Args:
-            document (:class:`~bokeh.document.Document`, optional) :
-                The document which will be kept in sync with the server document.
-                None to use session.document or create a new document.
+            document (|Document|, optional) :
+                The document that will be kept in sync with the server document.
+                None to use ``session.document`` or create a new document.
 
         '''
         if self.document is None:
@@ -420,7 +467,7 @@ class ClientSession:
         if self._document is None:
             self._attach_document(doc)
 
-    def request_server_info(self):
+    def request_server_info(self) -> ServerInfo:
         ''' Ask for information about the server.
 
         Returns:
@@ -429,7 +476,8 @@ class ClientSession:
         '''
         return self._connection.request_server_info()
 
-    def show(self, obj=None, browser=None, new="tab"):
+    def show(self, obj: LayoutDOM | None = None, browser: str | None = None,
+            new: Literal["tab", "window"] = "tab") -> None:
         ''' Open a browser displaying this session.
 
         Args:
@@ -440,8 +488,8 @@ class ClientSession:
             browser (str, optional) : browser to show with (default: None)
                 For systems that support it, the **browser** argument allows
                 specifying which browser to display in, e.g. "safari", "firefox",
-                "opera", "windows-default" (see the ``webbrowser`` module
-                documentation in the standard lib for more details).
+                "opera", "windows-default" (see the :doc:`webbrowser <python:library/webbrowser>`
+                module documentation in the standard lib for more details).
 
             new (str, optional) : new file output mode (default: "tab")
                 For file-based output, opens or raises the browser window
@@ -455,13 +503,13 @@ class ClientSession:
 
     # Internal methods --------------------------------------------------------
 
-    def _attach_document(self, document):
+    def _attach_document(self, document: Document) -> None:
         self._document = document
-        self._document.on_change_dispatch_to(self)
+        self._document.callbacks.on_change_dispatch_to(self)
 
         self._callbacks.add_session_callbacks(self._document.session_callbacks)
 
-    def _document_patched(self, event):
+    def _document_patched(self, event: DocumentPatchedEvent) -> None:
         if event.setter is self:
             log.debug("Not sending notification back to server for a change it requested")
             return
@@ -473,15 +521,15 @@ class ClientSession:
         self._connection._send_patch_document(self._id, event)
 
     @classmethod
-    def _ensure_session_id(cls, session_id):
+    def _ensure_session_id(cls, session_id: ID | None) -> ID:
         if session_id is None:
             session_id = generate_session_id()
         return session_id
 
-    def _handle_patch(self, message):
+    def _handle_patch(self, message: patch_doc) -> None:
         message.apply_to_document(self.document, self)
 
-    def _loop_until_closed(self):
+    def _loop_until_closed(self) -> None:
         ''' Execute a blocking loop that runs and executes event callbacks
         until the connection is closed (e.g. by hitting Ctrl-C).
 
@@ -490,7 +538,7 @@ class ClientSession:
         '''
         self._connection.loop_until_closed()
 
-    def _notify_disconnected(self):
+    def _notify_disconnected(self) -> None:
         ''' Called by the ClientConnection we are using to notify us of disconnect.
 
         '''
@@ -498,10 +546,10 @@ class ClientSession:
             self.document.remove_on_change(self)
             self._callbacks.remove_all_callbacks()
 
-    def _session_callback_added(self, event):
+    def _session_callback_added(self, event: SessionCallbackAdded) -> None:
         self._callbacks.add_session_callback(event.callback)
 
-    def _session_callback_removed(self, event):
+    def _session_callback_removed(self, event: SessionCallbackRemoved) -> None:
         self._callbacks.remove_session_callback(event.callback)
 
 #-----------------------------------------------------------------------------
