@@ -11,6 +11,8 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -176,6 +178,8 @@ yellowgreen          = NamedColor("yellowgreen",           154,  205,  50 )
 
 __all__ = NamedColor.__all__
 
+colors = NamedColor.colors
+
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
