@@ -17,12 +17,14 @@ This will print general information to standard output, such as Python and Bokeh
 
 .. code-block:: none
 
-    Python version      :  3.6.5
-    IPython version     :  6.3.1
-    Bokeh version       :  0.12.15
-    BokehJS static path :  /opt/anaconda/lib/python3.6/site-packages/bokeh/server/static
-    node.js version     :  v8.11.1
-    npm version         :  5.8.0
+    Python version      :  3.9.7 | packaged by conda-forge | (default, Sep 29 2021, 19:20:46)
+    IPython version     :  7.20.0
+    Tornado version     :  6.1
+    Bokeh version       :  3.0.0
+    BokehJS static path :  /opt/anaconda/envs/test/lib/python3.9/site-packages/bokeh/server/static
+    node.js version     :  v16.12.0
+    npm version         :  7.24.2
+    Operating system    :  Linux-5.11.0-40-generic-x86_64-with-glibc2.31
 
 Sometimes it can be useful to get just paths to the BokehJS static files in order
 to configure other servers or processes. To do this, use the ``--static`` option
@@ -35,13 +37,15 @@ This will produce output like what is shown below
 
 .. code-block:: none
 
-    /opt/anaconda/lib/python3.6/site-packages/bokeh/server/static
+    /opt/anaconda/envs/test/lib/python3.9/site-packages/bokeh/server/static
 
 '''
 
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -50,9 +54,9 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
+import platform
 import sys
 from argparse import Namespace
-from typing import Any, Optional
 
 # Bokeh imports
 from bokeh import __version__
@@ -61,7 +65,7 @@ from bokeh.util.compiler import nodejs_version, npmjs_version
 from bokeh.util.dependencies import import_optional
 
 # Bokeh imports
-from ..subcommand import Subcommand
+from ..subcommand import Argument, Subcommand
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -75,7 +79,7 @@ __all__ = (
 # Private API
 #-----------------------------------------------------------------------------
 
-def if_installed(version_or_none: Optional[str]) -> str:
+def if_installed(version_or_none: str | None) -> str:
     ''' helper method to optionally return module version number or not installed
 
     :param version_or_none:
@@ -83,12 +87,9 @@ def if_installed(version_or_none: Optional[str]) -> str:
     '''
     return version_or_none or "(not installed)"
 
-def _version(modname: str, attr: str) -> Optional[Any]:
-    mod = import_optional(modname)
-    if mod:
-        return getattr(mod, attr)
-    else:  # explicit None return for mypy typing
-        return None
+def _version(module_name: str, attr: str) -> str | None:
+    module = import_optional(module_name)
+    return getattr(module, attr) if module else None
 
 #-----------------------------------------------------------------------------
 # General API
@@ -106,8 +107,8 @@ class Info(Subcommand):
 
     args = (
 
-        ('--static', dict(
-            action='store_true',
+        ('--static', Argument(
+            action="store_true",
             help="Print the locations of BokehJS static files",
         )),
 
@@ -120,14 +121,15 @@ class Info(Subcommand):
         if args.static:
             print(settings.bokehjsdir())
         else:
-
-            print("Python version      :  %s" % sys.version.split('\n')[0])
-            print("IPython version     :  %s" % if_installed(_version('IPython', '__version__')))
-            print("Tornado version     :  %s" % if_installed(_version('tornado', 'version')))
-            print("Bokeh version       :  %s" % __version__)
-            print("BokehJS static path :  %s" % settings.bokehjsdir())
-            print("node.js version     :  %s" % if_installed(nodejs_version()))
-            print("npm version         :  %s" % if_installed(npmjs_version()))
+            newline = '\n'
+            print(f"Python version      :  {sys.version.split(newline)[0]}")
+            print(f"IPython version     :  {if_installed(_version('IPython', '__version__'))}")
+            print(f"Tornado version     :  {if_installed(_version('tornado', 'version'))}")
+            print(f"Bokeh version       :  {__version__}")
+            print(f"BokehJS static path :  {settings.bokehjsdir()}")
+            print(f"node.js version     :  {if_installed(nodejs_version())}")
+            print(f"npm version         :  {if_installed(npmjs_version())}")
+            print(f"Operating system    :  {platform.platform()}")
 
 #-----------------------------------------------------------------------------
 # Dev API
