@@ -11,6 +11,10 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
+import logging # isort:skip
+log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
 # Imports
@@ -23,13 +27,13 @@ from argparse import Namespace
 from bokeh.ext import init
 
 # Bokeh imports
-from ..subcommand import Subcommand
+from ..subcommand import Argument, Subcommand
 
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
 
-__all__ = ['Init']
+__all__ = ("Init",)
 
 #-----------------------------------------------------------------------------
 # Private API
@@ -49,21 +53,21 @@ class Init(Subcommand):
     help = "Initialize a bokeh extension"
 
     args = (
-        ("base_dir", dict(
+        ("base_dir", Argument(
             metavar="BASE_DIR",
             type=str,
             nargs="?",
             default=".",
         )),
-        ("--interactive", dict(
+        ("--interactive", Argument(
             action="store_true",
             help="Walk the user through creating an extension",
         )),
-        ("--bokehjs_version", dict(
+        ("--bokehjs_version", Argument(
             action="store_true",
             help="Use a specific version of bokehjs",
         )),
-        ("--debug", dict(
+        ("--debug", Argument(
             action="store_true",
             help="Run nodejs in debug mode (use --inspect-brk)",
         )),
