@@ -63,6 +63,8 @@ Enumerations can be easily documented in Sphinx documentation with the
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -70,8 +72,20 @@ log = logging.getLogger(__name__)
 # Imports
 #-----------------------------------------------------------------------------
 
+# Standard library imports
+from typing import (
+    Any,
+    Dict,
+    Iterator,
+    List,
+)
+
+# External imports
+from typing_extensions import Literal, get_args
+
 # Bokeh imports
 from .. import colors, palettes
+from ..util.string import nice_join
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -113,6 +127,7 @@ __all__ = (
     'OutputBackend',
     'PaddingUnits',
     'Palette',
+    'Place',
     'RenderLevel',
     'RenderMode',
     'ResetPolicy',
@@ -153,26 +168,29 @@ class Enumeration:
     '''
     __slots__ = ()
 
-    def __iter__(self):
+    _values: List[str]
+    _default: str
+    _case_sensitive: bool
+    _quote: bool
+
+    def __iter__(self) -> Iterator[str]:
         return iter(self._values)
 
-    def __contains__(self, value):
+    def __contains__(self, value: str) -> bool:
         if not self._case_sensitive:
             value = value.lower()
         return value in self._values
 
-    def __str__(self):
-        if self._quote:
-            return "Enumeration(%s)" % ", ".join(repr(x) for x in self._values)
-        else:
-            return "Enumeration(%s)" % ", ".join(self._values)
+    def __str__(self) -> str:
+        fn = repr if self._quote else str
+        return f"Enumeration({', '.join(fn(x) for x in self._values)})"
 
-    def __len__(self):
+    def __len__(self) -> int:
         return len(self._values)
 
     __repr__ = __str__
 
-def enumeration(*values, **kwargs):
+def enumeration(*values: Any, case_sensitive: bool = True, quote: bool = False) -> Enumeration:
     ''' Create an |Enumeration| object from a sequence of values.
 
     Call ``enumeration`` with a sequence of (unique) strings to create an
@@ -205,18 +223,21 @@ def enumeration(*values, **kwargs):
         Enumeration
 
     '''
+    if len(values) == 1 and hasattr(values[0], "__args__"):
+        values = get_args(values[0])
+
     if not (values and all(isinstance(value, str) and value for value in values)):
-        raise ValueError("expected a non-empty sequence of strings, got %s" % values)
+        raise ValueError(f"expected a non-empty sequence of strings, got {nice_join(values)}")
 
     if len(values) != len(set(values)):
-        raise ValueError("enumeration items must be unique, got %s" % values)
+        raise ValueError(f"enumeration items must be unique, got {nice_join(values)}")
 
-    attrs = {value: value for value in values}
+    attrs: Dict[str, Any] = {value: value for value in values}
     attrs.update({
         "_values": list(values),
         "_default": values[0],
-        "_case_sensitive": kwargs.get("case_sensitive", True),
-        "_quote": kwargs.get("quote", False),
+        "_case_sensitive": case_sensitive,
+        "_quote": quote,
     })
 
     return type("Enumeration", (Enumeration,), attrs)()
@@ -316,13 +337,15 @@ HatchPattern = enumeration(*list(zip(*_hatch_patterns))[1])
 HatchPatternAbbreviation = enumeration(*list(zip(*_hatch_patterns))[0], quote=True)
 
 #: Specify whether events should be combined or collected as-is when a Document hold is in effect
-HoldPolicy = enumeration("combine", "collect")
+HoldPolicyType = Literal["combine", "collect"]
+HoldPolicy = enumeration(HoldPolicyType)
 
 #: Specify a horizontal location in plot layouts
 HorizontalLocation = enumeration("left", "right")
 
 #: Specify a distribution to use for the Jitter class
-JitterRandomDistribution = enumeration("uniform", "normal")
+JitterRandomDistributionType = Literal["uniform", "normal"]
+JitterRandomDistribution = enumeration(JitterRandomDistributionType)
 
 #: Specify whether a dimension or coordinate is latitude or longitude
 LatLon = enumeration("lat", "lon")
@@ -343,7 +366,8 @@ LineDash = enumeration("solid", "dashed", "dotted", "dotdash", "dashdot")
 LineJoin = enumeration("miter", "round", "bevel")
 
 #: Specify a location in plot layouts
-Location = enumeration("above", "below", "left", "right")
+LocationType = Literal["above", "below", "left", "right"]
+Location = enumeration(LocationType)
 
 #: Specify a style for a Google map
 MapType = enumeration("satellite", "roadmap", "terrain", "hybrid")
@@ -378,6 +402,10 @@ PaddingUnits = enumeration("percent", "absolute")
 #: Specify the name of a palette from :ref:`bokeh.palettes`
 Palette = enumeration(*palettes.__palettes__)
 
+#:
+PlaceType = Literal["above", "below", "left", "right", "center"]
+Place = enumeration(PlaceType)
+
 #: Specify a position in the render order for a Bokeh renderer
 RenderLevel = enumeration("image", "underlay", "glyph", "guide", "annotation", "overlay")
 
@@ -394,9 +422,8 @@ RoundingFunction = enumeration("round", "nearest", "floor", "rounddown", "ceil",
 SelectionMode = enumeration("replace", "append", "intersect", "subtract")
 
 #: Sizing mode policies
-SizingMode = enumeration("stretch_width", "stretch_height", "stretch_both",
-                         "scale_width", "scale_height", "scale_both",
-                         "fixed")
+SizingModeType = Literal["stretch_width", "stretch_height", "stretch_both", "scale_width", "scale_height", "scale_both", "fixed"]
+SizingMode = enumeration(SizingModeType)
 
 #: Individual sizing mode policies
 SizingPolicy = enumeration("fixed", "fit", "min", "max")
