@@ -18,6 +18,8 @@ serializable properties.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -27,17 +29,48 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 import difflib
-from typing import Any, Dict, Optional
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    List,
+    Mapping,
+    NoReturn,
+    Set,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+    overload,
+)
 from warnings import warn
 
+# External imports
+from typing_extensions import Literal, TypedDict
+
+if TYPE_CHECKING:
+    F = TypeVar("F", bound=Callable[..., Any])
+    def lru_cache(arg: int | None) -> Callable[[F], F]: ...
+else:
+    from functools import lru_cache
+
 # Bokeh imports
-from ..util.string import nice_join
-from .property.alias import Alias
+from ..util.string import append_docstring, nice_join
 from .property.descriptor_factory import PropertyDescriptorFactory
 from .property.descriptors import PropertyDescriptor, UnsetValueError
 from .property.override import Override
 from .property.singletons import Undefined
 from .property.wrappers import PropertyValueContainer
+from .types import ID, JSON, Unknown
+
+if TYPE_CHECKING:
+    from ..client.session import ClientSession
+    from ..document.document import StaticSerializer
+    from ..server.session import ServerSession
+    from .property.bases import Property
+    from .property.dataspec import DataSpec
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -45,8 +78,6 @@ from .property.wrappers import PropertyValueContainer
 
 __all__ = (
     'abstract',
-    'accumulate_dict_from_superclasses',
-    'accumulate_from_superclasses',
     'HasProps',
     'MetaHasProps',
 )
@@ -59,23 +90,41 @@ __all__ = (
 # Dev API
 #-----------------------------------------------------------------------------
 
-def abstract(cls):
+if TYPE_CHECKING:
+    Setter = Union[ClientSession, ServerSession]
+
+C = TypeVar("C", bound=Type["HasProps"])
+
+def abstract(cls: C) -> C:
     ''' A decorator to mark abstract base classes derived from |HasProps|.
 
     '''
     if not issubclass(cls, HasProps):
-        raise TypeError("%s is not a subclass of HasProps" % cls.__name__)
-
-    # running python with -OO will discard docstrings -> __doc__ is None
-    if cls.__doc__ is not None:
-        cls.__doc__ += _ABSTRACT_ADMONITION
-
+        raise TypeError(f"{cls.__name__} is not a subclass of HasProps")
+    cls.__doc__ = append_docstring(cls.__doc__, _ABSTRACT_ADMONITION)
     return cls
 
-def is_DataModel(cls):
+def is_DataModel(cls: Type[HasProps]) -> bool:
     from ..model import DataModel
     return issubclass(cls, HasProps) and getattr(cls, "__data_model__", False) and cls != DataModel
 
+def _overridden_defaults(class_dict: Dict[str, Any]) -> Dict[str, Unknown]:
+    overridden_defaults: Dict[str, Unknown] = {}
+    for name, prop in tuple(class_dict.items()):
+        if isinstance(prop, Override):
+            del class_dict[name]
+            if prop.default_overridden:
+                overridden_defaults[name] = prop.default
+    return overridden_defaults
+
+def _generators(class_dict: Dict[str, Any]):
+    generators: Dict[str, PropertyDescriptorFactory[Any]] = {}
+    for name, generator in tuple(class_dict.items()):
+        if isinstance(generator, PropertyDescriptorFactory):
+            del class_dict[name]
+            generators[name] = generator
+    return generators
+
 class MetaHasProps(type):
     ''' Specialize the construction of |HasProps| classes.
 
@@ -86,191 +135,79 @@ class MetaHasProps(type):
     .. _metaclass: https://docs.python.org/3/reference/datamodel.html#metaclasses
 
     '''
-    def __new__(meta_cls, class_name, bases, class_dict):
+
+    __properties__: Dict[str, Property[Any]]
+    __overridden_defaults__: Dict[str, Unknown]
+    __themed_values__: Dict[str, Unknown]
+
+    def __new__(cls, class_name: str, bases: Tuple[type, ...], class_dict: Dict[str, Any]):
         '''
 
         '''
-        names_with_refs = set()
-        container_names = set()
+        overridden_defaults = _overridden_defaults(class_dict)
+        generators = _generators(class_dict)
 
-        # Now handle all the Override
-        overridden_defaults = {}
-        aliased_properties = {}
-        for name, prop in class_dict.items():
-            if isinstance(prop, Alias):
-                aliased_properties[name] = prop
-            elif isinstance(prop, Override):
-                if prop.default_overridden:
-                    overridden_defaults[name] = prop.default
-
-        for name, default in overridden_defaults.items():
-            del class_dict[name]
-
-        def make_property(target_name, help):
-            fget = lambda self: getattr(self, target_name)
-            fset = lambda self, value: setattr(self, target_name, value)
-            return property(fget, fset, None, help)
-
-        property_aliases = {}
-        for name, alias in aliased_properties.items():
-            property_aliases[name] = alias.name
-            class_dict[name] = make_property(alias.name, alias.help)
-
-        generators = dict()
-        for name, generator in class_dict.items():
-            if isinstance(generator, PropertyDescriptorFactory):
-                generators[name] = generator
-            elif isinstance(generator, type) and issubclass(generator, PropertyDescriptorFactory):
-                # Support the user adding a property without using parens,
-                # i.e. using just the Property subclass instead of an
-                # instance of the subclass
-                generators[name] = generator.autocreate()
-
-        dataspecs = {}
-        new_class_attrs = {}
+        properties = {}
 
         for name, generator in generators.items():
-            prop_descriptors = generator.make_descriptors(name)
-            replaced_self = False
-            for prop_descriptor in prop_descriptors:
-                if prop_descriptor.name in generators:
-                    if generators[prop_descriptor.name] is generator:
-                        # a generator can replace itself, this is the
-                        # standard case like `foo = Int()`
-                        replaced_self = True
-                        prop_descriptor.add_prop_descriptor_to_class(class_name, new_class_attrs, names_with_refs, container_names, dataspecs)
-                    else:
-                        # if a generator tries to overwrite another
-                        # generator that's been explicitly provided,
-                        # use the prop that was manually provided
-                        # and ignore this one.
-                        pass
-                else:
-                    prop_descriptor.add_prop_descriptor_to_class(class_name, new_class_attrs, names_with_refs, container_names, dataspecs)
-            # if we won't overwrite ourselves anyway, delete the generator
-            if not replaced_self:
-                del class_dict[name]
+            descriptors = generator.make_descriptors(name)
+            for descriptor in descriptors:
+                name = descriptor.name
+                if name in class_dict:
+                    raise RuntimeError(f"Two property generators both created {class_name}.{name}")
+                class_dict[name] = descriptor
+                properties[name] = descriptor.property
 
-        class_dict.update(new_class_attrs)
+        class_dict["__properties__"] = properties
+        class_dict["__overridden_defaults__"] = overridden_defaults
 
-        class_dict["__properties__"] = list(new_class_attrs)
-        class_dict["__properties_with_refs__"] = names_with_refs
-        class_dict["__container_props__"] = container_names
-        class_dict["__property_aliases__"] = property_aliases
-        if len(overridden_defaults) > 0:
-            class_dict["__overridden_defaults__"] = overridden_defaults
-        if dataspecs:
-            class_dict["__dataspecs__"] = dataspecs
+        return super().__new__(cls, class_name, bases, class_dict)
 
-        if "__example__" in class_dict:
-            path = class_dict["__example__"]
+    def __init__(cls, class_name: str, bases: Tuple[type, ...], _) -> None:
+        # Check for improperly redeclared a Property attribute.
+        base_properties: Dict[str, Any] = {}
+        for base in (x for x in bases if issubclass(x, HasProps)):
+            base_properties.update(base.properties(_with_props=True))
+        own_properties = {k: v for k, v in cls.__dict__.items() if isinstance(v, PropertyDescriptor)}
+        redeclared = own_properties.keys() & base_properties.keys()
+        if redeclared:
+            warn(f"Properties {redeclared!r} in class {cls.__name__} were previously declared on a parent "
+                 "class. It never makes sense to do this. Redundant properties should be deleted here, or on"
+                 "the parent class. Override() can be used to change a default value of a base class property.",
+                 RuntimeWarning, stacklevel=2)
 
-            # running python with -OO will discard docstrings -> __doc__ is None
-            if "__doc__" in class_dict and class_dict["__doc__"] is not None:
-                class_dict["__doc__"] += _EXAMPLE_TEMPLATE % dict(path=path)
-
-        return super().__new__(meta_cls, class_name, bases, class_dict)
-
-    def __init__(cls, class_name, bases, nmspc):
-        if class_name == 'HasProps':
-            return
-        # Check for improperly overriding a Property attribute.
-        # Overriding makes no sense except through the Override
-        # class which can be used to tweak the default.
-        # Historically code also tried changing the Property's
-        # type or changing from Property to non-Property: these
-        # overrides are bad conceptually because the type of a
-        # read-write property is invariant.
-        cls_attrs = cls.__dict__.keys() # we do NOT want inherited attrs here
-        for attr in cls_attrs:
-            for base in bases:
-                if issubclass(base, HasProps) and attr in base.properties():
-                    warn(('Property "%s" in class %s was overridden by a class attribute ' + \
-                          '"%s" in class %s; it never makes sense to do this. ' + \
-                          'Either %s.%s or %s.%s should be removed, or %s.%s should not ' + \
-                          'be a Property, or use Override(), depending on the intended effect.') %
-                         (attr, base.__name__, attr, class_name,
-                          base.__name__, attr,
-                          class_name, attr,
-                          base.__name__, attr),
-                         RuntimeWarning, stacklevel=2)
-
-        if "__overridden_defaults__" in cls.__dict__:
-            our_props = cls.properties()
-            for key in cls.__dict__["__overridden_defaults__"].keys():
-                if key not in our_props:
-                    warn(('Override() of %s in class %s does not override anything.') % (key, class_name),
-                         RuntimeWarning, stacklevel=2)
-
-def accumulate_from_superclasses(cls, propname):
-    ''' Traverse the class hierarchy and accumulate the special sets of names
-    ``MetaHasProps`` stores on classes:
-
-    Args:
-        name (str) : name of the special attribute to collect.
-
-            Typically meaningful values are: ``__container_props__``,
-            ``__properties__``, ``__properties_with_refs__``
-
-    '''
-    cachename = "__cached_all" + propname
-    # we MUST use cls.__dict__ NOT hasattr(). hasattr() would also look at base
-    # classes, and the cache must be separate for each class
-    if cachename not in cls.__dict__:
-        s = set()
-        for c in cls.__mro__:
-            if issubclass(c, HasProps) and hasattr(c, propname):
-                base = getattr(c, propname)
-                s.update(base)
-        setattr(cls, cachename, s)
-    return cls.__dict__[cachename]
-
-def accumulate_dict_from_superclasses(cls, propname):
-    ''' Traverse the class hierarchy and accumulate the special dicts
-    ``MetaHasProps`` stores on classes:
-
-    Args:
-        name (str) : name of the special attribute to collect.
-
-            Typically meaningful values are: ``__dataspecs__``,
-            ``__overridden_defaults__``
-
-    '''
-    cachename = "__cached_all" + propname
-    # we MUST use cls.__dict__ NOT hasattr(). hasattr() would also look at base
-    # classes, and the cache must be separate for each class
-    if cachename not in cls.__dict__:
-        d = dict()
-        for c in cls.__mro__:
-            if issubclass(c, HasProps) and hasattr(c, propname):
-                base = getattr(c, propname)
-                for k,v in base.items():
-                    if k not in d:
-                        d[k] = v
-        setattr(cls, cachename, d)
-    return cls.__dict__[cachename]
+        # Check for no-op Overrides
+        unused_overrides = cls.__overridden_defaults__.keys() - cls.properties(_with_props=True).keys()
+        if unused_overrides:
+            warn(f"Overrides of {unused_overrides} in class {cls.__name__} does not override anything.", RuntimeWarning, stacklevel=2)
 
 class HasProps(metaclass=MetaHasProps):
     ''' Base class for all class types that have Bokeh properties.
 
+    .. autoclasstoc::
+
     '''
     _initialized: bool = False
 
-    def __init__(self, **properties):
+    _property_values: Dict[str, Unknown]
+    _unstable_default_values: Dict[str, Unknown]
+    _unstable_themed_values: Dict[str, Unknown]
+
+    def __init__(self, **properties: Any) -> None:
         '''
 
         '''
         super().__init__()
-        self._property_values = dict()
-        self._unstable_default_values = dict()
-        self._unstable_themed_values = dict()
+        self._property_values = {}
+        self._unstable_default_values = {}
+        self._unstable_themed_values = {}
 
         for name, value in properties.items():
             setattr(self, name, value)
 
         self._initialized = True
 
-    def __setattr__(self, name, value):
+    def __setattr__(self, name: str, value: Unknown) -> None:
         ''' Intercept attribute setting on HasProps in order to special case
         a few situations:
 
@@ -285,25 +222,54 @@ class HasProps(metaclass=MetaHasProps):
             None
 
         '''
-        # self.properties() below can be expensive so avoid it
-        # if we're just setting a private underscore field
         if name.startswith("_"):
-            super().__setattr__(name, value)
-            return
+            return super().__setattr__(name, value)
+
+        properties = self.properties(_with_props=True)
+        if name in properties:
+            return super().__setattr__(name, value)
 
-        props = sorted(self.properties())
         descriptor = getattr(self.__class__, name, None)
+        if isinstance(descriptor, property): # Python property
+            return super().__setattr__(name, value)
 
-        if name in props or (descriptor is not None and descriptor.fset is not None):
-            super().__setattr__(name, value)
-        else:
-            matches, text = difflib.get_close_matches(name.lower(), props), "similar"
+        self._raise_attribute_error_with_matches(name, properties)
 
-            if not matches:
-                matches, text = props, "possible"
+    def __getattr__(self, name: str) -> Unknown:
+        ''' Intercept attribute setting on HasProps in order to special case
+        a few situations:
 
-            raise AttributeError("unexpected attribute '%s' to %s, %s attributes are %s" %
-                (name, self.__class__.__name__, text, nice_join(matches)))
+        * short circuit all property machinery for ``_private`` attributes
+        * suggest similar attribute names on attribute errors
+
+        Args:
+            name (str) : the name of the attribute to set on this object
+            value (obj) : the value to set
+
+        Returns:
+            None
+
+        '''
+        if name.startswith("_"):
+            return super().__getattr__(name)
+
+        properties = self.properties(_with_props=True)
+        if name in properties:
+            return super().__getattr__(name)
+
+        descriptor = getattr(self.__class__, name, None)
+        if isinstance(descriptor, property): # Python property
+            return super().__getattr__(name)
+
+        self._raise_attribute_error_with_matches(name, properties)
+
+    def _raise_attribute_error_with_matches(self, name: str, properties: Iterable[str]) -> NoReturn:
+        matches, text = difflib.get_close_matches(name.lower(), properties), "similar"
+
+        if not matches:
+            matches, text = sorted(properties), "possible"
+
+        raise AttributeError(f"unexpected attribute {name!r} to {self.__class__.__name__}, {text} attributes are {nice_join(matches)}")
 
     def __str__(self) -> str:
         name = self.__class__.__name__
@@ -311,7 +277,15 @@ class HasProps(metaclass=MetaHasProps):
 
     __repr__ = __str__
 
-    def equals(self, other):
+    # Unfortunately we cannot implement __eq__. We rely on the default __hash__
+    # based on object identity, in order to put HasProps instances in sets.
+    # Implementing __eq__ as structural equality would necessitate a __hash__
+    # that returns the same value different HasProps instances that compare
+    # equal [1], and this would break many things.
+    #
+    # [1] https://docs.python.org/3/reference/datamodel.html#object.__hash__
+    #
+    def equals(self, other: HasProps) -> bool:
         ''' Structural equality of models.
 
         Args:
@@ -321,12 +295,6 @@ class HasProps(metaclass=MetaHasProps):
             True, if properties are structurally equal, otherwise False
 
         '''
-
-        # NOTE: don't try to use this to implement __eq__. Because then
-        # you will be tempted to implement __hash__, which would interfere
-        # with mutability of models. However, not implementing __hash__
-        # will make bokeh unusable in Python 3, where proper implementation
-        # of __hash__ is required when implementing __eq__.
         if not isinstance(other, self.__class__):
             return False
         else:
@@ -334,14 +302,17 @@ class HasProps(metaclass=MetaHasProps):
 
     # TODO: this assumes that HasProps/Model are defined as in bokehjs, which
     # isn't the case here. HasProps must be serializable through refs only.
+    __view_model__: str
+    __view_module__: str
+
     @classmethod
-    def static_to_serializable(cls, serializer):
+    def static_to_serializable(cls, serializer: StaticSerializer) -> ModelRef:
         # TODO: resolving already visited objects should be serializer's duty
         modelref = serializer.get_ref(cls)
         if modelref is not None:
             return modelref
 
-        bases = [ basecls for basecls in cls.__bases__ if is_DataModel(basecls) ]
+        bases: List[Type[HasProps]] = [ basecls for basecls in cls.__bases__ if is_DataModel(basecls) ]
         if len(bases) == 0:
             extends = None
         elif len(bases) == 1:
@@ -356,29 +327,30 @@ class HasProps(metaclass=MetaHasProps):
         if module == "__main__" or module.split(".")[0] == "bokeh":
             module = None
 
-        properties = []
-        overrides = []
+        properties: List[PropertyDef] = []
+        overrides: List[OverrideDef] = []
 
         # TODO: don't use unordered sets
         for prop_name in cls.__properties__:
             descriptor = cls.lookup(prop_name)
             kind = None # TODO: serialize kinds
             default = descriptor.property._default # TODO: private member
-            properties.append(dict(name=prop_name, kind=kind, default=default))
+            properties.append(PropertyDef(name=prop_name, kind=kind, default=default))
 
         for prop_name, default in getattr(cls, "__overridden_defaults__", {}).items():
-            overrides.append(dict(name=prop_name, default=default))
+            overrides.append(OverrideDef(name=prop_name, default=default))
 
-        modeldef = dict(name=name, module=module, extends=extends, properties=properties, overrides=overrides)
-        modelref = dict(name=name, module=module)
+        modeldef = ModelDef(name=name, module=module, extends=extends, properties=properties, overrides=overrides)
+        modelref = ModelRef(name=name, module=module)
 
         serializer.add_ref(cls, modelref, modeldef)
         return modelref
 
-    def to_serializable(self, serializer):
+    def to_serializable(self, serializer: Any) -> Any:
         pass # TODO: new serializer, hopefully in near future
 
-    def set_from_json(self, name, json, models=None, setter=None):
+    def set_from_json(self, name: str, json: JSON, *,
+            models: Dict[ID, HasProps] | None = None, setter: Setter | None = None) -> None:
         ''' Set a property value on this object from JSON.
 
         Args:
@@ -406,14 +378,14 @@ class HasProps(metaclass=MetaHasProps):
             None
 
         '''
-        if name in self.properties():
-            log.trace("Patching attribute %r of %r with %r", name, self, json)
+        if name in self.properties(_with_props=True):
+            log.trace(f"Patching attribute {name!r} of {self!r} with {json!r}") # type: ignore # TODO: log.trace()
             descriptor = self.lookup(name)
-            descriptor.set_from_json(self, json, models, setter)
+            descriptor.set_from_json(self, json, models=models, setter=setter)
         else:
             log.warning("JSON had attr %r on obj %r, which is a client-only or invalid attribute that shouldn't have been sent", name, self)
 
-    def update(self, **kwargs):
+    def update(self, **kwargs: Any) -> None:
         ''' Updates the object's properties from the given keyword arguments.
 
         Returns:
@@ -437,10 +409,11 @@ class HasProps(metaclass=MetaHasProps):
                 r.update(start=10, end=20)
 
         '''
-        for k,v in kwargs.items():
+        for k, v in kwargs.items():
             setattr(self, k, v)
 
-    def update_from_json(self, json_attributes, models=None, setter=None):
+    def update_from_json(self, json_attributes: Dict[str, JSON], *,
+            models: Mapping[ID, HasProps] | None = None, setter: Setter | None = None) -> None:
         ''' Updates the object's properties from a JSON attributes dictionary.
 
         Args:
@@ -467,10 +440,18 @@ class HasProps(metaclass=MetaHasProps):
 
         '''
         for k, v in json_attributes.items():
-            self.set_from_json(k, v, models, setter)
+            self.set_from_json(k, v, models=models, setter=setter)
+
+    @overload
+    @classmethod
+    def lookup(cls, name: str, *, raises: Literal[True] = True) -> PropertyDescriptor[Any]: ...
+
+    @overload
+    @classmethod
+    def lookup(cls, name: str, *, raises: Literal[False] = False) -> PropertyDescriptor[Any] | None: ...
 
     @classmethod
-    def lookup(cls, name: str, *, raises: bool = True) -> Optional[PropertyDescriptor]:
+    def lookup(cls, name: str, *, raises: bool = True) -> PropertyDescriptor[Any] | None:
         ''' Find the ``PropertyDescriptor`` for a Bokeh property on a class,
         given the property name.
 
@@ -482,17 +463,47 @@ class HasProps(metaclass=MetaHasProps):
             PropertyDescriptor : descriptor for property named ``name``
 
         '''
-        resolved_name = cls._property_aliases().get(name, name)
-        attr = getattr(cls, resolved_name, None)
-        if attr is not None:
+        attr = getattr(cls, name, None)
+        if attr is not None or (attr is None and not raises):
             return attr
-        elif not raises:
-            return None
-        else:
-            raise AttributeError(f"{cls.__name__}.{name} property descriptor does not exist")
+        raise AttributeError(f"{cls.__name__}.{name} property descriptor does not exist")
+
+    @overload
+    @classmethod
+    @lru_cache(None)
+    def properties(cls, *, _with_props: Literal[False] = False) -> Set[str]: ...
+
+    @overload
+    @classmethod
+    @lru_cache(None)
+    def properties(cls, *, _with_props: Literal[True] = True) -> Dict[str, Property[Any]]: ...
 
     @classmethod
-    def properties_with_refs(cls):
+    @lru_cache(None)
+    def properties(cls, *, _with_props: bool = False) -> Union[Set[str], Dict[str, Property[Any]]]:
+        ''' Collect the names of properties on this class.
+
+        .. warning::
+            In a future version of Bokeh, this method will return a dictionary
+            mapping property names to property objects. To future-proof this
+            current usage of this method, wrap the return value in ``list``.
+
+        Returns:
+            property names
+
+        '''
+        props: Dict[str, Property[Any]] = {}
+        for c in cls.__mro__:
+            props.update(getattr(c, "__properties__", {}))
+
+        if not _with_props:
+            return set(props)
+
+        return props
+
+    @classmethod
+    @lru_cache(None)
+    def properties_with_refs(cls) -> Dict[str, Property[Any]]:
         ''' Collect the names of all properties on this class that also have
         references.
 
@@ -503,44 +514,11 @@ class HasProps(metaclass=MetaHasProps):
             set[str] : names of properties that have references
 
         '''
-        return accumulate_from_superclasses(cls, "__properties_with_refs__")
+        return {k: v for k, v in cls.properties(_with_props=True).items() if v.has_ref}
 
     @classmethod
-    def properties_containers(cls):
-        ''' Collect the names of all container properties on this class.
-
-        This method *always* traverses the class hierarchy and includes
-        properties defined on any parent classes.
-
-        Returns:
-            set[str] : names of container properties
-
-        '''
-        return accumulate_from_superclasses(cls, "__container_props__")
-
-    @classmethod
-    def properties(cls, with_bases=True):
-        ''' Collect the names of properties on this class.
-
-        This method *optionally* traverses the class hierarchy and includes
-        properties defined on any parent classes.
-
-        Args:
-            with_bases (bool, optional) :
-                Whether to include properties defined on parent classes in
-                the results. (default: True)
-
-        Returns:
-           set[str] : property names
-
-        '''
-        if with_bases:
-            return accumulate_from_superclasses(cls, "__properties__")
-        else:
-            return set(cls.__properties__)
-
-    @classmethod
-    def dataspecs(cls):
+    @lru_cache(None)
+    def dataspecs(cls) -> Dict[str, DataSpec]:
         ''' Collect the names of all ``DataSpec`` properties on this class.
 
         This method *always* traverses the class hierarchy and includes
@@ -550,23 +528,10 @@ class HasProps(metaclass=MetaHasProps):
             set[str] : names of ``DataSpec`` properties
 
         '''
-        return set(cls.dataspecs_with_props().keys())
+        from .property.dataspec import DataSpec  # avoid circular import
+        return {k: v for k, v in cls.properties(_with_props=True).items() if isinstance(v, DataSpec)}
 
-    @classmethod
-    def dataspecs_with_props(cls):
-        ''' Collect a dict mapping the names of all ``DataSpec`` properties
-        on this class to the associated properties.
-
-        This method *always* traverses the class hierarchy and includes
-        properties defined on any parent classes.
-
-        Returns:
-            dict[str, DataSpec] : mapping of names and ``DataSpec`` properties
-
-        '''
-        return accumulate_dict_from_superclasses(cls, "__dataspecs__")
-
-    def properties_with_values(self, *, include_defaults: bool = True, include_undefined: bool = False) -> Dict[str, Any]:
+    def properties_with_values(self, *, include_defaults: bool = True, include_undefined: bool = False) -> Dict[str, Unknown]:
         ''' Collect a dict mapping property names to their values.
 
         This method *always* traverses the class hierarchy and includes
@@ -591,25 +556,20 @@ class HasProps(metaclass=MetaHasProps):
             include_defaults=include_defaults, include_undefined=include_undefined)
 
     @classmethod
-    def _overridden_defaults(cls):
+    def _overridden_defaults(cls) -> Dict[str, Unknown]:
         ''' Returns a dictionary of defaults that have been overridden.
 
         .. note::
             This is an implementation detail of ``Property``.
 
         '''
-        return accumulate_dict_from_superclasses(cls, "__overridden_defaults__")
+        defaults: Dict[str, Unknown] = {}
+        for c in reversed(cls.__mro__):
+            defaults.update(getattr(c, "__overridden_defaults__", {}))
+        return defaults
 
-    @classmethod
-    def _property_aliases(cls) -> Dict[str, str]:
-        ''' Returns a dictionary of aliased properties.
-
-        .. note::
-            This is an implementation detail of ``Property``.
-        '''
-        return accumulate_dict_from_superclasses(cls, "__property_aliases__")
-
-    def query_properties_with_values(self, query, *, include_defaults: bool = True, include_undefined: bool = False):
+    def query_properties_with_values(self, query: Callable[[PropertyDescriptor[Any]], bool], *,
+            include_defaults: bool = True, include_undefined: bool = False) -> Dict[str, Unknown]:
         ''' Query the properties values of |HasProps| instances with a
         predicate.
 
@@ -626,18 +586,20 @@ class HasProps(metaclass=MetaHasProps):
             dict : mapping of property names and values for matching properties
 
         '''
-        themed_keys = set()
-        result = dict()
+        themed_keys: Set[str] = set()
+        result: Dict[str, Unknown] = {}
+
         if include_defaults:
-            keys = self.properties()
+            keys = self.properties(_with_props=True)
         else:
             # TODO (bev) For now, include unstable default values. Things rely on Instances
             # always getting serialized, even defaults, and adding unstable defaults here
             # accomplishes that. Unmodified defaults for property value containers will be
             # weeded out below.
             keys = set(self._property_values.keys()) | set(self._unstable_default_values.keys())
-            if self.themed_values():
-                themed_keys = set(self.themed_values().keys())
+            themed_values = self.themed_values()
+            if themed_values is not None:
+                themed_keys = set(themed_values.keys())
                 keys |= themed_keys
 
         for key in keys:
@@ -661,7 +623,7 @@ class HasProps(metaclass=MetaHasProps):
 
         return result
 
-    def themed_values(self):
+    def themed_values(self) -> Dict[str, Unknown] | None:
         ''' Get any theme-provided overrides.
 
         Results are returned as a dict from property name to value, or
@@ -673,7 +635,7 @@ class HasProps(metaclass=MetaHasProps):
         '''
         return getattr(self, '__themed_values__', None)
 
-    def apply_theme(self, property_values):
+    def apply_theme(self, property_values: Dict[str, Unknown]) -> None:
         ''' Apply a set of theme values which will be used rather than
         defaults, but will not override application-set values.
 
@@ -694,13 +656,13 @@ class HasProps(metaclass=MetaHasProps):
         if old_dict is property_values:  # lgtm [py/comparison-using-is]
             return
 
-        removed = set()
+        removed: Set[str] = set()
         # we're doing a little song-and-dance to avoid storing __themed_values__ or
         # an empty dict, if there's no theme that applies to this HasProps instance.
         if old_dict is not None:
             removed.update(set(old_dict.keys()))
         added = set(property_values.keys())
-        old_values = dict()
+        old_values: Dict[str, Unknown] = {}
         for k in added.union(removed):
             old_values[k] = getattr(self, k)
 
@@ -718,18 +680,19 @@ class HasProps(metaclass=MetaHasProps):
         # Emit any change notifications that result
         for k, v in old_values.items():
             descriptor = self.lookup(k)
-            descriptor.trigger_if_changed(self, v)
+            if isinstance(descriptor, PropertyDescriptor):
+                descriptor.trigger_if_changed(self, v)
 
-    def unapply_theme(self):
+    def unapply_theme(self) -> None:
         ''' Remove any themed values and restore defaults.
 
         Returns:
             None
 
         '''
-        self.apply_theme(property_values=dict())
+        self.apply_theme(property_values={})
 
-    def _clone(self):
+    def _clone(self) -> HasProps:
         ''' Duplicate a HasProps object.
 
         Values that are containers are shallow-copied.
@@ -737,6 +700,26 @@ class HasProps(metaclass=MetaHasProps):
         '''
         return self.__class__(**self._property_values)
 
+KindRef = Any # TODO
+
+class PropertyDef(TypedDict):
+    name: str
+    kind: KindRef | None
+    default: Unknown | None
+
+class OverrideDef(TypedDict):
+    name: str
+    default: Unknown
+
+class ModelRef(TypedDict):
+    name: str
+    module: str | None
+
+class ModelDef(ModelRef):
+    extends: ModelRef | None
+    properties: List[PropertyDef] | None
+    overrides: List[OverrideDef] | None
+
 #-----------------------------------------------------------------------------
 # Private API
 #-----------------------------------------------------------------------------
@@ -748,17 +731,6 @@ _ABSTRACT_ADMONITION = '''
 
 '''
 
-# The "../../" is needed for bokeh-plot to construct the correct path to examples
-_EXAMPLE_TEMPLATE = '''
-
-    Example
-    -------
-
-    .. bokeh-plot:: ../../%(path)s
-        :source-position: below
-
-'''
-
 #-----------------------------------------------------------------------------
 # Code
 #-----------------------------------------------------------------------------
