@@ -17,6 +17,8 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -27,7 +29,18 @@ log = logging.getLogger(__name__)
 # Standard library imports
 import types
 from copy import copy
-from typing import Any, Type, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    ClassVar,
+    Dict,
+    List,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+)
 
 # External imports
 import numpy as np
@@ -36,9 +49,19 @@ import numpy as np
 from ...util.dependencies import import_optional
 from ...util.string import nice_join
 from ..has_props import HasProps
+from ._sphinx import property_link, register_type_link, type_link
 from .descriptor_factory import PropertyDescriptorFactory
-from .descriptors import BasicPropertyDescriptor
-from .singletons import Intrinsic, Undefined
+from .descriptors import PropertyDescriptor
+from .singletons import (
+    Intrinsic,
+    IntrinsicType,
+    Undefined,
+    UndefinedType,
+)
+
+if TYPE_CHECKING:
+    from ...document.events import DocumentPatchedEvent
+    from ..types import ID
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -62,10 +85,16 @@ __all__ = (
 # Dev API
 #-----------------------------------------------------------------------------
 
+T = TypeVar("T")
+
+TypeOrInst = Union[Type[T], T]
+
+Init = Union[T, UndefinedType, IntrinsicType]
+
 class DeserializationError(Exception):
     pass
 
-class Property(PropertyDescriptorFactory):
+class Property(PropertyDescriptorFactory[T]):
     """ Base class for Bokeh property instances, which can be added to Bokeh
     Models.
 
@@ -89,9 +118,15 @@ class Property(PropertyDescriptorFactory):
     """
 
     # This class attribute is controlled by external helper API for validation
-    _should_validate = True
+    _should_validate: ClassVar[bool] = True
 
-    def __init__(self, default=Intrinsic, help=None, serialized=None, readonly=False):
+    _readonly: bool
+
+    alternatives: List[Tuple[Property[Any], Callable[[Property[Any]], T]]]
+    assertions: List[Tuple[Callable[[HasProps, T], bool], str | Callable[[HasProps, str, T], None]]]
+
+    def __init__(self, default: Init[T] = Intrinsic, help: str | None = None,
+            serialized: bool | None = None, readonly: bool = False):
         default = default if default is not Intrinsic else Undefined
 
         if serialized is None:
@@ -107,50 +142,25 @@ class Property(PropertyDescriptorFactory):
         self.alternatives = []
         self.assertions = []
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.__class__.__name__
 
-    @classmethod
-    def _sphinx_prop_link(cls):
-        """ Generate a sphinx :class: link to this property.
-
-        """
-        # (double) escaped space at the end is to appease Sphinx
-        # https://www.sphinx-doc.org/en/master/usage/restructuredtext/basics.html#gotchas
-        return f":class:`~bokeh.core.properties.{cls.__name__}`\\ "
-
-    @staticmethod
-    def _sphinx_model_link(name):
-        """ Generate a sphinx :class: link to given named model.
-
-        """
-        # (double) escaped space at the end is to appease Sphinx
-        # https://www.sphinx-doc.org/en/master/usage/restructuredtext/basics.html#gotchas
-        return f":class:`~{name}`\\ "
-
-    def _sphinx_type(self):
-        """ Generate a Sphinx-style reference to this type for documentation
-        automation purposes.
-
-        """
-        return self._sphinx_prop_link()
-
-    def make_descriptors(self, base_name):
-        """ Return a list of ``BasicPropertyDescriptor`` instances to install
+    def make_descriptors(self, name: str) -> List[PropertyDescriptor[T]]:
+        """ Return a list of ``PropertyDescriptor`` instances to install
         on a class, in order to delegate attribute access to this property.
 
         Args:
             name (str) : the name of the property these descriptors are for
 
         Returns:
-            list[BasicPropertyDescriptor]
+            list[PropertyDescriptor]
 
         The descriptors returned are collected by the ``MetaHasProps``
         metaclass and added to ``HasProps`` subclasses during class creation.
         """
-        return [ BasicPropertyDescriptor(base_name, self) ]
+        return [ PropertyDescriptor(name, self) ]
 
-    def _may_have_unstable_default(self):
+    def _may_have_unstable_default(self) -> bool:
         """ False if we have a default that is immutable, and will be the
         same every time (some defaults are generated on demand by a function
         to be called).
@@ -159,7 +169,7 @@ class Property(PropertyDescriptorFactory):
         return isinstance(self._default, types.FunctionType)
 
     @classmethod
-    def _copy_default(cls, default):
+    def _copy_default(cls, default: Callable[[], T] | T) -> T:
         """ Return a copy of the default, or a new value if the default
         is specified by a function.
 
@@ -169,7 +179,7 @@ class Property(PropertyDescriptorFactory):
         else:
             return default()
 
-    def _raw_default(self):
+    def _raw_default(self) -> T:
         """ Return the untransformed default value.
 
         The raw_default() needs to be validated and transformed by
@@ -179,7 +189,7 @@ class Property(PropertyDescriptorFactory):
         """
         return self._copy_default(self._default)
 
-    def themed_default(self, cls, name, theme_overrides):
+    def themed_default(self, cls: Type[HasProps], name: str, theme_overrides: Dict[str, Any] | None) -> T:
         """ The default, transformed by prepare_value() and the theme overrides.
 
         """
@@ -194,7 +204,7 @@ class Property(PropertyDescriptorFactory):
         return self.prepare_value(cls, name, default)
 
     @property
-    def serialized(self):
+    def serialized(self) -> bool:
         """ Whether the property should be serialized when serializing an object.
 
         This would be False for a "virtual" or "convenience" property that duplicates
@@ -203,7 +213,7 @@ class Property(PropertyDescriptorFactory):
         return self._serialized
 
     @property
-    def readonly(self):
+    def readonly(self) -> bool:
         """ Whether this property is read-only.
 
         Read-only properties may only be modified by the client (i.e., by BokehJS
@@ -212,7 +222,7 @@ class Property(PropertyDescriptorFactory):
         """
         return self._readonly
 
-    def matches(self, new, old):
+    def matches(self, new: T, old: T) -> bool:
         """ Whether two parameters match values.
 
         If either ``new`` or ``old`` is a NumPy array or Pandas Series or Index,
@@ -252,7 +262,7 @@ class Property(PropertyDescriptorFactory):
         except ValueError:
             return False
 
-    def from_json(self, json, models=None):
+    def from_json(self, json: Any, *, models: Dict[ID, HasProps] | None = None) -> T:
         """ Convert from JSON-compatible values into a value for this property.
 
         JSON-compatible values are: list, dict, number, string, bool, None
@@ -260,13 +270,13 @@ class Property(PropertyDescriptorFactory):
         """
         return json
 
-    def serialize_value(self, value):
+    def serialize_value(self, value: T) -> Any:
         """ Change the value into a JSON serializable format.
 
         """
         return value
 
-    def transform(self, value):
+    def transform(self, value: Any) -> T:
         """ Change the value into the canonical format for this property.
 
         Args:
@@ -278,7 +288,7 @@ class Property(PropertyDescriptorFactory):
         """
         return value
 
-    def validate(self, value, detail=True):
+    def validate(self, value: Any, detail: bool = True) -> None:
         """ Determine whether we can set this property from this value.
 
         Validation happens before transform()
@@ -301,7 +311,7 @@ class Property(PropertyDescriptorFactory):
         """
         pass
 
-    def is_valid(self, value):
+    def is_valid(self, value: Any) -> bool:
         """ Whether the value passes validation
 
         Args:
@@ -319,22 +329,27 @@ class Property(PropertyDescriptorFactory):
         else:
             return True
 
-    def wrap(self, value):
+    def wrap(self, value: T) -> T:
         """ Some property types need to wrap their values in special containers, etc.
 
         """
         return value
 
-    def prepare_value(self, owner: Union[HasProps, Type[HasProps]], name: str, value: Any):
+    def _hinted_value(self, value: Any, hint: DocumentPatchedEvent | None) -> Any:
+        return value
+
+    def prepare_value(self, owner: HasProps | Type[HasProps], name: str, value: Any, *, hint: DocumentPatchedEvent | None = None) -> T:
         if value is Intrinsic:
             value = self._raw_default()
         if value is Undefined:
             return value
 
+
         error = None
         try:
             if validation_on():
-                self.validate(value)
+                hinted_value = self._hinted_value(value, hint)
+                self.validate(hinted_value)
         except ValueError as e:
             for tp, converter in self.alternatives:
                 if tp.is_valid(value):
@@ -369,10 +384,10 @@ class Property(PropertyDescriptorFactory):
         return self.wrap(value)
 
     @property
-    def has_ref(self):
+    def has_ref(self) -> bool:
         return False
 
-    def accepts(self, tp, converter):
+    def accepts(self, tp: TypeOrInst[Property[Any]], converter: Callable[[Property[Any]], T]) -> Property[T]:
         """ Declare that other types may be converted to this property type.
 
         Args:
@@ -393,7 +408,7 @@ class Property(PropertyDescriptorFactory):
         self.alternatives.append((tp, converter))
         return self
 
-    def asserts(self, fn, msg_or_fn):
+    def asserts(self, fn: Callable[[HasProps, T], bool], msg_or_fn: str | Callable[[HasProps, str, T], None]) -> Property[T]:
         """ Assert that prepared values satisfy given conditions.
 
         Assertions are intended in enforce conditions beyond simple value
@@ -418,20 +433,22 @@ class Property(PropertyDescriptorFactory):
         self.assertions.append((fn, msg_or_fn))
         return self
 
-class ParameterizedProperty(Property):
+TItem = TypeVar("TItem", bound=Property[Any])
+
+class ParameterizedProperty(Property[TItem]):
     """ A base class for Properties that have type parameters, e.g. ``List(String)``.
 
     """
 
     @staticmethod
-    def _validate_type_param(type_param):
+    def _validate_type_param(type_param: TypeOrInst[Property[Any]], *, help_allowed: bool = False) -> Property[Any]:
         if isinstance(type_param, type):
             if issubclass(type_param, Property):
                 return type_param()
             else:
                 type_param = type_param.__name__
         elif isinstance(type_param, Property):
-            if type_param._help is not None:
+            if type_param._help is not None and not help_allowed:
                 raise ValueError("setting 'help' on type parameters doesn't make sense")
 
             return type_param
@@ -439,48 +456,46 @@ class ParameterizedProperty(Property):
         raise ValueError(f"expected a Property as type parameter, got {type_param}")
 
     @property
-    def type_params(self):
+    def type_params(self) -> List[Property[Any]]:
         raise NotImplementedError("abstract method")
 
     @property
-    def has_ref(self):
+    def has_ref(self) -> bool:
         return any(type_param.has_ref for type_param in self.type_params)
 
-class SingleParameterizedProperty(ParameterizedProperty):
+class SingleParameterizedProperty(ParameterizedProperty[T]):
     """ A parameterized property with a single type parameter. """
 
-    def __init__(self, type_param, *, default=Intrinsic, help=None, serialized=None, readonly=False):
+    def __init__(self, type_param: TypeOrInst[Property[Any]], *, default: Init[T] = Intrinsic,
+            help: str | None = None, serialized: bool | None = None, readonly: bool = False):
         self.type_param = self._validate_type_param(type_param)
         default = default if default is not Intrinsic else self.type_param._raw_default()
         super().__init__(default=default, help=help, serialized=serialized, readonly=readonly)
 
     @property
-    def type_params(self):
+    def type_params(self) -> List[Property[Any]]:
         return [self.type_param]
 
-    def __str__(self):
+    def __str__(self) -> str:
         return f"{self.__class__.__name__}({self.type_param})"
 
-    def _sphinx_type(self):
-        return f"{self._sphinx_prop_link()}({self.type_param._sphinx_type()})"
-
     def validate(self, value: Any, detail: bool = True) -> None:
         super().validate(value, detail=detail)
         self.type_param.validate(value, detail=detail)
 
-    def from_json(self, json, models=None):
+    def from_json(self, json: Any, *, models: Dict[str, HasProps] | None = None) -> T:
         return self.type_param.from_json(json, models=models)
 
-    def transform(self, value):
+    def transform(self, value: T) -> T:
         return self.type_param.transform(value)
 
-    def wrap(self, value):
+    def wrap(self, value: T) -> T:
         return self.type_param.wrap(value)
 
-    def _may_have_unstable_default(self):
+    def _may_have_unstable_default(self) -> bool:
         return self.type_param._may_have_unstable_default()
 
-class PrimitiveProperty(Property):
+class PrimitiveProperty(Property[T]):
     """ A base class for simple property types.
 
     Subclasses should define a class attribute ``_underlying_type`` that is
@@ -497,9 +512,9 @@ class PrimitiveProperty(Property):
 
     """
 
-    _underlying_type = None
+    _underlying_type: ClassVar[Tuple[Type[T]]]
 
-    def validate(self, value, detail=True):
+    def validate(self, value: Any, detail: bool = True) -> None:
         super().validate(value, detail)
 
         if isinstance(value, self._underlying_type):
@@ -512,26 +527,23 @@ class PrimitiveProperty(Property):
         msg = f"expected a value of type {expected_type}, got {value} of type {type(value).__name__}"
         raise ValueError(msg)
 
-    def from_json(self, json, models=None):
+    def from_json(self, json: Any, *, models: Dict[str, HasProps] | None = None) -> T:
         if isinstance(json, self._underlying_type):
             return json
         expected_type = nice_join([ cls.__name__ for cls in self._underlying_type ])
         msg = f"{self} expected {expected_type}, got {json} of type {type(json).__name__}"
         raise DeserializationError(msg)
 
-    def _sphinx_type(self):
-        return self._sphinx_prop_link()
-
-class ContainerProperty(ParameterizedProperty):
+class ContainerProperty(ParameterizedProperty[T]):
     """ A base class for Container-like type properties.
 
     """
 
-    def _may_have_unstable_default(self):
+    def _may_have_unstable_default(self) -> bool:
         # all containers are mutable, so the default can be modified
         return True
 
-def validation_on():
+def validation_on() -> bool:
     """ Check if property validation is currently active
 
     Returns:
@@ -547,3 +559,7 @@ def validation_on():
 #-----------------------------------------------------------------------------
 # Code
 #-----------------------------------------------------------------------------
+
+@register_type_link(SingleParameterizedProperty)
+def _sphinx_type(obj):
+    return f"{property_link(obj)}({type_link(obj.type_param)})"
