@@ -1,16 +1,18 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
-''' Provide the DataSpec properties and helpers.
+""" Provide the DataSpec properties and helpers.
 
-'''
+"""
 
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -18,9 +20,17 @@ log = logging.getLogger(__name__)
 # Imports
 #-----------------------------------------------------------------------------
 
+# Standard library imports
+from typing import TYPE_CHECKING
+
+# External imports
+from typing_extensions import TypedDict
+
 # Bokeh imports
 from ... import colors
+from ...util.deprecation import deprecated
 from ...util.serialization import convert_datetime_type, convert_timedelta_type
+from ...util.string import nice_join
 from .. import enums
 from .color import Color
 from .container import Dict, List
@@ -29,28 +39,51 @@ from .descriptors import DataSpecPropertyDescriptor, UnitsSpecPropertyDescriptor
 from .either import Either
 from .enum import Enum
 from .instance import Instance
-from .primitive import Float, String
-from .visual import FontSize, HatchPatternType, MarkerType
+from .nullable import Nullable
+from .primitive import (
+    Float,
+    Int,
+    Null,
+    String,
+)
+from .singletons import Undefined
+from .visual import (
+    DashPattern,
+    FontSize,
+    HatchPatternType,
+    MarkerType,
+)
+
+if TYPE_CHECKING:
+    from ...core.types import Unknown
+    from ...models.expressions import Expression
+    from ...models.transforms import Transform
 
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
 
 __all__ = (
+    'AlphaSpec',
     'AngleSpec',
     'ColorSpec',
+    'DashPatternSpec',
     'DataSpec',
-    'DataDistanceSpec',
     'DistanceSpec',
     'expr',
     'field',
     'FontSizeSpec',
+    'FontStyleSpec',
     'HatchPatternSpec',
+    'IntSpec',
+    'LineCapSpec',
+    'LineJoinSpec',
     'MarkerSpec',
     'NumberSpec',
-    'ScreenDistanceSpec',
+    'SizeSpec',
     'StringSpec',
-    'UnitsSpec',
+    'TextAlignSpec',
+    'TextBaselineSpec',
     'value',
 )
 
@@ -67,7 +100,7 @@ _ExprFieldValueTransformUnits = Enum("expr", "field", "value", "transform", "uni
 #-----------------------------------------------------------------------------
 
 class DataSpec(Either):
-    ''' Base class for properties that accept either a fixed value, or a
+    """ Base class for properties that accept either a fixed value, or a
     string name that references a column in a
     :class:`~bokeh.models.sources.ColumnDataSource`.
 
@@ -149,8 +182,8 @@ class DataSpec(Either):
 
             color = ColorSpec(help="docs for color") # defaults to None
 
-    '''
-    def __init__(self, key_type, value_type, default, help=None):
+    """
+    def __init__(self, key_type, value_type, default, help=None) -> None:
         super().__init__(
             String,
             Dict(
@@ -165,11 +198,12 @@ class DataSpec(Either):
             help=help
         )
         self._type = self._validate_type_param(value_type)
+        self.accepts(Instance("bokeh.models.expressions.Expression"), lambda obj: expr(obj))
 
     # TODO (bev) add stricter validation on keys
 
     def make_descriptors(self, base_name):
-        ''' Return a list of ``DataSpecPropertyDescriptor`` instances to
+        """ Return a list of ``DataSpecPropertyDescriptor`` instances to
         install on a class, in order to delegate attribute access to this
         property.
 
@@ -181,15 +215,10 @@ class DataSpec(Either):
 
         The descriptors returned are collected by the ``MetaHasProps``
         metaclass and added to ``HasProps`` subclasses during class creation.
-        '''
+        """
         return [ DataSpecPropertyDescriptor(base_name, self) ]
 
     def to_serializable(self, obj, name, val):
-        # Check for None value; this means "the whole thing is
-        # unset," not "the value is None."
-        if val is None:
-            return None
-
         # Check for spec type value
         try:
             self._type.validate(val, False)
@@ -204,12 +233,12 @@ class DataSpec(Either):
         # Must be dict, return a new dict
         return dict(val)
 
-    def _sphinx_type(self):
-        return self._sphinx_prop_link()
-
+class IntSpec(DataSpec):
+    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
+        super().__init__(key_type, Int, default=default, help=help)
 
 class NumberSpec(DataSpec):
-    ''' A |DataSpec| property that accepts numeric and datetime fixed values.
+    """ A |DataSpec| property that accepts numeric and datetime fixed values.
 
     By default, date and datetime values are immediately converted to
     milliseconds since epoch. It is possible to disable processing of datetime
@@ -227,17 +256,30 @@ class NumberSpec(DataSpec):
 
         m.location = "foo" # field
 
-    '''
-    def __init__(self, default=None, help=None, key_type=_ExprFieldValueTransform, accept_datetime=True, accept_timedelta=True):
+    """
+    def __init__(self, default=Undefined, help=None, key_type=_ExprFieldValueTransform, accept_datetime=True, accept_timedelta=True) -> None:
         super().__init__(key_type, Float, default=default, help=help)
         if accept_timedelta:
             self.accepts(TimeDelta, convert_timedelta_type)
         if accept_datetime:
             self.accepts(Datetime, convert_datetime_type)
 
+class AlphaSpec(NumberSpec):
+
+    _default_help = """\
+    Acceptable values are numbers in 0..1 range (transparent..opaque).
+    """
+
+    def __init__(self, default=1.0, help=None) -> None:
+        help = f"{help or ''}\n{self._default_help}"
+        super().__init__(default=default, help=help, key_type=_ExprFieldValueTransform, accept_datetime=False, accept_timedelta=False)
+
+class NullStringSpec(DataSpec):
+    def __init__(self, default=None, help=None, key_type=_ExprFieldValueTransform) -> None:
+        super().__init__(key_type, Nullable(List(String)), default=default, help=help)
 
 class StringSpec(DataSpec):
-    ''' A |DataSpec| property that accepts string fixed values.
+    """ A |DataSpec| property that accepts string fixed values.
 
     Because acceptable fixed values and field names are both strings, it can
     be necessary explicitly to disambiguate these possibilities. By default,
@@ -250,8 +292,8 @@ class StringSpec(DataSpec):
 
         m.title = "foo"        # field
 
-    '''
-    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform):
+    """
+    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
         super().__init__(key_type, List(String), default=default, help=help)
 
     def prepare_value(self, cls, name, value):
@@ -262,7 +304,7 @@ class StringSpec(DataSpec):
         return super().prepare_value(cls, name, value)
 
 class FontSizeSpec(DataSpec):
-    ''' A |DataSpec| property that accepts font-size fixed values.
+    """ A |DataSpec| property that accepts font-size fixed values.
 
     The ``FontSizeSpec`` property attempts to first interpret string values as
     font sizes (i.e. valid CSS length values). Otherwise string values are
@@ -280,22 +322,47 @@ class FontSizeSpec(DataSpec):
 
     https://drafts.csswg.org/css-values/#lengths
 
-    '''
+    """
 
-    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform):
+    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
         super().__init__(key_type, FontSize, default=default, help=help)
 
     def validate(self, value, detail=True):
         # We want to preserve existing semantics and be a little more restrictive. This
         # validations makes m.font_size = "" or m.font_size = "6" an error
         super().validate(value, detail)
+
         if isinstance(value, str):
-            if len(value) == 0 or value[0].isdigit() and FontSize._font_size_re.match(value) is None:
-                msg = "" if not detail else "%r is not a valid font size value" % value
+            if len(value) == 0 or value[0].isdigit() and not FontSize._font_size_re.match(value):
+                msg = "" if not detail else f"{value!r} is not a valid font size value"
                 raise ValueError(msg)
 
+class FontStyleSpec(DataSpec):
+    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
+        super().__init__(key_type, Enum(enums.FontStyle), default=default, help=help)
+
+class TextAlignSpec(DataSpec):
+    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
+        super().__init__(key_type, Enum(enums.TextAlign), default=default, help=help)
+
+class TextBaselineSpec(DataSpec):
+    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
+        super().__init__(key_type, Enum(enums.TextBaseline), default=default, help=help)
+
+class LineJoinSpec(DataSpec):
+    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
+        super().__init__(key_type, Enum(enums.LineJoin), default=default, help=help)
+
+class LineCapSpec(DataSpec):
+    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
+        super().__init__(key_type, Enum(enums.LineCap), default=default, help=help)
+
+class DashPatternSpec(DataSpec):
+    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
+        super().__init__(key_type, DashPattern, default=default, help=help)
+
 class HatchPatternSpec(DataSpec):
-    ''' A |DataSpec| property that accepts hatch pattern types as fixed values.
+    """ A |DataSpec| property that accepts hatch pattern types as fixed values.
 
     The ``HatchPatternSpec`` property attempts to first interpret string values
     as hatch pattern types. Otherwise string values are interpreted as field
@@ -309,13 +376,13 @@ class HatchPatternSpec(DataSpec):
 
         m.font_size = "foo"  # field
 
-    '''
+    """
 
-    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform):
-        super().__init__(key_type, HatchPatternType, default=default, help=help)
+    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
+        super().__init__(key_type, Nullable(HatchPatternType), default=default, help=help)
 
 class MarkerSpec(DataSpec):
-    ''' A |DataSpec| property that accepts marker types as fixed values.
+    """ A |DataSpec| property that accepts marker types as fixed values.
 
     The ``MarkerSpec`` property attempts to first interpret string values as
     marker types. Otherwise string values are interpreted as field names.
@@ -329,37 +396,36 @@ class MarkerSpec(DataSpec):
 
         m.font_size = "foo"    # field
 
-    '''
+    """
 
-    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform):
+    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
         super().__init__(key_type, MarkerType, default=default, help=help)
 
-
 class UnitsSpec(NumberSpec):
-    ''' A |DataSpec| property that accepts numeric fixed values, and also
-    serializes associated units values.
+    """ A |DataSpec| property that accepts numeric fixed values, and also
+    provides an associated units property to store units information.
 
-    '''
-    def __init__(self, default, units_type, units_default, help=None):
+    """
+
+    def __init__(self, default, units_enum, units_default, help=None) -> None:
         super().__init__(default=default, help=help, key_type=_ExprFieldValueTransformUnits)
-        self._units_type = self._validate_type_param(units_type)
+        units_type = Enum(units_enum, default=units_default, serialized=False, help=f"""
+        Units to use for the associated property: {nice_join(units_enum)}
+        """)
+        self._units_type = self._validate_type_param(units_type, help_allowed=True)
 
-        # TODO (bev) units_type was already constructed, so this really should not be needed
-        self._units_type.validate(units_default)
-        self._units_type._default = units_default
-        self._units_type._serialized = False
-
-    def __str__(self):
-        return "%s(units_default=%r)" % (self.__class__.__name__, self._units_type._default)
+    def __str__(self) -> str:
+        units_default = self._units_type._default
+        return f"{self.__class__.__name__}(units_default={units_default!r})"
 
     def get_units(self, obj, name):
-        raise NotImplementedError()
+        return getattr(obj, name+"_units")
 
     def make_descriptors(self, base_name):
-        ''' Return a list of ``PropertyDescriptor`` instances to install on a
+        """ Return a list of ``PropertyDescriptor`` instances to install on a
         class, in order to delegate attribute access to this property.
 
-        Unlike simpler property types, ``UnitsSpec`` returns multiple
+        Unlike simpler property types, ``PropertyUnitsSpec`` returns multiple
         descriptors to install. In particular, descriptors for the base
         property as well as the associated units property are returned.
 
@@ -371,9 +437,10 @@ class UnitsSpec(NumberSpec):
 
         The descriptors returned are collected by the ``MetaHasProps``
         metaclass and added to ``HasProps`` subclasses during class creation.
-        '''
-        units_props = self._units_type.make_descriptors("unused")
-        return [ UnitsSpecPropertyDescriptor(base_name, self, units_props[0]) ]
+        """
+        units_name = base_name + "_units"
+        units_props = self._units_type.make_descriptors(units_name)
+        return units_props + [ UnitsSpecPropertyDescriptor(base_name, self, units_props[0]) ]
 
     def to_serializable(self, obj, name, val):
         d = super().to_serializable(obj, name, val)
@@ -381,58 +448,49 @@ class UnitsSpec(NumberSpec):
             # d is a PropertyValueDict at this point, we need to convert it to
             # a plain dict if we are going to modify its value, otherwise a
             # notify_change that should not happen will be triggered
-            d = dict(d)
-            d["units"] = self.get_units(obj, name)
+            units = self.get_units(obj, name)
+            if units != self._units_type._default:
+                d = dict(**d, units=units)
         return d
 
+# Deprecated
 class PropertyUnitsSpec(UnitsSpec):
-    ''' A |DataSpec| property that accepts numeric fixed values, and also
-    provides an associated units property to store units information.
-
-    '''
-    def get_units(self, obj, name):
-        return getattr(obj, name+"_units")
-
-    def make_descriptors(self, base_name):
-        ''' Return a list of ``PropertyDescriptor`` instances to install on a
-        class, in order to delegate attribute access to this property.
-
-        Unlike simpler property types, ``UnitsSpec`` returns multiple
-        descriptors to install. In particular, descriptors for the base
-        property as well as the associated units property are returned.
-
-        Args:
-            name (str) : the name of the property these descriptors are for
-
-        Returns:
-            list[PropertyDescriptor]
-
-        The descriptors returned are collected by the ``MetaHasProps``
-        metaclass and added to ``HasProps`` subclasses during class creation.
-        '''
-        units_name = base_name + "_units"
-        units_props = self._units_type.make_descriptors(units_name)
-        return units_props + [ UnitsSpecPropertyDescriptor(base_name, self, units_props[0]) ]
+    pass
 
 class AngleSpec(PropertyUnitsSpec):
-    ''' A |DataSpec| property that accepts numeric fixed values, and also
+    """ A |DataSpec| property that accepts numeric fixed values, and also
     provides an associated units property to store angle units.
 
-    Acceptable values for units are ``"rad"`` and ``"deg"``.
+    Acceptable values for units are ``"deg"``, ``"rad"``, ``"grad"`` and ``"turn"``.
 
-    '''
-    def __init__(self, default=None, units_default="rad", help=None):
-        super().__init__(default=default, units_type=Enum(enums.AngleUnits), units_default=units_default, help=help)
+    """
+    def __init__(self, default=Undefined, units_default="rad", help=None) -> None:
+        super().__init__(default=default, units_enum=enums.AngleUnits, units_default=units_default, help=help)
 
 class DistanceSpec(PropertyUnitsSpec):
-    ''' A |DataSpec| property that accepts numeric fixed values or strings
+    """ A |DataSpec| property that accepts numeric fixed values or strings
     that refer to columns in a :class:`~bokeh.models.sources.ColumnDataSource`,
     and also provides an associated units property to store units information.
     Acceptable values for units are ``"screen"`` and ``"data"``.
 
-    '''
-    def __init__(self, default=None, units_default="data", help=None):
-        super().__init__(default=default, units_type=Enum(enums.SpatialUnits), units_default=units_default, help=help)
+    """
+    def __init__(self, default=Undefined, units_default="data", help=None) -> None:
+        super().__init__(default=default, units_enum=enums.SpatialUnits, units_default=units_default, help=help)
+
+    def prepare_value(self, cls, name, value):
+        try:
+            if value < 0:
+                raise ValueError("Distances must be positive!")
+        except TypeError:
+            pass
+        return super().prepare_value(cls, name, value)
+
+class NullDistanceSpec(DistanceSpec):
+
+    def __init__(self, default=None, units_default="data", help=None) -> None:
+        super().__init__(default=default, units_default=units_default, help=help)
+        self._type = Nullable(self._type)
+        self._type_params = [Null()] + self._type_params
 
     def prepare_value(self, cls, name, value):
         try:
@@ -442,40 +500,22 @@ class DistanceSpec(PropertyUnitsSpec):
             pass
         return super().prepare_value(cls, name, value)
 
-class _FixedUnitsDistanceSpec(UnitsSpec):
-
-    def __init__(self, default=None, help=None):
-        super().__init__(default=default, units_type=Enum(enums.enumeration(self._units)), units_default=self._units, help=help)
-
-    def get_units(self, _obj, _name):
-        return self._units
+class SizeSpec(NumberSpec):
+    """ A |DataSpec| property that accepts non-negative numeric fixed values
+    for size values or strings that refer to columns in a
+    :class:`~bokeh.models.sources.ColumnDataSource`.
+    """
 
     def prepare_value(self, cls, name, value):
         try:
-            if value is not None and value < 0:
-                raise ValueError("Distances must be positive or None!")
+            if value < 0:
+                raise ValueError("Screen sizes must be positive")
         except TypeError:
             pass
         return super().prepare_value(cls, name, value)
 
-class ScreenDistanceSpec(_FixedUnitsDistanceSpec):
-    ''' A |DataSpec| property that accepts numeric fixed values for screen-space
-    distances, and also provides an associated units property that reports
-    ``"screen"`` as the units.
-
-    '''
-    _units = "screen"
-
-class DataDistanceSpec(_FixedUnitsDistanceSpec):
-    ''' A |DataSpec| property that accepts numeric fixed values for data-space
-    distances, and also provides an associated units property that reports
-    ``"data"`` as the units.
-
-    '''
-    _units = "data"
-
 class ColorSpec(DataSpec):
-    ''' A |DataSpec| property that accepts |Color| fixed values.
+    """ A |DataSpec| property that accepts |Color| fixed values.
 
     The ``ColorSpec`` property attempts to first interpret string values as
     colors. Otherwise, string values are interpreted as field names. For
@@ -498,13 +538,27 @@ class ColorSpec(DataSpec):
 
         m.color = field("firebrick")       # field (named "firebrick")
 
-    '''
-    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform):
-        super().__init__(key_type, Color, default=default, help=help)
+    """
+
+    _default_help = """\
+    Acceptable values are:
+
+    - any of the |named CSS colors|, e.g ``'green'``, ``'indigo'``
+    - RGB(A) hex strings, e.g., ``'#FF0000'``, ``'#44444444'``
+    - CSS4 color strings, e.g., ``'rgba(255, 0, 127, 0.6)'``, ``'rgb(0 127 0 / 1.0)'``
+    - a 3-tuple of integers (r, g, b) between 0 and 255
+    - a 4-tuple of (r, g, b, a) where r, g, b are integers between 0..255 and a is between 0..1
+    - a 32-bit unsiged integers using the 0xRRGGBBAA byte order pattern
+
+    """
+
+    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
+        help = f"{help or ''}\n{self._default_help}"
+        super().__init__(key_type, Nullable(Color), default=default, help=help)
 
     @classmethod
     def isconst(cls, val):
-        ''' Whether the value is a string color literal.
+        """ Whether the value is a string color literal.
 
         Checks for a well-formed hexadecimal color value or a named color.
 
@@ -514,10 +568,25 @@ class ColorSpec(DataSpec):
         Returns:
             True, if the value is a string color literal
 
-        '''
+        """
         return isinstance(val, str) and \
                ((len(val) == 7 and val[0] == "#") or val in enums.NamedColor)
 
+    @classmethod
+    def is_color_tuple_shape(cls, val):
+        """ Whether the value is the correct shape to be a color tuple
+
+        Checks for a 3 or 4-tuple of numbers
+
+        Args:
+            val (str) : the value to check
+
+        Returns:
+            True, if the value could be a color tuple
+
+        """
+        return isinstance(val, tuple) and len(val) in (3, 4) and all(isinstance(v, (float, int)) for v in val)
+
     def to_serializable(self, obj, name, val):
         if val is None:
             return dict(value=None)
@@ -526,7 +595,7 @@ class ColorSpec(DataSpec):
         if self.isconst(val):
             return dict(value=val)
 
-        # Check for RGB or RGBa tuple
+        # Check for RGB or RGBA tuple
         if isinstance(val, tuple):
             return dict(value=colors.RGB(*val).to_css())
 
@@ -553,15 +622,27 @@ class ColorSpec(DataSpec):
         # at this point, since Color is very strict about only accepting
         # tuples of (integer) bytes. This conditions tuple values to only
         # have integer RGB components
-        if isinstance(value, tuple) and len(value) in (3, 4) and all(isinstance(v, (float, int)) for v in value):
+        if self.is_color_tuple_shape(value):
             value = tuple(int(v) if i < 3 else v for i, v in enumerate(value))
         return super().prepare_value(cls, name, value)
 
 # DataSpec helpers ------------------------------------------------------------
 
-def expr(expression, transform=None):
-    ''' Convenience function to explicitly return an "expr" specification for
-    a Bokeh :class:`~bokeh.core.properties.DataSpec` property.
+class Expr(TypedDict, total=False):
+    expr: Expression
+    transform: Transform | None
+
+class Field(TypedDict, total=False):
+    field: str
+    transform: Transform | None
+
+class Value(TypedDict, total=False):
+    value: Unknown
+    transform: Transform | None
+
+def expr(expression: Expression, transform: Transform | None = None) -> Expr:
+    """ Convenience function to explicitly return an "expr" specification for
+    a Bokeh |DataSpec| property.
 
     Args:
         expression (Expression) : a computed expression for a
@@ -576,15 +657,14 @@ def expr(expression, transform=None):
         This function is included for completeness. String values for
         property specifications are by default interpreted as field names.
 
-    '''
+    """
     if transform:
-        return dict(expr=expression, transform=transform)
-    return dict(expr=expression)
+        return Expr(expr=expression, transform=transform)
+    return Expr(expr=expression)
 
-
-def field(name, transform=None):
-    ''' Convenience function to explicitly return a "field" specification for
-    a Bokeh :class:`~bokeh.core.properties.DataSpec` property.
+def field(name: str, transform: Transform | None = None) -> Field:
+    """ Convenience function to explicitly return a "field" specification for
+    a Bokeh |DataSpec| property.
 
     Args:
         name (str) : name of a data source field to reference for a
@@ -599,14 +679,14 @@ def field(name, transform=None):
         This function is included for completeness. String values for
         property specifications are by default interpreted as field names.
 
-    '''
+    """
     if transform:
-        return dict(field=name, transform=transform)
-    return dict(field=name)
+        return Field(field=name, transform=transform)
+    return Field(field=name)
 
-def value(val, transform=None):
-    ''' Convenience function to explicitly return a "value" specification for
-    a Bokeh :class:`~bokeh.core.properties.DataSpec` property.
+def value(val: Unknown, transform: Transform | None = None) -> Value:
+    """ Convenience function to explicitly return a "value" specification for
+    a Bokeh |DataSpec| property.
 
     Args:
         val (any) : a fixed value to specify for a ``DataSpec`` property.
@@ -630,15 +710,24 @@ def value(val, transform=None):
             p.text("x", "y", text="text_column",
                    text_font_size=value("16px"), source=source)
 
-    '''
+    """
     if transform:
-        return dict(value=val, transform=transform)
-    return dict(value=val)
+        return Value(value=val, transform=transform)
+    return Value(value=val)
 
 #-----------------------------------------------------------------------------
 # Dev API
 #-----------------------------------------------------------------------------
 
+def DataDistanceSpec(*args, **kw):
+    deprecated((2, 4, 0), "DataDistanceSpec()", "SizeSpec()")
+    return SizeSpec(*args, **kw)
+
+def ScreenDistanceSpec(*args, **kw):
+    deprecated((2, 4, 0), "DataDistanceSpec()", "SizeSpec()")
+    return SizeSpec(*args, **kw)
+
+
 #-----------------------------------------------------------------------------
 # Code
 #-----------------------------------------------------------------------------
