@@ -1,16 +1,18 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
-''' Provide the JSON property.
+""" Provide the JSON property.
 
-'''
+"""
 
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -34,16 +36,16 @@ __all__ = (
 #-----------------------------------------------------------------------------
 
 class JSON(String):
-    ''' Accept JSON string values.
+    """ Accept JSON string values.
 
     The value is transmitted and received by BokehJS as a *string*
     containing JSON content. i.e., you must use ``JSON.parse`` to unpack
     the value into a JavaScript hash.
 
     Args:
-        default (string or None, optional) :
+        default (string, optional) :
             A default value for attributes created from this property to
-            have (default: None)
+            have.
 
         help (str or None, optional) :
             A documentation string for this property. It will be automatically
@@ -58,17 +60,15 @@ class JSON(String):
             Whether attributes created from this property are read-only.
             (default: False)
 
-    '''
+    """
     def validate(self, value, detail=True):
         super().validate(value, detail)
 
-        if value is None: return
-
         try:
             import json
             json.loads(value)
         except ValueError:
-            msg = "" if not detail else "expected JSON text, got %r" % value
+            msg = "" if not detail else f"expected JSON text, got {value!r}"
             raise ValueError(msg)
 
 #-----------------------------------------------------------------------------
