@@ -39,6 +39,8 @@ function provided.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -47,7 +49,14 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Bokeh imports
-from .check import check_integrity, silence, silenced
+from . import errors  # NOQA
+from . import warnings  # NOQA
+from .check import (
+    check_integrity,
+    process_validation_issues,
+    silence,
+    silenced,
+)
 from .decorators import error, warning
 
 #-----------------------------------------------------------------------------
