@@ -24,6 +24,8 @@ validation checks.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -31,20 +33,45 @@ log = logging.getLogger(__name__)
 # Imports
 #-----------------------------------------------------------------------------
 
+# Bokeh imports
+from .issue import Warning
+
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
 
-codes = {
-    1000: ("MISSING_RENDERERS",               "Plot has no renderers"),
-    1002: ("EMPTY_LAYOUT",                    "Layout has no children"),
-    1004: ("BOTH_CHILD_AND_ROOT",             "Models should not be a document root if they are in a layout box"),
-    1005: ("FIXED_SIZING_MODE",               "'fixed' sizing mode requires width and height to be set"),
-    1006: ("FIXED_WIDTH_POLICY",              "'fixed' width policy requires width to be set"),
-    1007: ("FIXED_HEIGHT_POLICY",             "'fixed' height policy requires height to be set"),
-    1008: ("PALETTE_LENGTH_FACTORS_MISMATCH", "Palette length does not match number of factors"),
-    9999: ("EXT",                             "Custom extension reports warning"),
-}
+MISSING_RENDERERS = Warning(
+    1000,
+    "MISSING_RENDERERS",
+    "Plot has no renderers")
+EMPTY_LAYOUT = Warning(
+    1002,
+    "EMPTY_LAYOUT",
+    "Layout has no children")
+BOTH_CHILD_AND_ROOT = Warning(
+    1004,
+    "BOTH_CHILD_AND_ROOT",
+    "Models should not be a document root if they are in a layout box")
+FIXED_SIZING_MODE = Warning(
+    1005,
+    "FIXED_SIZING_MODE",
+    "'fixed' sizing mode requires width and height to be set")
+FIXED_WIDTH_POLICY = Warning(
+    1006,
+    "FIXED_WIDTH_POLICY",
+    "'fixed' width policy requires width to be set")
+FIXED_HEIGHT_POLICY = Warning(
+    1007,
+    "FIXED_HEIGHT_POLICY",
+    "'fixed' height policy requires height to be set")
+PALETTE_LENGTH_FACTORS_MISMATCH = Warning(
+    1008,
+    "PALETTE_LENGTH_FACTORS_MISMATCH",
+    "Palette length does not match number of factors")
+EXT = Warning(
+    9999,
+    "EXT",
+    "Custom extension reports warning")
 
 __all__ = ()
 
@@ -63,6 +90,3 @@ __all__ = ()
 #-----------------------------------------------------------------------------
 # Code
 #-----------------------------------------------------------------------------
-
-for code in codes:
-    exec("%s = %d" % (codes[code][0], code))
