@@ -10,11 +10,44 @@ These events are used internally to signal changes to Documents. For
 information about user-facing (e.g. UI or tool) events, see the reference
 for :ref:`bokeh.events`.
 
+These events are employed for incoming and outgoing websocket messages and
+internally for triggering callbacks. For example, the sequence of events that
+happens when a user calls a Document API or sets a property resulting in a
+"patch event" to the Document:
+
+.. code-block::
+
+    user invokes Document API
+        -> Document API triggers event objects
+        -> registered callbacks are executed
+        -> Sesssion callback generates JSON message from event object
+        -> Session sends JSON message over websocket
+
+But events may also be triggered from the client, and arrive as JSON messages
+over the transport layer, which is why the JSON handling and Document API must
+be separated. Consider the alternative sequence of events:
+
+.. code-block::
+
+    Session recieves JSON message over websocket
+        -> Document calls event.handle_json
+        -> handle_json invokes appropriate Document API
+        -> Document API triggers event objects
+        -> registered callbacks are executed
+        -> Session callback suppresses outgoing event
+
+As a final note, message "ping-pong" is avoided by recording a "setter" when
+events objects are created. If the session callback notes the event setter is
+itself, then no further action (e.g. sending an outgoing change event identical
+to the incoming event it just processed) is taken.
+
 '''
 
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -23,18 +56,51 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-from typing import Any, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    List,
+    Set,
+    Union,
+    cast,
+)
+
+## External imports
+if TYPE_CHECKING:
+    import pandas as pd
+else:
+    from ..util.dependencies import import_optional
+    pd = import_optional('pandas')
 
 # Bokeh imports
-from ..util.dependencies import import_optional
 from ..util.serialization import make_id
+from .json import (
+    ColumnDataChanged,
+    ColumnsPatched,
+    ColumnsStreamed,
+    DocumentPatched,
+    MessageSent,
+    ModelChanged,
+    RootAdded,
+    RootRemoved,
+    TitleChanged,
+)
+
+if TYPE_CHECKING:
+    from ..core.has_props import Setter
+    from ..core.types import Unknown
+    from ..model import Model
+    from ..models.sources import ColumnarDataSource, DataDict
+    from ..protocol.message import BufferRef
+    from ..server.callbacks import SessionCallback
+    from .document import Document
+    from .json import Patches, ReferenceJson
 
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
 
-pd = import_optional('pandas')
-
 __all__ = (
     'ColumnDataChangedEvent',
     'ColumnsStreamedEvent',
@@ -58,13 +124,41 @@ __all__ = (
 # Dev API
 #-----------------------------------------------------------------------------
 
+if TYPE_CHECKING:
+    Buffers = Union[List[BufferRef], None]
+
+    Invoker = Callable[..., Any] # TODO
+
+class DocumentChangedMixin:
+    def _document_changed(self, event: DocumentChangedEvent) -> None: ...
+class DocumentPatchedMixin:
+    def _document_patched(self, event: DocumentPatchedEvent) -> None: ...
+class DocumentMessageSentMixin:
+    def _document_message_sent(self, event: MessageSentEvent) -> None: ...
+class DocumentModelChangedMixin:
+    def _document_model_changed(self, event: ModelChangedEvent) -> None: ...
+class ColumnDataChangedMixin:
+    def _column_data_changed(self, event: ColumnDataChangedEvent) -> None: ...
+class ColumnsStreamedMixin:
+    def _columns_streamed(self, event: ColumnsStreamedEvent) -> None: ...
+class ColumnsPatchedMixin:
+    def _columns_patched(self, event: ColumnsPatchedEvent) -> None: ...
+class SessionCallbackAddedMixin:
+    def _session_callback_added(self, event: SessionCallbackAdded) -> None: ...
+class SessionCallbackRemovedMixin:
+    def _session_callback_removed(self, event: SessionCallbackRemoved) -> None: ...
+
 class DocumentChangedEvent:
     ''' Base class for all internal events representing a change to a
     Bokeh Document.
 
     '''
 
-    def __init__(self, document, setter=None, callback_invoker=None):
+    document: Document
+    setter: Setter | None
+    callback_invoker: Invoker | None
+
+    def __init__(self, document: Document, setter: Setter | None = None, callback_invoker: Invoker | None = None) -> None:
         '''
 
         Args:
@@ -92,20 +186,20 @@ class DocumentChangedEvent:
         self.setter = setter
         self.callback_invoker = callback_invoker
 
-    def combine(self, event):
+    def combine(self, event: DocumentChangedEvent) -> bool:
         '''
 
         '''
         return False
 
-    def dispatch(self, receiver):
+    def dispatch(self, receiver: Any) -> None:
         ''' Dispatch handling of this event to a receiver.
 
         This method will invoke ``receiver._document_changed`` if it exists.
 
         '''
         if hasattr(receiver, '_document_changed'):
-            receiver._document_changed(self)
+            cast(DocumentChangedMixin, receiver)._document_changed(self)
 
 class DocumentPatchedEvent(DocumentChangedEvent):
     ''' A Base class for events that represent updating Bokeh Models and
@@ -113,7 +207,12 @@ class DocumentPatchedEvent(DocumentChangedEvent):
 
     '''
 
-    def dispatch(self, receiver):
+    _handlers = {}
+
+    def __init_subclass__(cls):
+        cls._handlers[cls.kind] = cls._handle_json
+
+    def dispatch(self, receiver: Any) -> None:
         ''' Dispatch handling of this event to a receiver.
 
         This method will invoke ``receiver._document_patched`` if it exists.
@@ -121,9 +220,9 @@ class DocumentPatchedEvent(DocumentChangedEvent):
         '''
         super().dispatch(receiver)
         if hasattr(receiver, '_document_patched'):
-            receiver._document_patched(self)
+            cast(DocumentPatchedMixin, receiver)._document_patched(self)
 
-    def generate(self, references, buffers):
+    def generate(self, references: Set[Model], buffers: Buffers) -> DocumentPatched:
         ''' Create a JSON representation of this event suitable for sending
         to clients.
 
@@ -147,34 +246,59 @@ class DocumentPatchedEvent(DocumentChangedEvent):
         '''
         raise NotImplementedError()
 
-class MessageSentEvent(DocumentPatchedEvent):
-    """ """
 
-    def __init__(self, document, msg_type: str, msg_data: Union[Any, bytes], setter=None, callback_invoker=None):
+    @staticmethod
+    def handle_json(doc: Document, event_json: DocumentPatched, references: List[ReferenceJson], setter: Setter) -> None:
+        '''
+
+        '''
+        event_kind = event_json['kind']
+        handler = DocumentPatchedEvent._handlers.get(event_kind, None)
+        if handler is None:
+            raise RuntimeError(f"Unknown patch event {event_json!r}")
+        handler(doc, event_json, references, setter)
+
+class MessageSentEvent(DocumentPatchedEvent):
+    '''
+
+    '''
+
+    kind = "MessageSent"
+
+    def __init__(self, document: Document, msg_type: str, msg_data: Union[Any, bytes],
+            setter: Setter | None = None, callback_invoker: Invoker | None = None):
         super().__init__(document, setter, callback_invoker)
         self.msg_type = msg_type
         self.msg_data = msg_data
 
-    def dispatch(self, receiver):
+    def dispatch(self, receiver: Any) -> None:
         super().dispatch(receiver)
         if hasattr(receiver, "_document_message_sent"):
-            receiver._document_message_sent(self)
+            cast(DocumentMessageSentMixin, receiver)._document_message_sent(self)
 
-    def generate(self, references, buffers):
-        msg = {
-            "kind": "MessageSent",
-            "msg_type": self.msg_type,
-        }
+    def generate(self, references: Set[Model], buffers: Buffers) -> MessageSent:
+        msg = MessageSent(
+            kind=self.kind,
+            msg_type=self.msg_type,
+            msg_data=None,
+        )
 
         if not isinstance(self.msg_data, bytes):
             msg["msg_data"] = self.msg_data
         else:
+            assert buffers is not None
             buffer_id = make_id()
             buf = (dict(id=buffer_id), self.msg_data)
             buffers.append(buf)
 
         return msg
 
+    @staticmethod
+    def _handle_json(doc: Document, event_json: DocumentPatched, references: List[ReferenceJson], setter: Setter) -> None:
+        message_callbacks = doc.callbacks._message_callbacks.get(event_json["msg_type"], [])
+        for cb in message_callbacks:
+            cb(event_json["msg_data"])
+
 class ModelChangedEvent(DocumentPatchedEvent):
     ''' A concrete event representing updating an attribute and value of a
     specific Bokeh Model.
@@ -185,7 +309,11 @@ class ModelChangedEvent(DocumentPatchedEvent):
 
     '''
 
-    def __init__(self, document, model, attr, old, new, serializable_new, hint=None, setter=None, callback_invoker=None):
+    kind = "ModelChanged"
+
+    def __init__(self, document: Document, model: Model, attr: str, old: Unknown, new: Unknown,
+            serializable_new: Unknown | None, hint: DocumentPatchedEvent | None = None,
+            setter: Setter | None = None, callback_invoker: Invoker | None = None):
         '''
 
         Args:
@@ -238,18 +366,21 @@ class ModelChangedEvent(DocumentPatchedEvent):
         self.serializable_new = serializable_new
         self.hint = hint
 
-    def combine(self, event):
+    def combine(self, event: DocumentChangedEvent) -> bool:
         '''
 
         '''
-        if not isinstance(event, ModelChangedEvent): return False
+        if not isinstance(event, ModelChangedEvent):
+            return False
 
         # If these are not true something weird is going on, maybe updates from
         # Python bokeh.client, don't try to combine
-        if self.setter != event.setter: return False
-        if self.document != event.document: return False
+        if self.setter != event.setter:
+            return False
+        if self.document != event.document:
+            return False
 
-        if self.hint:
+        if self.hint is not None and event.hint is not None:
             return self.hint.combine(event.hint)
 
         if (self.model == event.model) and (self.attr == event.attr):
@@ -260,7 +391,7 @@ class ModelChangedEvent(DocumentPatchedEvent):
 
         return False
 
-    def dispatch(self, receiver):
+    def dispatch(self, receiver: Any) -> None:
         ''' Dispatch handling of this event to a receiver.
 
         This method will invoke ``receiver._document_model_changed`` if it exists.
@@ -268,9 +399,9 @@ class ModelChangedEvent(DocumentPatchedEvent):
         '''
         super().dispatch(receiver)
         if hasattr(receiver, '_document_model_changed'):
-            receiver._document_model_changed(self)
+            cast(DocumentModelChangedMixin, receiver)._document_model_changed(self)
 
-    def generate(self, references, buffers):
+    def generate(self, references: Set[Model], buffers: Buffers) -> ModelChanged:
         ''' Create a JSON representation of this event suitable for sending
         to clients.
 
@@ -297,17 +428,14 @@ class ModelChangedEvent(DocumentPatchedEvent):
 
         value = self.serializable_new
 
-        # the new value is an object that may have
-        # not-yet-in-the-remote-doc references, and may also
-        # itself not be in the remote doc yet.  the remote may
-        # already have some of the references, but
-        # unfortunately we don't have an easy way to know
-        # unless we were to check BEFORE the attr gets changed
-        # (we need the old _all_models before setting the
-        # property). So we have to send all the references the
-        # remote could need, even though it could be inefficient.
-        # If it turns out we need to fix this we could probably
-        # do it by adding some complexity.
+        # the new value is an object that may have not-yet-in-the-remote-doc
+        # references, and may also itself not be in the remote doc yet. The
+        # remote may already have some of the references, but unfortunately
+        # we don't have an easy way to know unless we were to check BEFORE the
+        # attr gets changed (we need the old _all_models before setting the
+        # property). So we have to send all the references the remote could
+        # need, even though it could be inefficient. If it turns out we need to
+        # fix this we could probably do it by adding some complexity.
         value_refs = set(collect_models(value))
 
         # we know we don't want a whole new copy of the obj we're patching
@@ -317,10 +445,26 @@ class ModelChangedEvent(DocumentPatchedEvent):
 
         references.update(value_refs)
 
-        return { 'kind'  : 'ModelChanged',
-                 'model' : self.model.ref,
-                 'attr'  : self.attr,
-                 'new'   : value }
+        return ModelChanged(
+            kind  = self.kind,
+            model = self.model.ref,
+            attr  = self.attr,
+            new   = value,
+            hint  = None,
+        )
+
+    @staticmethod
+    def _handle_json(doc: Document, event_json: DocumentPatched, references: List[ReferenceJson], setter: Setter) -> None:
+        patched_id = event_json['model']['id']
+        if patched_id not in doc.models:
+            if doc.models.seen(patched_id):
+                log.trace(f"Cannot apply patch to {patched_id} which is not in the document anymore. This is usually harmless")
+                return
+            raise RuntimeError(f"Cannot apply patch to {patched_id} which is not in the document")
+        patched_obj = doc.models[patched_id]
+        attr = event_json['attr']
+        value = event_json['new']
+        patched_obj.set_from_json(attr, value, models=references, setter=setter)
 
 class ColumnDataChangedEvent(DocumentPatchedEvent):
     ''' A concrete event representing efficiently replacing *all*
@@ -328,7 +472,10 @@ class ColumnDataChangedEvent(DocumentPatchedEvent):
 
     '''
 
-    def __init__(self, document, column_source, cols=None, setter=None, callback_invoker=None):
+    kind = "ColumnDataChanged"
+
+    def __init__(self, document: Document, column_source: ColumnarDataSource,
+            cols: List[str] | None = None, setter: Setter | None = None, callback_invoker: Invoker | None = None):
         '''
 
         Args:
@@ -359,7 +506,7 @@ class ColumnDataChangedEvent(DocumentPatchedEvent):
         self.column_source = column_source
         self.cols = cols
 
-    def dispatch(self, receiver):
+    def dispatch(self, receiver: Any) -> None:
         ''' Dispatch handling of this event to a receiver.
 
         This method will invoke ``receiver._column_data_changed`` if it exists.
@@ -367,9 +514,9 @@ class ColumnDataChangedEvent(DocumentPatchedEvent):
         '''
         super().dispatch(receiver)
         if hasattr(receiver, '_column_data_changed'):
-            receiver._column_data_changed(self)
+            cast(ColumnDataChangedMixin, receiver)._column_data_changed(self)
 
-    def generate(self, references, buffers):
+    def generate(self, references: Set[Model], buffers: Buffers) -> ColumnDataChanged:
         ''' Create a JSON representation of this event suitable for sending
         to clients.
 
@@ -403,10 +550,21 @@ class ColumnDataChangedEvent(DocumentPatchedEvent):
 
         data_dict = transform_column_source_data(self.column_source.data, buffers=buffers, cols=self.cols)
 
-        return { 'kind'          : 'ColumnDataChanged',
-                 'column_source' : self.column_source.ref,
-                 'new'           : data_dict,
-                 'cols'          : self.cols}
+        return ColumnDataChanged(
+            kind          = self.kind,
+            column_source = self.column_source.ref,
+            new           = data_dict,
+            cols          = self.cols,
+        )
+
+    @staticmethod
+    def _handle_json(doc: Document, event_json: DocumentPatched, references: List[ReferenceJson], setter: Setter) -> None:
+        source_id = event_json['column_source']['id']
+        if source_id not in doc.models:
+            raise RuntimeError(f"Cannot apply patch to {source_id} which is not in the document")
+        source = doc.models[source_id]
+        value = event_json['new']
+        source.set_from_json('data', value, models=references, setter=setter)
 
 class ColumnsStreamedEvent(DocumentPatchedEvent):
     ''' A concrete event representing efficiently streaming new data
@@ -414,7 +572,12 @@ class ColumnsStreamedEvent(DocumentPatchedEvent):
 
     '''
 
-    def __init__(self, document, column_source, data, rollover, setter=None, callback_invoker=None):
+    kind = "ColumnsStreamed"
+
+    data: DataDict
+
+    def __init__(self, document: Document, column_source: ColumnarDataSource, data: DataDict | pd.DataFrame,
+            rollover: int | None, setter: Setter | None = None, callback_invoker: Invoker | None = None):
         '''
 
         Args:
@@ -456,7 +619,7 @@ class ColumnsStreamedEvent(DocumentPatchedEvent):
         self.data = data
         self.rollover = rollover
 
-    def dispatch(self, receiver):
+    def dispatch(self, receiver: Any) -> None:
         ''' Dispatch handling of this event to a receiver.
 
         This method will invoke ``receiver._columns_streamed`` if it exists.
@@ -464,9 +627,9 @@ class ColumnsStreamedEvent(DocumentPatchedEvent):
         '''
         super().dispatch(receiver)
         if hasattr(receiver, '_columns_streamed'):
-            receiver._columns_streamed(self)
+            cast(ColumnsStreamedMixin, receiver)._columns_streamed(self)
 
-    def generate(self, references, buffers):
+    def generate(self, references: Set[Model], buffers: Buffers) -> ColumnsStreamed:
         ''' Create a JSON representation of this event suitable for sending
         to clients.
 
@@ -495,10 +658,22 @@ class ColumnsStreamedEvent(DocumentPatchedEvent):
                 modified in-place.
 
         '''
-        return { 'kind'          : 'ColumnsStreamed',
-                 'column_source' : self.column_source.ref,
-                 'data'          : self.data,
-                 'rollover'      : self.rollover }
+        return ColumnsStreamed(
+            kind          = self.kind,
+            column_source = self.column_source.ref,
+            data          = self.data,
+            rollover      = self.rollover,
+        )
+
+    @staticmethod
+    def _handle_json(doc: Document, event_json: DocumentPatched, references: List[ReferenceJson], setter: Setter) -> None:
+        source_id = event_json['column_source']['id']
+        if source_id not in doc.models:
+            raise RuntimeError(f"Cannot stream to {source_id} which is not in the document")
+        source = doc.models[source_id]
+        data = event_json['data']
+        rollover = event_json.get('rollover', None)
+        source._stream(data, rollover, setter)
 
 class ColumnsPatchedEvent(DocumentPatchedEvent):
     ''' A concrete event representing efficiently applying data patches
@@ -506,7 +681,10 @@ class ColumnsPatchedEvent(DocumentPatchedEvent):
 
     '''
 
-    def __init__(self, document, column_source, patches, setter=None, callback_invoker=None):
+    kind = "ColumnsPatched"
+
+    def __init__(self, document: Document, column_source: ColumnarDataSource, patches: Patches,
+            setter: Setter | None = None, callback_invoker: Invoker | None = None):
         '''
 
         Args:
@@ -535,7 +713,7 @@ class ColumnsPatchedEvent(DocumentPatchedEvent):
         self.column_source = column_source
         self.patches = patches
 
-    def dispatch(self, receiver):
+    def dispatch(self, receiver: Any) -> None:
         ''' Dispatch handling of this event to a receiver.
 
         This method will invoke ``receiver._columns_patched`` if it exists.
@@ -543,9 +721,9 @@ class ColumnsPatchedEvent(DocumentPatchedEvent):
         '''
         super().dispatch(receiver)
         if hasattr(receiver, '_columns_patched'):
-            receiver._columns_patched(self)
+            cast(ColumnsPatchedMixin, receiver)._columns_patched(self)
 
-    def generate(self, references, buffers):
+    def generate(self, references: Set[Model], buffers: Buffers) -> ColumnsPatched:
         ''' Create a JSON representation of this event suitable for sending
         to clients.
 
@@ -573,9 +751,20 @@ class ColumnsPatchedEvent(DocumentPatchedEvent):
                 modified in-place.
 
         '''
-        return { 'kind'          : 'ColumnsPatched',
-                 'column_source' : self.column_source.ref,
-                 'patches'       : self.patches }
+        return ColumnsPatched(
+            kind          = self.kind,
+            column_source = self.column_source.ref,
+            patches       = self.patches,
+        )
+
+    @staticmethod
+    def _handle_json(doc: Document, event_json: DocumentPatched, references: List[ReferenceJson], setter: Setter) -> None:
+        source_id = event_json['column_source']['id']
+        if source_id not in doc.models:
+            raise RuntimeError(f"Cannot apply patch to {source_id} which is not in the document")
+        source = doc.models[source_id]
+        patches = event_json['patches']
+        source.patch(patches, setter)
 
 class TitleChangedEvent(DocumentPatchedEvent):
     ''' A concrete event representing a change to the title of a Bokeh
@@ -583,7 +772,10 @@ class TitleChangedEvent(DocumentPatchedEvent):
 
     '''
 
-    def __init__(self, document, title, setter=None, callback_invoker=None):
+    kind = "TitleChanged"
+
+    def __init__(self, document: Document, title: str,
+            setter: Setter | None = None, callback_invoker: Invoker | None = None):
         '''
 
         Args:
@@ -610,22 +802,25 @@ class TitleChangedEvent(DocumentPatchedEvent):
         super().__init__(document, setter, callback_invoker)
         self.title = title
 
-    def combine(self, event):
+    def combine(self, event: DocumentChangedEvent) -> bool:
         '''
 
         '''
-        if not isinstance(event, TitleChangedEvent): return False
+        if not isinstance(event, TitleChangedEvent):
+            return False
 
         # If these are not true something weird is going on, maybe updates from
         # Python bokeh.client, don't try to combine
-        if self.setter != event.setter: return False
-        if self.document != event.document: return False
+        if self.setter != event.setter:
+            return False
+        if self.document != event.document:
+            return False
 
         self.title = event.title
         self.callback_invoker = event.callback_invoker
         return True
 
-    def generate(self, references, buffers):
+    def generate(self, references: Set[Model], buffers: Buffers) -> TitleChanged:
         ''' Create a JSON representation of this event suitable for sending
         to clients.
 
@@ -652,8 +847,14 @@ class TitleChangedEvent(DocumentPatchedEvent):
                 modified in-place.
 
         '''
-        return { 'kind'  : 'TitleChanged',
-                 'title' : self.title }
+        return TitleChanged(
+            kind  = self.kind,
+            title = self.title,
+        )
+
+    @staticmethod
+    def _handle_json(doc: Document, event_json: DocumentPatched, references: List[ReferenceJson], setter: Setter) -> None:
+        doc.set_title(event_json['title'], setter)
 
 class RootAddedEvent(DocumentPatchedEvent):
     ''' A concrete event representing a change to add a new Model to a
@@ -661,7 +862,9 @@ class RootAddedEvent(DocumentPatchedEvent):
 
     '''
 
-    def __init__(self, document, model, setter=None, callback_invoker=None):
+    kind = "RootAdded"
+
+    def __init__(self, document: Document, model: Model, setter: Setter | None = None, callback_invoker: Invoker | None = None) -> None:
         '''
 
         Args:
@@ -687,7 +890,7 @@ class RootAddedEvent(DocumentPatchedEvent):
         super().__init__(document, setter, callback_invoker)
         self.model = model
 
-    def generate(self, references, buffers):
+    def generate(self, references: Set[Model], buffers: Buffers) -> RootAdded:
         ''' Create a JSON representation of this event suitable for sending
         to clients.
 
@@ -715,8 +918,16 @@ class RootAddedEvent(DocumentPatchedEvent):
 
         '''
         references.update(self.model.references())
-        return { 'kind'  : 'RootAdded',
-                 'model' : self.model.ref }
+        return RootAdded(
+            kind  = self.kind,
+            model = self.model.ref,
+        )
+
+    @staticmethod
+    def _handle_json(doc: Document, event_json: DocumentPatched, references: List[ReferenceJson], setter: Setter) -> None:
+        root_id = event_json['model']['id']
+        root_obj = references[root_id]
+        doc.add_root(root_obj, setter)
 
 class RootRemovedEvent(DocumentPatchedEvent):
     ''' A concrete event representing a change to remove an existing Model
@@ -724,7 +935,9 @@ class RootRemovedEvent(DocumentPatchedEvent):
 
     '''
 
-    def __init__(self, document, model, setter=None, callback_invoker=None):
+    kind = "RootRemoved"
+
+    def __init__(self, document: Document, model: Model, setter: Setter | None = None, callback_invoker: Invoker | None = None) -> None:
         '''
 
         Args:
@@ -751,7 +964,7 @@ class RootRemovedEvent(DocumentPatchedEvent):
         super().__init__(document, setter, callback_invoker)
         self.model = model
 
-    def generate(self, references, buffers):
+    def generate(self, references: Set[Model], buffers: Buffers) -> RootRemoved:
         ''' Create a JSON representation of this event suitable for sending
         to clients.
 
@@ -778,8 +991,16 @@ class RootRemovedEvent(DocumentPatchedEvent):
                 modified in-place.
 
         '''
-        return { 'kind'  : 'RootRemoved',
-                 'model' : self.model.ref }
+        return RootRemoved(
+            kind  = self.kind,
+            model = self.model.ref,
+        )
+
+    @staticmethod
+    def _handle_json(doc: Document, event_json: DocumentPatched, references: List[ReferenceJson], setter: Setter) -> None:
+        root_id = event_json['model']['id']
+        root_obj = references[root_id]
+        doc.remove_root(root_obj, setter)
 
 class SessionCallbackAdded(DocumentChangedEvent):
     ''' A concrete event representing a change to add a new callback (e.g.
@@ -787,7 +1008,7 @@ class SessionCallbackAdded(DocumentChangedEvent):
 
     '''
 
-    def __init__(self, document, callback):
+    def __init__(self, document: Document, callback: SessionCallback) -> None:
         '''
 
         Args:
@@ -801,7 +1022,7 @@ class SessionCallbackAdded(DocumentChangedEvent):
         super().__init__(document)
         self.callback = callback
 
-    def dispatch(self, receiver):
+    def dispatch(self, receiver: Any) -> None:
         ''' Dispatch handling of this event to a receiver.
 
         This method will invoke ``receiver._session_callback_added`` if
@@ -810,7 +1031,7 @@ class SessionCallbackAdded(DocumentChangedEvent):
         '''
         super().dispatch(receiver)
         if hasattr(receiver, '_session_callback_added'):
-            receiver._session_callback_added(self)
+            cast(SessionCallbackAddedMixin, receiver)._session_callback_added(self)
 
 class SessionCallbackRemoved(DocumentChangedEvent):
     ''' A concrete event representing a change to remove an existing callback
@@ -819,7 +1040,7 @@ class SessionCallbackRemoved(DocumentChangedEvent):
 
     '''
 
-    def __init__(self, document, callback):
+    def __init__(self, document: Document, callback: SessionCallback) -> None:
         '''
 
         Args:
@@ -833,7 +1054,7 @@ class SessionCallbackRemoved(DocumentChangedEvent):
         super().__init__(document)
         self.callback = callback
 
-    def dispatch(self, receiver):
+    def dispatch(self, receiver: Any) -> None:
         ''' Dispatch handling of this event to a receiver.
 
         This method will invoke ``receiver._session_callback_removed`` if
@@ -842,7 +1063,9 @@ class SessionCallbackRemoved(DocumentChangedEvent):
         '''
         super().dispatch(receiver)
         if hasattr(receiver, '_session_callback_removed'):
-            receiver._session_callback_removed(self)
+            cast(SessionCallbackRemovedMixin, receiver)._session_callback_removed(self)
+
+DocumentChangeCallback = Callable[[DocumentChangedEvent], None]
 
 #-----------------------------------------------------------------------------
 # Private API
