@@ -12,6 +12,8 @@ documents.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -20,12 +22,24 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-from typing import Dict
+from typing import (
+    TYPE_CHECKING,
+    Dict,
+    Iterable,
+    List,
+    Mapping,
+    Type,
+)
 
 # Bokeh imports
 from ..core.has_props import HasProps
 from ..model import get_class
 
+if TYPE_CHECKING:
+    from ..core.has_props import Setter
+    from ..core.types import ID, ReferenceJson
+    from ..model import Model
+
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
@@ -44,7 +58,8 @@ __all__ = (
 # Dev API
 #-----------------------------------------------------------------------------
 
-def initialize_references_json(references_json, references, setter=None):
+def initialize_references_json(references_json: List[ReferenceJson],
+        references: Mapping[ID, Model], setter: Setter | None = None) -> None:
     ''' Given a JSON representation of the models in a graph, and new model
     objects, set the properties on the models from the JSON
 
@@ -87,7 +102,7 @@ def initialize_references_json(references_json, references, setter=None):
 
         instance.update_from_json(obj_attrs, models=references, setter=setter)
 
-def instantiate_references_json(references_json, existing_instances: Dict[str, HasProps]):
+def instantiate_references_json(references_json: List[ReferenceJson], existing_instances: Dict[ID, Model]) -> Dict[ID, Model]:
     ''' Given a JSON representation of all the models in a graph, return a
     dict of new model objects.
 
@@ -101,15 +116,15 @@ def instantiate_references_json(references_json, existing_instances: Dict[str, H
     '''
 
     # Create all instances, but without setting their props
-    references = {}
+    references: Dict[ID, Model] = {}
     for obj in references_json:
         obj_id = obj['id']
-        obj_type = obj.get('subtype', obj['type'])
+        obj_type = obj.get('subtype', None) or obj['type']
 
         if obj_id in existing_instances:
             references[obj_id] = existing_instances[obj_id]
         else:
-            cls = get_class(obj_type)
+            cls: Type[Model] = get_class(obj_type)
             instance = cls.__new__(cls, id=obj_id)
             if instance is None:
                 raise RuntimeError(f"Error loading model from JSON (type: {obj_type}, id: {obj_id})")
@@ -117,7 +132,7 @@ def instantiate_references_json(references_json, existing_instances: Dict[str, H
 
     return references
 
-def references_json(references):
+def references_json(references: Iterable[Model]) -> List[ReferenceJson]:
     ''' Given a list of all models in a graph, return JSON representing
     them and their properties.
 
@@ -130,7 +145,7 @@ def references_json(references):
 
     '''
 
-    references_json = []
+    references_json: List[ReferenceJson] = []
     for r in references:
         struct = r.struct
         struct['attributes'] = r._to_json_like(include_defaults=False)
