@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -36,6 +36,8 @@ Example:
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -45,7 +47,14 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 from functools import partial
-from typing import Any, Callable, Iterable, Iterator, Sequence
+from typing import (
+    Any,
+    Callable,
+    Iterable,
+    Iterator,
+    Sequence,
+    TypeVar,
+)
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -64,7 +73,7 @@ __all__ = (
 # General API
 #-----------------------------------------------------------------------------
 
-def bounce(sequence: Sequence[int]) -> "partial[Callable[[], None]]":
+def bounce(sequence: Sequence[int]) -> partial[Callable[[], None]]:
     ''' Return a driver function that can advance a "bounced" sequence
     of values.
 
@@ -87,7 +96,7 @@ def bounce(sequence: Sequence[int]) -> "partial[Callable[[], None]]":
             return sequence[N-mod-1]
     return partial(force, sequence=_advance(f))
 
-def cosine(w: float, A: float = 1, phi: float = 0, offset: float = 0) -> "partial[Callable[[], None]]":
+def cosine(w: float, A: float = 1, phi: float = 0, offset: float = 0) -> partial[Callable[[], None]]:
     ''' Return a driver function that can advance a sequence of cosine values.
 
     .. code-block:: none
@@ -106,7 +115,7 @@ def cosine(w: float, A: float = 1, phi: float = 0, offset: float = 0) -> "partia
         return A * cos(w*i + phi) + offset
     return partial(force, sequence=_advance(f))
 
-def count() -> "partial[Callable[[], None]]":
+def count() -> partial[Callable[[], None]]:
     ''' Return a driver function that can advance a simple count.
 
     '''
@@ -128,7 +137,7 @@ def force(f: Callable[[Any], None], sequence: Iterator[Any]) -> Callable[[], Non
         f(next(sequence))
     return wrapper
 
-def linear(m: float = 1, b: float = 0) -> "partial[Callable[[], None]]":
+def linear(m: float = 1, b: float = 0) -> partial[Callable[[], None]]:
     ''' Return a driver function that can advance a sequence of linear values.
 
     .. code-block:: none
@@ -144,7 +153,7 @@ def linear(m: float = 1, b: float = 0) -> "partial[Callable[[], None]]":
         return m * i + b
     return partial(force, sequence=_advance(f))
 
-def repeat(sequence: Sequence[int]) -> "partial[Callable[[], None]]":
+def repeat(sequence: Sequence[int]) -> partial[Callable[[], None]]:
     ''' Return a driver function that can advance a repeated of values.
 
     .. code-block:: none
@@ -162,7 +171,7 @@ def repeat(sequence: Sequence[int]) -> "partial[Callable[[], None]]":
         return sequence[i%N]
     return partial(force, sequence=_advance(f))
 
-def sine(w: float, A: float = 1, phi: float = 0, offset: float = 0) -> "partial[Callable[[], None]]":
+def sine(w: float, A: float = 1, phi: float = 0, offset: float = 0) -> partial[Callable[[], None]]:
     ''' Return a driver function that can advance a sequence of sine values.
 
     .. code-block:: none
@@ -189,7 +198,9 @@ def sine(w: float, A: float = 1, phi: float = 0, offset: float = 0) -> "partial[
 # Private API
 #-----------------------------------------------------------------------------
 
-def _advance(f: Callable[[int], Any]) -> Iterable[Any]:
+T = TypeVar("T")
+
+def _advance(f: Callable[[int], T]) -> Iterable[T]:
     ''' Yield a sequence generated by calling a given function with
     successively incremented integer values.
 
