@@ -12,6 +12,8 @@ web pages.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -21,7 +23,12 @@ log = logging.getLogger(__name__)
 
 # Bokeh imports
 from .server import server_document, server_session
-from .standalone import autoload_static, components, file_html, json_item
+from .standalone import (
+    autoload_static,
+    components,
+    file_html,
+    json_item,
+)
 
 #-----------------------------------------------------------------------------
 # Globals and constants
