@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,6 +11,8 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -19,11 +21,33 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
+import hashlib
 import json
-from os.path import abspath, basename, dirname, exists, join, normpath
-from typing import Dict, List, NamedTuple, Optional
+from dataclasses import dataclass
+from os.path import (
+    abspath,
+    basename,
+    dirname,
+    exists,
+    join,
+    normpath,
+)
+from typing import (
+    TYPE_CHECKING,
+    Callable,
+    Dict,
+    Iterator,
+    List,
+    Sequence,
+    Set,
+    Tuple,
+    Type,
+)
 from warnings import warn
 
+# External imports
+from typing_extensions import TypedDict
+
 # Bokeh imports
 from ..core.templates import CSS_RESOURCES, JS_RESOURCES
 from ..document.document import Document
@@ -31,6 +55,10 @@ from ..model import Model
 from ..resources import BaseResources, Resources
 from ..settings import settings
 from ..util.compiler import bundle_models
+from .util import contains_tex_string, is_tex_string
+
+if TYPE_CHECKING:
+    from ..resources import Hashes
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -49,66 +77,72 @@ __all__ = (
 # Dev API
 #-----------------------------------------------------------------------------
 
+class Artifact:
+    pass
 
-class ScriptRef:
-    def __init__(self, url, type="text/javascript"):
+class ScriptRef(Artifact):
+    def __init__(self, url: str, type: str = "text/javascript") -> None:
         self.url = url
         self.type = type
 
 
-class Script:
-    def __init__(self, content, type="text/javascript"):
+class Script(Artifact):
+    def __init__(self, content: str, type: str = "text/javascript") -> None:
         self.content = content
         self.type = type
 
 
-class StyleRef:
-    def __init__(self, url):
+class StyleRef(Artifact):
+    def __init__(self, url: str) -> None:
         self.url = url
 
 
-class Style:
-    def __init__(self, content):
+class Style(Artifact):
+    def __init__(self, content: str) -> None:
         self.content = content
 
 
 class Bundle:
-    @classmethod
-    def of(cls, js_files, js_raw, css_files, css_raw, hashes):
-        return cls(js_files=js_files, js_raw=js_raw, css_files=css_files, css_raw=css_raw, hashes=hashes)
 
-    def __init__(self, **kwargs):
-        self.js_files = kwargs.get("js_files", [])
-        self.js_raw = kwargs.get("js_raw", [])
-        self.css_files = kwargs.get("css_files", [])
-        self.css_raw = kwargs.get("css_raw", [])
-        self.hashes = kwargs.get("hashes", {})
+    js_files: List[str]
+    js_raw: List[str]
+    css_files: List[str]
+    css_raw: List[str]
+    hashes: Hashes
 
-    def __iter__(self):
+    def __init__(self, js_files: List[str] = [], js_raw: List[str] = [],
+            css_files: List[str] = [], css_raw: List[str] = [], hashes: Hashes = {}):
+        self.js_files = js_files[:]
+        self.js_raw = js_raw[:]
+        self.css_files = css_files[:]
+        self.css_raw = css_raw[:]
+        self.hashes = {**hashes}
+
+    def __iter__(self) -> Iterator[str]:
         yield self._render_js()
         yield self._render_css()
 
-    def _render_js(self):
+    def _render_js(self) -> str:
         return JS_RESOURCES.render(js_files=self.js_files, js_raw=self.js_raw, hashes=self.hashes)
 
-    def _render_css(self):
+    def _render_css(self) -> str:
         return CSS_RESOURCES.render(css_files=self.css_files, css_raw=self.css_raw)
 
-    def scripts(self, tag=True):
+    def scripts(self, tag: bool = True) -> str:
         if tag:
             return JS_RESOURCES.render(js_raw=self.js_raw, js_files=[])
         else:
             return "\n".join(self.js_raw)
 
     @property
-    def js_urls(self):
+    def js_urls(self) -> List[str]:
         return self.js_files
 
     @property
-    def css_urls(self):
+    def css_urls(self) -> List[str]:
         return self.css_files
 
-    def add(self, artifact):
+    def add(self, artifact: Artifact) -> None:
         if isinstance(artifact, ScriptRef):
             self.js_files.append(artifact.url)
         elif isinstance(artifact, Script):
@@ -118,7 +152,8 @@ class Bundle:
         elif isinstance(artifact, Style):
             self.css_raw.append(artifact.content)
 
-def bundle_for_objs_and_resources(objs, resources):
+def bundle_for_objs_and_resources(objs: Sequence[Model | Document] | None,
+        resources: BaseResources | Tuple[BaseResources | None, BaseResources | None] | None) -> Bundle:
     ''' Generate rendered CSS and JS resources suitable for the given
     collection of Bokeh objects
 
@@ -147,18 +182,20 @@ def bundle_for_objs_and_resources(objs, resources):
         if css_resources and not js_resources:
             warn('No Bokeh JS Resources provided to template. If required you will need to provide them manually.')
     else:
-        raise ValueError("expected Resources or a pair of optional Resources, got %r" % resources)
+        raise ValueError(f"expected Resources or a pair of optional Resources, got {resources!r}")
 
     from copy import deepcopy
 
     # XXX: force all components on server and in notebook, because we don't know in advance what will be used
     use_widgets = _use_widgets(objs) if objs else True
     use_tables  = _use_tables(objs)  if objs else True
+    use_gl      = _use_gl(objs)      if objs else True
+    use_mathjax = _use_mathjax(objs) if objs else True
 
-    js_files = []
-    js_raw = []
-    css_files = []
-    css_raw = []
+    js_files: List[str] = []
+    js_raw: List[str] = []
+    css_files: List[str] = []
+    css_raw: List[str] = []
 
     if js_resources:
         js_resources = deepcopy(js_resources)
@@ -166,6 +203,10 @@ def bundle_for_objs_and_resources(objs, resources):
             js_resources.js_components.remove("bokeh-widgets")
         if not use_tables and "bokeh-tables" in js_resources.js_components:
             js_resources.js_components.remove("bokeh-tables")
+        if not use_gl and "bokeh-gl" in js_resources.js_components:
+            js_resources.js_components.remove("bokeh-gl")
+        if not use_mathjax and "bokeh-mathjax" in js_resources.js_components:
+            js_resources.js_components.remove("bokeh-mathjax")
 
         js_files.extend(js_resources.js_files)
         js_raw.extend(js_resources.js_raw)
@@ -196,14 +237,14 @@ def bundle_for_objs_and_resources(objs, resources):
     if ext is not None:
         js_raw.append(ext)
 
-    return Bundle.of(js_files, js_raw, css_files, css_raw, js_resources.hashes if js_resources else {})
+    return Bundle(js_files, js_raw, css_files, css_raw, js_resources.hashes if js_resources else {})
 
 #-----------------------------------------------------------------------------
 # Private API
 #-----------------------------------------------------------------------------
 
-def _query_extensions(objs, query):
-    names = set()
+def _query_extensions(objs: Sequence[Model | Document], query: Callable[[Type[Model]], bool]) -> bool:
+    names: Set[str] = set()
 
     for obj in _all_objs(objs):
         if hasattr(obj, "__implementation__"):
@@ -224,16 +265,23 @@ def _query_extensions(objs, query):
 
 _default_cdn_host = "https://unpkg.com"
 
-class ExtensionEmbed(NamedTuple):
+@dataclass
+class ExtensionEmbed:
     artifact_path: str
     server_url: str
-    cdn_url: Optional[str] = None
+    cdn_url: str | None = None
+
+class Pkg(TypedDict, total=False):
+    name: str
+    version: str
+    module: str
+    main: str
 
 extension_dirs: Dict[str, str] = {} # name -> path
 
-def _bundle_extensions(objs, resources: Resources) -> List[ExtensionEmbed]:
-    names = set()
-    bundles = []
+def _bundle_extensions(objs: Sequence[Model | Document], resources: Resources) -> List[ExtensionEmbed]:
+    names: Set[str] = set()
+    bundles: List[ExtensionEmbed] = []
 
     extensions = [".min.js", ".js"] if resources.minified else [".js"]
 
@@ -258,8 +306,7 @@ def _bundle_extensions(objs, resources: Resources) -> List[ExtensionEmbed]:
         server_prefix = f"{resources.root_url}static/extensions"
         package_path = join(base_dir, "package.json")
 
-        pkg: Optional[str] = None
-
+        pkg: Pkg | None = None
         if exists(package_path):
             with open(package_path) as io:
                 try:
@@ -269,20 +316,27 @@ def _bundle_extensions(objs, resources: Resources) -> List[ExtensionEmbed]:
 
         artifact_path: str
         server_url: str
-        cdn_url: Optional[str] = None
+        cdn_url: str | None = None
 
         if pkg is not None:
-            pkg_name = pkg["name"]
+            pkg_name: str | None = pkg.get("name", None)
+            if pkg_name is None:
+                raise ValueError("invalid package.json; missing package name")
             pkg_version = pkg.get("version", "latest")
             pkg_main = pkg.get("module", pkg.get("main", None))
             if pkg_main is not None:
-                cdn_url = f"{_default_cdn_host}/{pkg_name}@^{pkg_version}/{pkg_main}"
+                cdn_url = f"{_default_cdn_host}/{pkg_name}@{pkg_version}/{pkg_main}"
             else:
                 pkg_main = join(dist_dir, f"{name}.js")
             artifact_path = join(base_dir, normpath(pkg_main))
             artifacts_dir = dirname(artifact_path)
             artifact_name = basename(artifact_path)
             server_path = f"{name}/{artifact_name}"
+            if not settings.dev:
+                sha = hashlib.sha256()
+                sha.update(pkg_version.encode())
+                vstring = sha.hexdigest()
+                server_path = f"{server_path}?v={vstring}"
         else:
             for ext in extensions:
                 artifact_path = join(dist_dir, f"{name}{ext}")
@@ -300,8 +354,8 @@ def _bundle_extensions(objs, resources: Resources) -> List[ExtensionEmbed]:
 
     return bundles
 
-def _all_objs(objs):
-    all_objs = set()
+def _all_objs(objs: Sequence[Model | Document]) -> Set[Model]:
+    all_objs: Set[Model] = set()
 
     for obj in objs:
         if isinstance(obj, Document):
@@ -312,7 +366,7 @@ def _all_objs(objs):
 
     return all_objs
 
-def _any(objs, query):
+def _any(objs: Sequence[Model | Document], query: Callable[[Model], bool]) -> bool:
     ''' Whether any of a collection of objects satisfies a given query predicate
 
     Args:
@@ -333,7 +387,7 @@ def _any(objs, query):
                 return True
     return False
 
-def _use_tables(objs):
+def _use_tables(objs: Sequence[Model | Document]) -> bool:
     ''' Whether a collection of Bokeh objects contains a TableWidget
 
     Args:
@@ -346,7 +400,7 @@ def _use_tables(objs):
     from ..models.widgets import TableWidget
     return _any(objs, lambda obj: isinstance(obj, TableWidget)) or _ext_use_tables(objs)
 
-def _use_widgets(objs):
+def _use_widgets(objs: Sequence[Model | Document]) -> bool:
     ''' Whether a collection of Bokeh objects contains a any Widget
 
     Args:
@@ -359,14 +413,69 @@ def _use_widgets(objs):
     from ..models.widgets import Widget
     return _any(objs, lambda obj: isinstance(obj, Widget)) or _ext_use_widgets(objs)
 
-def _ext_use_tables(objs):
+def _model_requires_mathjax(model: Model) -> bool:
+    """Whether a model requires MathJax to be loaded
+    Args:
+        model (Model): Model to check
+    Returns:
+        bool: True if MathJax required, False if not
+    """
+    from ..models.axes import Axis
+    from ..models.widgets.markups import Div, Paragraph
+
+    if isinstance(model, Axis):
+        if isinstance(model.axis_label, str) and is_tex_string(model.axis_label):
+            return True
+
+        for val in model.major_label_overrides.values():
+            if isinstance(val, str) and is_tex_string(val):
+                return True
+
+    if isinstance(model, Div) and not model.disable_math and not model.render_as_text:
+        if contains_tex_string(model.text):
+            return True
+
+    if isinstance(model, Paragraph) and not model.disable_math:
+        if contains_tex_string(model.text):
+            return True
+
+    return False
+
+def _use_mathjax(objs: Sequence[Model | Document]) -> bool:
+    ''' Whether a collection of Bokeh objects contains a model requesting MathJax
+    Args:
+        objs (seq[Model or Document]) :
+    Returns:
+        bool
+    '''
+    from ..models.text import MathText
+
+    return _any(objs, lambda obj: isinstance(obj, MathText) or _model_requires_mathjax(obj)) or _ext_use_mathjax(objs)
+
+def _use_gl(objs: Sequence[Model | Document]) -> bool:
+    ''' Whether a collection of Bokeh objects contains a plot requesting WebGL
+
+    Args:
+        objs (seq[Model or Document]) :
+
+    Returns:
+        bool
+
+    '''
+    from ..models.plots import Plot
+    return _any(objs, lambda obj: isinstance(obj, Plot) and obj.output_backend == "webgl")
+
+def _ext_use_tables(objs: Sequence[Model | Document]) -> bool:
     from ..models.widgets import TableWidget
     return _query_extensions(objs, lambda cls: issubclass(cls, TableWidget))
 
-def _ext_use_widgets(objs):
+def _ext_use_widgets(objs: Sequence[Model | Document]) -> bool:
     from ..models.widgets import Widget
     return _query_extensions(objs, lambda cls: issubclass(cls, Widget))
 
+def _ext_use_mathjax(objs: Sequence[Model | Document]) -> bool:
+    from ..models.text import MathText
+    return _query_extensions(objs, lambda cls: issubclass(cls, MathText))
 #-----------------------------------------------------------------------------
 # Code
 #-----------------------------------------------------------------------------
