@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -56,9 +56,9 @@ event object that triggered the callback.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
+from __future__ import annotations
 
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -66,11 +66,22 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
+from typing import (
+    TYPE_CHECKING,
+    ClassVar,
+    Dict,
+    Type,
+)
 
 # External imports
+from typing_extensions import Literal, TypedDict
 
-# Bokeh imports
-from .util.future import with_metaclass
+## Bokeh imports
+if TYPE_CHECKING:
+    from .core.types import ID, GeometryData, Unknown
+    from .model import Model
+    from .models.plots import Plot
+    from .models.widgets.buttons import AbstractButton
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -78,10 +89,14 @@ from .util.future import with_metaclass
 
 __all__ = (
     'ButtonClick',
+    'DocumentEvent',
+    'DocumentReady',
     'DoubleTap',
     'Event',
     'LODStart',
     'LODEnd',
+    'MenuItemClick',
+    'ModelEvent',
     'MouseEnter',
     'MouseLeave',
     'MouseMove',
@@ -92,9 +107,14 @@ __all__ = (
     'Pinch',
     'PinchEnd',
     'PinchStart',
+    'RangesUpdate',
+    'Rotate',
+    'RotateEnd',
+    'RotateStart',
     'PlotEvent',
     'PointEvent',
     'Press',
+    'PressUp',
     'Reset',
     'SelectionGeometry',
     'Tap',
@@ -104,49 +124,33 @@ __all__ = (
 # Private API
 #-----------------------------------------------------------------------------
 
-_CONCRETE_EVENT_CLASSES = dict()
-
-class _MetaEvent(type):
-    ''' Metaclass used to keep track of all classes subclassed from Event.
-
-    All Concrete Event classes (i.e. not "abstract" event base classes with
-    no ``event_name``) will be added to the _CONCRETE_EVENT_CLASSES set which
-    is used to decode event instances from JSON.
-
-    '''
-    def __new__(cls, clsname, bases, attrs):
-        newclass = super(_MetaEvent, cls).__new__(cls, clsname, bases, attrs)
-        if newclass.event_name is not None:
-            _CONCRETE_EVENT_CLASSES[newclass.event_name] = newclass
-        return newclass
+_CONCRETE_EVENT_CLASSES: Dict[str, Type[Event]] = {}
 
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
 
-class Event(with_metaclass(_MetaEvent, object)):
+class EventJson(TypedDict):
+    event_name: str
+    event_values: Dict[str, Unknown]
+
+class Event:
     ''' Base class for all Bokeh events.
 
     This base class is not typically useful to instantiate on its own.
 
     '''
-    _event_classes = []
-    event_name = None
-
-    def __init__(self, model):
-        ''' Create a new base event.
-
-        Args:
-
-            model (Model) : a Bokeh model to register event callbacks on
-
-        '''
-        self._model_id = None
-        if model is not None:
-            self._model_id = model.id
+    event_name: ClassVar[str]
 
     @classmethod
-    def decode_json(cls, dct):
+    def __init_subclass__(cls):
+        super().__init_subclass__()
+
+        if hasattr(cls, "event_name"):
+            _CONCRETE_EVENT_CLASSES[cls.event_name] = cls
+
+    @classmethod
+    def decode_json(cls, dct: EventJson) -> Event:
         ''' Custom JSON decoder for Events.
 
         Can be used as the ``object_hook`` argument of ``json.load`` or
@@ -170,7 +174,7 @@ class Event(with_metaclass(_MetaEvent, object)):
                 <bokeh.events.Pan object at 0x1040f84a8>
 
         '''
-        if not (('event_name' in dct) and ('event_values' in dct)):
+        if not ('event_name' in dct and 'event_values' in dct):
             return dct
 
         event_name = dct['event_name']
@@ -179,35 +183,85 @@ class Event(with_metaclass(_MetaEvent, object)):
             raise ValueError("Could not find appropriate Event class for event_name: %r" % event_name)
 
         event_values = dct['event_values']
-        model_id = event_values.pop('model_id')
-        event = _CONCRETE_EVENT_CLASSES[event_name](model=None, **event_values)
-        event._model_id = model_id
+        model_id = event_values.pop('model', {"id": None})["id"]
+        event_cls = _CONCRETE_EVENT_CLASSES[event_name]
+        if issubclass(event_cls, ModelEvent):
+            event = event_cls(model=None, **event_values)
+            event._model_id = model_id
+        else:
+            event = event_cls(**event_values)
         return event
 
-class ButtonClick(Event):
-    ''' Announce a button click event on a Bokeh Button widget.
+
+class DocumentEvent(Event):
+    ''' Base class for all Bokeh Document events.
+
+    This base class is not typically useful to instantiate on its own.
+
+    '''
+
+
+class DocumentReady(DocumentEvent):
+    ''' Announce when a Document is fully idle.
+
+    '''
+    event_name = 'document_ready'
+
+
+class ModelEvent(Event):
+    ''' Base class for all Bokeh Model events.
+
+    This base class is not typically useful to instantiate on its own.
+
+    '''
+
+    _model_id: ID | None
+
+    def __init__(self, model: Model | None) -> None:
+        ''' Create a new base event.
+
+        Args:
+
+            model (Model) : a Bokeh model to register event callbacks on
+
+        '''
+        self._model_id = None
+        if model is not None:
+            self._model_id = model.id
+
+
+class ButtonClick(ModelEvent):
+    ''' Announce a button click event on a Bokeh button widget.
 
     '''
     event_name = 'button_click'
 
-    def __init__(self, model):
-        from .models.widgets import Button
-        if model is not None and not isinstance(model, Button):
-            msg ='{clsname} event only applies to Button models'
-            raise ValueError(msg.format(clsname=self.__class__.__name__))
-        super(ButtonClick, self).__init__(model=model)
+    def __init__(self, model: AbstractButton | None) -> None:
+        from .models.widgets import AbstractButton
+        if model is not None and not isinstance(model, AbstractButton):
+            clsname = self.__class__.__name__
+            raise ValueError(f"{clsname} event only applies to button models")
+        super().__init__(model=model)
 
-class PlotEvent(Event):
+class MenuItemClick(ModelEvent):
+    ''' Announce a button click event on a Bokeh menu item.
+
+    '''
+    event_name = 'menu_item_click'
+
+    def __init__(self, model: Model, item: str | None = None) -> None:
+        self.item = item
+        super().__init__(model=model)
+
+class PlotEvent(ModelEvent):
     ''' The base class for all events applicable to Plot models.
 
     '''
-
-    def __init__(self, model):
+    def __init__(self, model: Plot | None) -> None:
         from .models import Plot
         if model is not None and not isinstance(model, Plot):
-            msg ='{clsname} event only applies to Plot models'
-            raise ValueError(msg.format(clsname=self.__class__.__name__))
-        super(PlotEvent, self).__init__(model=model)
+            raise ValueError(f"{self.__class__.__name__} event only applies to Plot models")
+        super().__init__(model)
 
 class LODStart(PlotEvent):
     ''' Announce the start of "interactive level-of-detail" mode on a plot.
@@ -233,6 +287,35 @@ class LODEnd(PlotEvent):
     '''
     event_name = 'lodend'
 
+class RangesUpdate(PlotEvent):
+    ''' Announce combined range updates in a single event.
+
+    Attributes:
+        x0 (float) : start x-coordinate for the default x-range
+        x1 (float) : end x-coordinate for the default x-range
+        y0 (float) : start x-coordinate for the default y-range
+        y1 (float) : end y-coordinate for the default x-range
+
+    Callbacks may be added to range ``start`` and ``end`` properties to respond
+    to range changes, but this can result in multiple callbacks being invoked
+    for a single logical operation (e.g. a pan or zoom). This event is emitted
+    by supported tools when the entire range update is complete, in order to
+    afford a *single* event that can be responded to.
+
+    '''
+    event_name = 'rangesupdate'
+
+    def __init__(self, model: Plot | None, *,
+            x0: float | None = None,
+            x1: float | None = None,
+            y0: float | None = None,
+            y1: float | None = None):
+        self.x0 = x0
+        self.x1 = x1
+        self.y0 = y0
+        self.y1 = y1
+        super().__init__(model=model)
+
 class SelectionGeometry(PlotEvent):
     ''' Announce the coordinates of a selection event on a plot.
 
@@ -245,10 +328,10 @@ class SelectionGeometry(PlotEvent):
     '''
     event_name = "selectiongeometry"
 
-    def __init__(self, model, geometry=None, final=True):
+    def __init__(self, model: Plot | None, geometry: GeometryData | None = None, final: bool = True) -> None:
         self.geometry = geometry
         self.final = final
-        super(SelectionGeometry, self).__init__(model=model)
+        super().__init__(model=model)
 
 class Reset(PlotEvent):
     ''' Announce a button click event on a plot ``ResetTool``.
@@ -256,8 +339,8 @@ class Reset(PlotEvent):
     '''
     event_name = "reset"
 
-    def __init__(self, model):
-        super(Reset, self).__init__(model=model)
+    def __init__(self, model: Plot | None) -> None:
+        super().__init__(model=model)
 
 class PointEvent(PlotEvent):
     ''' Base class for UI events associated with a specific (x,y) point.
@@ -273,14 +356,13 @@ class PointEvent(PlotEvent):
     the HTML canvas.
 
     '''
-    event_name = None
-
-    def __init__(self, model, sx=None, sy=None, x=None, y=None):
+    def __init__(self, model: Plot | None, sx: float | None = None, sy:
+            float | None = None, x: float | None = None, y: float | None = None):
         self.sx = sx
         self.sy = sy
         self.x = x
         self.y = y
-        super(PointEvent, self).__init__(model=model)
+        super().__init__(model=model)
 
 class Tap(PointEvent):
     ''' Announce a tap or click event on a Bokeh plot.
@@ -318,6 +400,18 @@ class Press(PointEvent):
     '''
     event_name = 'press'
 
+class PressUp(PointEvent):
+    ''' Announce a pressup event on a Bokeh plot.
+
+    Attributes:
+        sx (float) : x-coordinate of the event in *screen* space
+        sy (float) : y-coordinate of the event in *screen* space
+        x (float) : x-coordinate of the event in *data* space
+        y (float) : y-coordinate of the event in *data* space
+
+    '''
+    event_name = 'pressup'
+
 class MouseEnter(PointEvent):
     ''' Announce a mouse enter event onto a Bokeh plot.
 
@@ -385,9 +479,16 @@ class MouseWheel(PointEvent):
     '''
     event_name = 'wheel'
 
-    def __init__(self, model, delta=None, **kwargs):
+    def __init__(self,
+            model: Plot | None,
+            *,
+            delta: float | None = None,
+            sx: float | None = None,
+            sy: float | None = None,
+            x: float | None = None,
+            y: float | None = None):
         self.delta = delta
-        super(MouseWheel, self).__init__(model, **kwargs)
+        super().__init__(model, sx=sx, sy=sy, x=x, y=y)
 
 class Pan(PointEvent):
     ''' Announce a pan event on a Bokeh plot.
@@ -404,11 +505,20 @@ class Pan(PointEvent):
     '''
     event_name = 'pan'
 
-    def __init__(self, model, delta_x=None, delta_y=None, direction=None, **kwargs):
+    def __init__(self,
+            model: Plot | None,
+            *,
+            delta_x: float | None = None,
+            delta_y: float | None = None,
+            direction: Literal[-1, -1] | None = None,
+            sx: float | None = None,
+            sy: float | None = None,
+            x: float | None = None,
+            y: float | None = None):
         self.delta_x = delta_x
         self.delta_y = delta_y
         self.direction = direction
-        super(Pan, self).__init__(model, **kwargs)
+        super().__init__(model, sx=sx, sy=sy, x=x, y=y)
 
 class PanEnd(PointEvent):
     ''' Announce the end of a pan event on a Bokeh plot.
@@ -450,9 +560,16 @@ class Pinch(PointEvent):
     '''
     event_name = 'pinch'
 
-    def __init__(self, model, scale=None, **kwargs):
+    def __init__(self,
+            model: Plot | None,
+            *,
+            scale: float | None = None,
+            sx: float | None = None,
+            sy: float | None = None,
+            x: float | None = None,
+            y: float | None = None):
         self.scale = scale
-        super(Pinch, self).__init__(model, **kwargs)
+        super().__init__(model, sx=sx, sy=sy, x=x, y=y)
 
 class PinchEnd(PointEvent):
     ''' Announce the end of a pinch event on a Bokeh plot.
@@ -484,6 +601,63 @@ class PinchStart(PointEvent):
     '''
     event_name = 'pinchstart'
 
+class Rotate(PointEvent):
+    ''' Announce a rotate event on a Bokeh plot.
+
+    Attributes:
+        rotation (float) : the rotation that has been done (in deg)
+        sx (float) : x-coordinate of the event in *screen* space
+        sy (float) : y-coordinate of the event in *screen* space
+        x (float) : x-coordinate of the event in *data* space
+        y (float) : y-coordinate of the event in *data* space
+
+    .. note::
+        This event is only applicable for touch-enabled devices.
+
+    '''
+    event_name = 'rotate'
+
+    def __init__(self,
+            model: Plot | None,
+            *,
+            rotation: float | None = None,
+            sx: float | None = None,
+            sy: float | None = None,
+            x: float | None = None,
+            y: float | None = None):
+        self.rotation = rotation
+        super().__init__(model, sx=sx, sy=sy, x=x, y=y)
+
+class RotateEnd(PointEvent):
+    ''' Announce the end of a rotate event on a Bokeh plot.
+
+    Attributes:
+        sx (float) : x-coordinate of the event in *screen* space
+        sy (float) : y-coordinate of the event in *screen* space
+        x (float) : x-coordinate of the event in *data* space
+        y (float) : y-coordinate of the event in *data* space
+
+    .. note::
+        This event is only applicable for touch-enabled devices.
+
+    '''
+    event_name = 'rotateend'
+
+class RotateStart(PointEvent):
+    ''' Announce the start of a rotate event on a Bokeh plot.
+
+    Attributes:
+        sx (float) : x-coordinate of the event in *screen* space
+        sy (float) : y-coordinate of the event in *screen* space
+        x (float) : x-coordinate of the event in *data* space
+        y (float) : y-coordinate of the event in *data* space
+
+    .. note::
+        This event is only applicable for touch-enabled devices.
+
+    '''
+    event_name = 'rotatestart'
+
 #-----------------------------------------------------------------------------
 # Dev API
 #-----------------------------------------------------------------------------
