@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,9 +11,9 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
+from __future__ import annotations
 
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -21,10 +21,16 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-
-# External imports
+import weakref
+from contextlib import contextmanager
+from typing import TYPE_CHECKING, Iterator, List
 
 # Bokeh imports
+from .state import curstate
+
+if TYPE_CHECKING:
+    from ..document import Document
+    from ..document.locking import UnlockedDocumentProxy
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -32,32 +38,58 @@ log = logging.getLogger(__name__)
 
 __all__ = (
     'curdoc',
+    'patch_curdoc',
     'set_curdoc',
 )
 
+# annotated global must come first in py 3.7
+_PATCHED_CURDOCS: List[weakref.ReferenceType[Document|UnlockedDocumentProxy]] = []
+
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
 
-def curdoc():
+def curdoc() -> Document | UnlockedDocumentProxy:
     ''' Return the document for the current default state.
 
     Returns:
         Document : the current default document object.
 
     '''
-    from .state import curstate
+    if len(_PATCHED_CURDOCS) > 0:
+        doc = _PATCHED_CURDOCS[-1]()
+        if doc is None:
+            raise RuntimeError("Patched curdoc has been previously destroyed")
+        return doc
     return curstate().document
 
 #-----------------------------------------------------------------------------
 # Dev API
 #-----------------------------------------------------------------------------
 
-def set_curdoc(doc):
-    '''Configure the current document (returned by curdoc()).
+@contextmanager
+def patch_curdoc(doc: Document|UnlockedDocumentProxy) -> Iterator[None]:
+    ''' Temporarily override the value of ``curdoc()`` and then return it to
+    its original state.
+
+    This context manager is useful for controlling the value of ``curdoc()``
+    while invoking functions (e.g. callbacks). The cont
 
     Args:
-        doc (Document) : Document we will output.
+        doc (Document) : new Document to use for ``curdoc()``
+
+    '''
+    global _PATCHED_CURDOCS
+    _PATCHED_CURDOCS.append(weakref.ref(doc))
+    del doc
+    yield
+    _PATCHED_CURDOCS.pop()
+
+def set_curdoc(doc: Document) -> None:
+    ''' Configure the current document (returned by curdoc()).
+
+    Args:
+        doc (Document) : new Document to use for curdoc()
 
     Returns:
         None
@@ -66,7 +98,6 @@ def set_curdoc(doc):
         Calling this function will replace any existing document.
 
     '''
-    from .state import curstate
     curstate().document = doc
 
 #-----------------------------------------------------------------------------
