@@ -8,13 +8,13 @@
 
 .. note::
     While ``State`` objects can also be manipulated explicitly, they are
-    automatically configured when the functions :func:`~bokeh.io.output_file`,
-    etc. from :ref:`bokeh.io` are used, so this is not necessary under
-    typical usage.
+    automatically configured when the output functions like |output_file|
+    from :ref:`bokeh.io` are used. Therefore, manipulating ``State`` objects is
+    usually not necessary.
 
 Generating output for Bokeh plots requires coordinating several things:
 
-:class:`~bokeh.document.Document`
+|Document|
     Groups together Bokeh models that may be shared between plots (e.g.,
     range or data source objects) into one common structure.
 
@@ -34,6 +34,8 @@ ensures their proper configuration in many common usage scenarios.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -43,10 +45,18 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 import os
+from dataclasses import dataclass
+from typing import TYPE_CHECKING, Dict, cast
 
 # Bokeh imports
+from ..core.types import PathLike
 from ..document import Document
-from ..resources import Resources
+from ..resources import Resources, ResourcesMode
+
+if TYPE_CHECKING:
+    from ..core.types import ID
+    from ..server.server import Server
+    from .notebook import CommsHandle, NotebookType
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -57,6 +67,14 @@ __all__ = (
     'State',
 )
 
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
+# XXX: move this to the bottom when Python 3.7 is dropped
+
+_STATE: State | None = None
+
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
@@ -64,8 +82,18 @@ __all__ = (
 class State:
     ''' Manage state related to controlling Bokeh output.
 
+    .. autoclasstoc::
+
     '''
-    def __init__(self):
+
+    _file: FileConfig | None
+
+    _notebook: bool
+    _notebook_type: NotebookType | None
+    last_comms_handle: CommsHandle | None
+    uuid_to_server: Dict[ID, Server]
+
+    def __init__(self) -> None:
         self.last_comms_handle = None
         self.uuid_to_server = {} # Mapping from uuid to server instance
         self.reset()
@@ -73,70 +101,62 @@ class State:
     # Properties --------------------------------------------------------------
 
     @property
-    def document(self):
-        ''' A default :class:`~bokeh.document.Document` to use for all
-        output operations.
+    def document(self) -> Document:
+        ''' A default |Document| to use for all output operations.
 
         '''
         return self._document
 
     @document.setter
-    def document(self, doc):
+    def document(self, doc: Document) -> None:
         self._document = doc
 
     @property
-    def file(self):
-        ''' A dict with the default configuration for file output (READ ONLY)
+    def file(self) -> FileConfig | None:
+        ''' A structure with the default configuration for file output (READ ONLY)
 
-        The dictionary value has the following form:
-
-        .. code-block:: python
-
-            {
-                'filename'  : # filename to use when saving
-                'resources' : # resources configuration
-                'title'     : # a title for the HTML document
-            }
+            See :class:`~bokeh.io.state.FileConfig`.
 
         '''
         return self._file
 
     @property
-    def notebook(self):
+    def notebook(self) -> bool:
         ''' Whether to generate notebook output on show operations. (READ ONLY)
 
         '''
         return self._notebook
 
     @property
-    def notebook_type(self):
+    def notebook_type(self) -> NotebookType | None:
         ''' Notebook type
 
         '''
         return self._notebook_type
 
     @notebook_type.setter
-    def notebook_type(self, notebook_type):
+    def notebook_type(self, notebook_type: NotebookType) -> None:
         ''' Notebook type, acceptable values are 'jupyter' as well as any names
         defined by external notebook hooks that have been installed.
 
         '''
         if notebook_type is None or not isinstance(notebook_type, str):
             raise ValueError("Notebook type must be a string")
-        self._notebook_type = notebook_type.lower()
+        self._notebook_type = cast("NotebookType", notebook_type.lower())
 
     # Public methods ----------------------------------------------------------
 
-    def output_file(self, filename, title="Bokeh Plot", mode=None, root_dir=None):
+    def output_file(self, filename: PathLike, title: str = "Bokeh Plot",
+            mode: ResourcesMode | None = None, root_dir: PathLike | None = None) -> None:
         ''' Configure output to a standalone HTML file.
 
-        Calling ``output_file`` not clear the effects of any other calls to
-        ``output_notebook``, etc. It adds an additional output destination
+        Calling ``output_file`` does not clear the effects of any other calls to
+        |output_notebook|, etc. It adds an additional output destination
         (publishing to HTML files). Any other active output modes continue
         to be active.
 
         Args:
-            filename (str) : a filename for saving the HTML document
+            filename (PathLike, e.g. str, Path) : a filename for saving the HTML document
 
             title (str, optional) : a title for the HTML document
 
@@ -156,20 +176,20 @@ class State:
             every time ``show()`` or ``save()`` is called.
 
         '''
-        self._file = {
-            'filename'  : filename,
-            'resources' : Resources(mode=mode, root_dir=root_dir),
-            'title'     : title
-        }
+        self._file = FileConfig(
+            filename=filename,
+            resources=Resources(mode=mode, root_dir=root_dir),
+            title=title,
+        )
 
         if os.path.isfile(filename):
-            log.info("Session output file '%s' already exists, will be overwritten." % filename)
+            log.info(f"Session output file '{filename}' already exists, will be overwritten.")
 
-    def output_notebook(self, notebook_type='jupyter'):
+    def output_notebook(self, notebook_type: NotebookType = "jupyter") -> None:
         ''' Generate output in notebook cells.
 
-        Calling ``output_notebook`` not clear the effects of any other calls
-        to ``output_file``, etc. It adds an additional output destination
+        Calling ``output_notebook`` does not clear the effects of any other
+        calls to |output_file|, etc. It adds an additional output destination
         (publishing to notebook output cells). Any other active output modes
         continue to be active.
 
@@ -180,7 +200,7 @@ class State:
         self._notebook = True
         self.notebook_type = notebook_type
 
-    def reset(self):
+    def reset(self) -> None:
         ''' Deactivate all currently active output modes and set ``curdoc()``
         to a fresh empty ``Document``.
 
@@ -195,7 +215,7 @@ class State:
 
     # Private methods ---------------------------------------------------------
 
-    def _reset_keeping_doc(self):
+    def _reset_keeping_doc(self) -> None:
         ''' Reset output modes but DO NOT replace the default Document
 
         '''
@@ -203,14 +223,14 @@ class State:
         self._notebook = False
         self._notebook_type = None
 
-    def _reset_with_doc(self, doc):
+    def _reset_with_doc(self, doc: Document) -> None:
         ''' Reset output modes but DO replace the default Document
 
         '''
         self._document = doc
         self._reset_keeping_doc()
 
-def curstate():
+def curstate() -> State:
     ''' Return the current State object
 
     Returns:
@@ -226,11 +246,11 @@ def curstate():
 # Dev API
 #-----------------------------------------------------------------------------
 
-#-----------------------------------------------------------------------------
-# Private API
-#-----------------------------------------------------------------------------
-
-_STATE = None
+@dataclass
+class FileConfig:
+    filename: PathLike
+    resources: Resources
+    title: str
 
 #-----------------------------------------------------------------------------
 # Code
