@@ -20,6 +20,8 @@ a Bokeh scene graph are called :ref:`Models <bokeh.model>`.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -34,6 +36,7 @@ from .annotations import *
 from .arrow_heads import *
 from .axes import *
 from .callbacks import *
+from .canvas import *
 from .expressions import *
 from .filters import *
 from .formatters import *
@@ -51,6 +54,7 @@ from .renderers import *
 from .scales import *
 from .selections import *
 from .sources import *
+from .text import *
 from .textures import *
 from .tickers import *
 from .tiles import *
