@@ -11,6 +11,8 @@ Bokeh plots
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -33,7 +35,7 @@ from ..core.enums import (
     TooltipAttachment,
     VerticalAlign,
 )
-from ..core.has_props import abstract
+from ..core.has_props import HasProps, abstract
 from ..core.properties import (
     Alpha,
     Angle,
@@ -60,6 +62,7 @@ from ..core.properties import (
     Seq,
     String,
     StringSpec,
+    TextLike,
     Tuple,
     field,
     value,
@@ -74,10 +77,7 @@ from ..core.property_mixins import (
     TextProps,
 )
 from ..core.validation import error
-from ..core.validation.errors import (
-    BAD_COLUMN_NAME,
-    NON_MATCHING_DATA_SOURCES_ON_LEGEND_ITEM_RENDERERS,
-)
+from ..core.validation.errors import BAD_COLUMN_NAME, NON_MATCHING_DATA_SOURCES_ON_LEGEND_ITEM_RENDERERS
 from ..model import Model
 from ..util.serialization import convert_datetime_type
 from .formatters import TickFormatter
@@ -148,18 +148,17 @@ class DataAnnotation(Annotation):
     Local data source to use when rendering annotations on the plot.
     """)
 
-@abstract
-class TextAnnotation(Annotation):
-    ''' Base class for text annotation models such as labels and titles.
-
-    '''
+class _HasRenderMode(HasProps):
 
     render_mode = Enum(RenderMode, default="canvas", help="""
-    Specifies whether the text is rendered as a canvas element or as a
-    CSS element overlaid on the canvas. The default mode is "canvas".
+    Specifies whether the contents are rendered to a canvas or as a
+    HTML element overlaid on the canvas. The default mode is "canvas".
 
     .. note::
-        The CSS labels won't be present in the output using the "save" tool.
+        This property is deprecated and will be removed in bokeh 3.0.
+
+    .. note::
+        The HTML labels won't be present in the output using the "save" tool.
 
     .. warning::
         Not all visual styling properties are supported if the render_mode is
@@ -171,11 +170,17 @@ class TextAnnotation(Annotation):
 
     """)
 
+@abstract
+class TextAnnotation(Annotation, _HasRenderMode):
+    ''' Base class for text annotation models such as labels and titles.
+
+    '''
+
 class LegendItem(Model):
     '''
 
     '''
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
         if isinstance(self.label, str):
             # Allow convenience of setting label as a string
@@ -204,6 +209,11 @@ class LegendItem(Model):
     If set to a number, Bokeh will use that number as the index in all cases.
     """)
 
+    visible = Bool(default=True, help="""
+    Whether the legend item should be displayed. See
+    :ref:`userguide_annotations_legends_item_visibility` in the user guide.
+    """)
+
     @error(NON_MATCHING_DATA_SOURCES_ON_LEGEND_ITEM_RENDERERS)
     def _check_data_sources_on_renderers(self):
         if self.label and 'field' in self.label:
@@ -222,7 +232,7 @@ class LegendItem(Model):
 class Legend(Annotation):
     ''' Render informational legends for a plot.
 
-    See :ref:`userguide_plotting_legends` for information on plotting legends.
+    See :ref:`userguide_annotations_legends` for information on plotting legends.
 
     '''
 
@@ -242,8 +252,8 @@ class Legend(Annotation):
     The title text to render.
     """)
 
-    title_props = Include(ScalarTextProps, help="""
-    The %s values for the title text.
+    title_props = Include(ScalarTextProps, prefix="title", help="""
+    The {prop} values for the title text.
     """)
 
     title_text_font_size = Override(default="13px")
@@ -254,20 +264,20 @@ class Legend(Annotation):
     The distance (in pixels) to separate the title from the legend.
     """)
 
-    border_props = Include(ScalarLineProps, help="""
-    The %s for the legend border outline.
+    border_props = Include(ScalarLineProps, prefix="border", help="""
+    The {prop} for the legend border outline.
     """)
 
     border_line_color = Override(default="#e5e5e5")
 
     border_line_alpha = Override(default=0.5)
 
-    background_props = Include(ScalarFillProps, help="""
-    The %s for the legend background style.
+    background_props = Include(ScalarFillProps, prefix="background", help="""
+    The {prop} for the legend background style.
     """)
 
-    inactive_props = Include(ScalarFillProps, help="""
-    The %s for the legend item style when inactive. These control an overlay
+    inactive_props = Include(ScalarFillProps, prefix="inactive", help="""
+    The {prop} for the legend item style when inactive. These control an overlay
     on the item that can be used to obscure it when the corresponding glyph
     is inactive (e.g. by making it semi-transparent).
     """)
@@ -284,8 +294,8 @@ class Legend(Annotation):
 
     inactive_fill_alpha = Override(default=0.7)
 
-    label_props = Include(ScalarTextProps, help="""
-    The %s for the legend labels.
+    label_props = Include(ScalarTextProps, prefix="label", help="""
+    The {prop} for the legend labels.
     """)
 
     label_text_baseline = Override(default='middle')
@@ -356,7 +366,7 @@ class Legend(Annotation):
 class ColorBar(Annotation):
     ''' Render a color bar based on a color mapper.
 
-    See :ref:`userguide_plotting_color_bars` for information on plotting color bars.
+    See :ref:`userguide_annotations_color_bars` for information on plotting color bars.
 
     '''
 
@@ -391,8 +401,8 @@ class ColorBar(Annotation):
     The title text to render.
     """)
 
-    title_props = Include(ScalarTextProps, help="""
-    The %s values for the title text.
+    title_props = Include(ScalarTextProps, prefix="title", help="""
+    The {prop} values for the title text.
     """)
 
     title_text_font_size = Override(default="13px")
@@ -411,7 +421,7 @@ class ColorBar(Annotation):
     A ``TickFormatter`` to use for formatting the visual appearance of ticks.
     """)
 
-    major_label_overrides = Dict(Either(Float, String), String, default={}, help="""
+    major_label_overrides = Dict(Either(Float, String), TextLike, default={}, help="""
     Provide explicit tick label values for specific tick locations that
     override normal formatting.
     """)
@@ -439,8 +449,8 @@ class ColorBar(Annotation):
     Amount of padding (in pixels) between the color scale and color bar border.
     """)
 
-    major_label_props = Include(ScalarTextProps, help="""
-    The %s of the major tick labels.
+    major_label_props = Include(ScalarTextProps, prefix="major_label", help="""
+    The {prop} of the major tick labels.
     """)
 
     major_label_text_font_size = Override(default="11px")
@@ -449,8 +459,8 @@ class ColorBar(Annotation):
     The distance (in pixels) to separate the tick labels from the color bar.
     """)
 
-    major_tick_props = Include(ScalarLineProps, help="""
-    The %s of the major ticks.
+    major_tick_props = Include(ScalarLineProps, prefix="major_tick", help="""
+    The {prop} of the major ticks.
     """)
 
     major_tick_line_color = Override(default="#ffffff")
@@ -465,8 +475,8 @@ class ColorBar(Annotation):
     main plot area.
     """)
 
-    minor_tick_props = Include(ScalarLineProps, help="""
-    The %s of the minor ticks.
+    minor_tick_props = Include(ScalarLineProps, prefix="minor_tick", help="""
+    The {prop} of the minor ticks.
     """)
 
     minor_tick_line_color = Override(default=None)
@@ -481,20 +491,20 @@ class ColorBar(Annotation):
     main plot area.
     """)
 
-    bar_props = Include(ScalarLineProps, help="""
-    The %s for the color scale bar outline.
+    bar_props = Include(ScalarLineProps, prefix="bar", help="""
+    The {prop} for the color scale bar outline.
     """)
 
     bar_line_color = Override(default=None)
 
-    border_props = Include(ScalarLineProps, help="""
-    The %s for the color bar border outline.
+    border_props = Include(ScalarLineProps, prefix="border", help="""
+    The {prop} for the color bar border outline.
     """)
 
     border_line_color = Override(default=None)
 
-    background_props = Include(ScalarFillProps, help="""
-    The %s for the color bar background style.
+    background_props = Include(ScalarFillProps, prefix="background", help="""
+    The {prop} for the color bar background style.
     """)
 
     background_fill_color = Override(default="#ffffff")
@@ -504,7 +514,7 @@ class ColorBar(Annotation):
 class Arrow(DataAnnotation):
     ''' Render arrows as an annotation.
 
-    See :ref:`userguide_plotting_arrows` for information on plotting arrows.
+    See :ref:`userguide_annotations_arrows` for information on plotting arrows.
 
     '''
 
@@ -542,14 +552,14 @@ class Arrow(DataAnnotation):
     Instance of ``ArrowHead``.
     """)
 
-    body_props = Include(LineProps, use_prefix=False, help="""
-    The %s values for the arrow body.
+    body_props = Include(LineProps, help="""
+    The {prop} values for the arrow body.
     """)
 
-class BoxAnnotation(Annotation):
+class BoxAnnotation(Annotation, _HasRenderMode):
     ''' Render a shaded rectangular region as an annotation.
 
-    See :ref:`userguide_plotting_box_annotations` for information on plotting box annotations.
+    See :ref:`userguide_annotations_box_annotations` for information on plotting box annotations.
 
     '''
 
@@ -561,8 +571,8 @@ class BoxAnnotation(Annotation):
     """)
 
     left_units = Enum(SpatialUnits, default='data', help="""
-    The unit type for the left attribute. Interpreted as "data space" units
-    by default.
+    The unit type for the left attribute. Interpreted as |data units| by
+    default.
     """)
 
     right = Either(Null, Auto, NumberSpec(), help="""
@@ -573,8 +583,8 @@ class BoxAnnotation(Annotation):
     """)
 
     right_units = Enum(SpatialUnits, default='data', help="""
-    The unit type for the right attribute. Interpreted as "data space" units
-    by default.
+    The unit type for the right attribute. Interpreted as |data units| by
+    default.
     """)
 
     bottom = Either(Null, Auto, NumberSpec(), help="""
@@ -585,8 +595,8 @@ class BoxAnnotation(Annotation):
     """)
 
     bottom_units = Enum(SpatialUnits, default='data', help="""
-    The unit type for the bottom attribute. Interpreted as "data space" units
-    by default.
+    The unit type for the bottom attribute. Interpreted as |data units| by
+    default.
     """)
 
     top = Either(Null, Auto, NumberSpec(), help="""
@@ -597,20 +607,20 @@ class BoxAnnotation(Annotation):
     """)
 
     top_units = Enum(SpatialUnits, default='data', help="""
-    The unit type for the top attribute. Interpreted as "data space" units
-    by default.
+    The unit type for the top attribute. Interpreted as |data units| by
+    default.
     """)
 
-    line_props = Include(ScalarLineProps, use_prefix=False, help="""
-    The %s values for the box.
+    line_props = Include(ScalarLineProps, help="""
+    The {prop} values for the box.
     """)
 
-    fill_props = Include(ScalarFillProps, use_prefix=False, help="""
-    The %s values for the box.
+    fill_props = Include(ScalarFillProps, help="""
+    The {prop} values for the box.
     """)
 
-    hatch_props = Include(ScalarHatchProps, use_prefix=False, help="""
-    The %s values for the box.
+    hatch_props = Include(ScalarHatchProps, help="""
+    The {prop} values for the box.
     """)
 
     line_alpha = Override(default=0.3)
@@ -621,34 +631,21 @@ class BoxAnnotation(Annotation):
 
     fill_color = Override(default="#fff9ba")
 
-    render_mode = Enum(RenderMode, default="canvas", help="""
-    Specifies whether the box is rendered as a canvas element or as an
-    css element overlaid on the canvas. The default mode is "canvas".
-
-    .. note:
-        This property is deprecated and will be removed in bokeh 3.0.
-
-    .. warning::
-        The line_dash and line_dash_offset attributes aren't supported if
-        the render_mode is set to "css"
-
-    """)
-
 class Band(DataAnnotation):
     ''' Render a filled area band along a dimension.
 
-    See :ref:`userguide_plotting_bands` for information on plotting bands.
+    See :ref:`userguide_annotations_bands` for information on plotting bands.
 
     '''
-    lower = PropertyUnitsSpec(default=field("lower"), units_type=Enum(SpatialUnits), units_default="data", help="""
+    lower = PropertyUnitsSpec(default=field("lower"), units_enum=SpatialUnits, units_default="data", help="""
     The coordinates of the lower portion of the filled area band.
     """)
 
-    upper = PropertyUnitsSpec(default=field("upper"), units_type=Enum(SpatialUnits), units_default="data", help="""
+    upper = PropertyUnitsSpec(default=field("upper"), units_enum=SpatialUnits, units_default="data", help="""
     The coordinates of the upper portion of the filled area band.
     """)
 
-    base = PropertyUnitsSpec(default=field("base"), units_type=Enum(SpatialUnits), units_default="data", help="""
+    base = PropertyUnitsSpec(default=field("base"), units_enum=SpatialUnits, units_default="data", help="""
     The orthogonal coordinates of the upper and lower values.
     """)
 
@@ -657,16 +654,16 @@ class Band(DataAnnotation):
     to "height" (``y`` direction) or "width" (``x`` direction).
     """)
 
-    line_props = Include(ScalarLineProps, use_prefix=False, help="""
-    The %s values for the band.
+    line_props = Include(ScalarLineProps, help="""
+    The {prop} values for the band.
     """)
 
     line_alpha = Override(default=0.3)
 
     line_color = Override(default="#cccccc")
 
-    fill_props = Include(ScalarFillProps, use_prefix=False, help="""
-    The %s values for the band.
+    fill_props = Include(ScalarFillProps, help="""
+    The {prop} values for the band.
     """)
 
     fill_alpha = Override(default=0.4)
@@ -689,7 +686,7 @@ class Label(TextAnnotation):
     appearance of the text, its background, as well as the rectangular bounding
     box border.
 
-    See :ref:`userguide_plotting_labels` for information on plotting labels.
+    See :ref:`userguide_annotations_labels` for information on plotting labels.
 
     '''
 
@@ -701,8 +698,8 @@ class Label(TextAnnotation):
     """).accepts(Datetime, convert_datetime_type)
 
     x_units = Enum(SpatialUnits, default='data', help="""
-    The unit type for the x attribute. Interpreted as "data space" units
-    by default.
+    The unit type for the x attribute. Interpreted as |data units| by
+    default.
     """)
 
     y = NonNullable(Float, help="""
@@ -713,8 +710,8 @@ class Label(TextAnnotation):
     """).accepts(Datetime, convert_datetime_type)
 
     y_units = Enum(SpatialUnits, default='data', help="""
-    The unit type for the y attribute. Interpreted as "data space" units
-    by default.
+    The unit type for the y attribute. Interpreted as |data units| by
+    default.
     """)
 
     text = String(default="", help="""
@@ -723,12 +720,6 @@ class Label(TextAnnotation):
 
     angle = Angle(default=0, help="""
     The angle to rotate the text, as measured from the horizontal.
-
-    .. warning::
-        The center of rotation for canvas and css render_modes is different.
-        For `render_mode="canvas"` the label is rotated from the top-left
-        corner of the annotation, while for `render_mode="css"` the annotation
-        is rotated around it's center.
     """)
 
     angle_units = Enum(AngleUnits, default='rad', help="""
@@ -739,33 +730,33 @@ class Label(TextAnnotation):
     Offset value to apply to the x-coordinate.
 
     This is useful, for instance, if it is desired to "float" text a fixed
-    distance in screen units from a given data position.
+    distance in |screen units| from a given data position.
     """)
 
     y_offset = Float(default=0, help="""
     Offset value to apply to the y-coordinate.
 
     This is useful, for instance, if it is desired to "float" text a fixed
-    distance in screen units from a given data position.
+    distance in |screen units| from a given data position.
     """)
 
-    text_props = Include(ScalarTextProps, use_prefix=False, help="""
-    The %s values for the text.
+    text_props = Include(ScalarTextProps, help="""
+    The {prop} values for the text.
     """)
 
-    background_props = Include(ScalarFillProps, use_prefix=True, help="""
-    The %s values for the text bounding box.
+    background_props = Include(ScalarFillProps, prefix="background", help="""
+    The {prop} values for the text bounding box.
     """)
 
     background_fill_color = Override(default=None)
 
-    border_props = Include(ScalarLineProps, use_prefix=True, help="""
-    The %s values for the text bounding box.
+    border_props = Include(ScalarLineProps, prefix="border", help="""
+    The {prop} values for the text bounding box.
     """)
 
     border_line_color = Override(default=None)
 
-class LabelSet(TextAnnotation): # TODO: DataAnnotation
+class LabelSet(DataAnnotation, _HasRenderMode):
     ''' Render multiple text labels as annotations.
 
     ``LabelSet`` will render multiple text labels at given ``x`` and ``y``
@@ -795,8 +786,8 @@ class LabelSet(TextAnnotation): # TODO: DataAnnotation
     """)
 
     x_units = Enum(SpatialUnits, default='data', help="""
-    The unit type for the ``xs`` attribute. Interpreted as "data space" units
-    by default.
+    The unit type for the ``xs`` attribute. Interpreted as |data units| by
+    default.
     """)
 
     y = NumberSpec(default=field("y"), help="""
@@ -804,8 +795,8 @@ class LabelSet(TextAnnotation): # TODO: DataAnnotation
     """)
 
     y_units = Enum(SpatialUnits, default='data', help="""
-    The unit type for the ``ys`` attribute. Interpreted as "data space" units
-    by default.
+    The unit type for the ``ys`` attribute. Interpreted as |data units| by
+    default.
     """)
 
     text = StringSpec(default=field("text"), help="""
@@ -814,52 +805,42 @@ class LabelSet(TextAnnotation): # TODO: DataAnnotation
 
     angle = AngleSpec(default=0, help="""
     The angles to rotate the text, as measured from the horizontal.
-
-    .. warning::
-        The center of rotation for canvas and css render_modes is different.
-        For `render_mode="canvas"` the label is rotated from the top-left
-        corner of the annotation, while for `render_mode="css"` the annotation
-        is rotated around it's center.
     """)
 
     x_offset = NumberSpec(default=0, help="""
     Offset values to apply to the x-coordinates.
 
     This is useful, for instance, if it is desired to "float" text a fixed
-    distance in screen units from a given data position.
+    distance in |screen units| from a given data position.
     """)
 
     y_offset = NumberSpec(default=0, help="""
     Offset values to apply to the y-coordinates.
 
     This is useful, for instance, if it is desired to "float" text a fixed
-    distance in screen units from a given data position.
+    distance in |screen units| from a given data position.
     """)
 
-    text_props = Include(TextProps, use_prefix=False, help="""
-    The %s values for the text.
+    text_props = Include(TextProps, help="""
+    The {prop} values for the text.
     """)
 
-    background_props = Include(FillProps, use_prefix=True, help="""
-    The %s values for the text bounding box.
+    background_props = Include(FillProps, prefix="background", help="""
+    The {prop} values for the text bounding box.
     """)
 
     background_fill_color = Override(default=None)
 
-    border_props = Include(LineProps, use_prefix=True, help="""
-    The %s values for the text bounding box.
+    border_props = Include(LineProps, prefix="border", help="""
+    The {prop} values for the text bounding box.
     """)
 
     border_line_color = Override(default=None)
 
-    source = Instance(DataSource, default=lambda: ColumnDataSource(), help="""
-    Local data source to use when rendering annotations on the plot.
-    """)
-
 class PolyAnnotation(Annotation):
     ''' Render a shaded polygonal region as an annotation.
 
-    See :ref:`userguide_plotting_polygon_annotations` for information on
+    See :ref:`userguide_annotations_polygon_annotations` for information on
     plotting polygon annotations.
 
     '''
@@ -869,8 +850,8 @@ class PolyAnnotation(Annotation):
     """)
 
     xs_units = Enum(SpatialUnits, default='data', help="""
-    The unit type for the ``xs`` attribute. Interpreted as "data space" units
-    by default.
+    The unit type for the ``xs`` attribute. Interpreted as |data units| by
+    default.
     """)
 
     ys = Seq(Float, default=[], help="""
@@ -878,20 +859,20 @@ class PolyAnnotation(Annotation):
     """)
 
     ys_units = Enum(SpatialUnits, default='data', help="""
-    The unit type for the ``ys`` attribute. Interpreted as "data space" units
-    by default.
+    The unit type for the ``ys`` attribute. Interpreted as |data units| by
+    default.
     """)
 
-    line_props = Include(ScalarLineProps, use_prefix=False, help="""
-    The %s values for the polygon.
+    line_props = Include(ScalarLineProps, help="""
+    The {prop} values for the polygon.
     """)
 
-    fill_props = Include(ScalarFillProps, use_prefix=False, help="""
-    The %s values for the polygon.
+    fill_props = Include(ScalarFillProps, help="""
+    The {prop} values for the polygon.
     """)
 
-    hatch_props = Include(ScalarHatchProps, use_prefix=False, help="""
-    The %s values for the polygon.
+    hatch_props = Include(ScalarHatchProps, help="""
+    The {prop} values for the polygon.
     """)
 
     line_alpha = Override(default=0.3)
@@ -905,26 +886,26 @@ class PolyAnnotation(Annotation):
 class Slope(Annotation):
     """ Render a sloped line as an annotation.
 
-    See :ref:`userguide_plotting_slope` for information on plotting slopes.
+    See :ref:`userguide_annotations_slope` for information on plotting slopes.
 
     """
 
     gradient = Nullable(Float, help="""
-    The gradient of the line, in data units
+    The gradient of the line, in |data units|
     """)
 
     y_intercept = Nullable(Float, help="""
-    The y intercept of the line, in data units
+    The y intercept of the line, in |data units|
     """)
 
-    line_props = Include(ScalarLineProps, use_prefix=False, help="""
-    The %s values for the line.
+    line_props = Include(ScalarLineProps, help="""
+    The {prop} values for the line.
     """)
 
-class Span(Annotation):
+class Span(Annotation, _HasRenderMode):
     """ Render a horizontal or vertical line span.
 
-    See :ref:`userguide_plotting_spans` for information on plotting spans.
+    See :ref:`userguide_annotations_spans` for information on plotting spans.
 
     """
 
@@ -945,27 +926,14 @@ class Span(Annotation):
     to "height" (``y`` direction) or "width" (``x`` direction).
     """)
 
-    render_mode = Enum(RenderMode, default="canvas", help="""
-    Specifies whether the span is rendered as a canvas element or as a
-    CSS element overlaid on the canvas. The default mode is "canvas".
-
-    .. note:
-        This property is deprecated and will be removed in bokeh 3.0.
-
-    .. warning::
-        The line_dash and line_dash_offset attributes aren't supported if
-        the render_mode is set to "css"
-
-    """)
-
-    line_props = Include(ScalarLineProps, use_prefix=False, help="""
-    The %s values for the span.
+    line_props = Include(ScalarLineProps, help="""
+    The {prop} values for the span.
     """)
 
 class Title(TextAnnotation):
     ''' Render a single title box as an annotation.
 
-    See :ref:`userguide_plotting_titles` for information on plotting titles.
+    See :ref:`userguide_annotations_titles` for information on plotting titles.
 
     '''
 
@@ -1028,14 +996,14 @@ class Title(TextAnnotation):
     An alpha value to use to fill text with.
     """)
 
-    background_props = Include(ScalarFillProps, use_prefix=True, help="""
-    The %s values for the text bounding box.
+    background_props = Include(ScalarFillProps, prefix="background", help="""
+    The {prop} values for the text bounding box.
     """)
 
     background_fill_color = Override(default=None)
 
-    border_props = Include(ScalarLineProps, use_prefix=True, help="""
-    The %s values for the text bounding box.
+    border_props = Include(ScalarLineProps, prefix="border", help="""
+    The {prop} values for the text bounding box.
     """)
 
     border_line_color = Override(default=None)
@@ -1071,11 +1039,11 @@ class Tooltip(Annotation):
 class Whisker(DataAnnotation):
     ''' Render a whisker along a dimension.
 
-    See :ref:`userguide_plotting_whiskers` for information on plotting whiskers.
+    See :ref:`userguide_annotations_whiskers` for information on plotting whiskers.
 
     '''
 
-    lower = PropertyUnitsSpec(default=field("lower"), units_type=Enum(SpatialUnits), units_default="data", help="""
+    lower = PropertyUnitsSpec(default=field("lower"), units_enum=SpatialUnits, units_default="data", help="""
     The coordinates of the lower end of the whiskers.
     """)
 
@@ -1083,7 +1051,7 @@ class Whisker(DataAnnotation):
     Instance of ``ArrowHead``.
     """)
 
-    upper = PropertyUnitsSpec(default=field("upper"), units_type=Enum(SpatialUnits), units_default="data", help="""
+    upper = PropertyUnitsSpec(default=field("upper"), units_enum=SpatialUnits, units_default="data", help="""
     The coordinates of the upper end of the whiskers.
     """)
 
@@ -1091,7 +1059,7 @@ class Whisker(DataAnnotation):
     Instance of ``ArrowHead``.
     """)
 
-    base = PropertyUnitsSpec(default=field("base"), units_type=Enum(SpatialUnits), units_default="data", help="""
+    base = PropertyUnitsSpec(default=field("base"), units_enum=SpatialUnits, units_default="data", help="""
     The orthogonal coordinates of the upper and lower values.
     """)
 
@@ -1100,8 +1068,8 @@ class Whisker(DataAnnotation):
     to "height" (``y`` direction) or "width" (``x`` direction).
     """)
 
-    line_props = Include(LineProps, use_prefix=False, help="""
-    The %s values for the whisker body.
+    line_props = Include(LineProps, help="""
+    The {prop} values for the whisker body.
     """)
 
     level = Override(default="underlay")
