@@ -1,52 +1,120 @@
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 ''' Models for representing top-level plot objects.
 
 '''
-from __future__ import absolute_import
 
-from ..core.enums import Location, OutputBackend
-from ..core.properties import Bool, Dict, Enum, Include, Instance, Int, List, Override, String, Float
-from ..core.property_mixins import LineProps, FillProps
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+from __future__ import annotations
+
+import logging # isort:skip
+log = logging.getLogger(__name__)
+
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Standard library imports
+import warnings
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    List as TList,
+    overload,
+)
+
+# External imports
+from typing_extensions import Literal
+
+# Bokeh imports
+from ..core.enums import (
+    Location,
+    OutputBackend,
+    Place,
+    PlaceType,
+    ResetPolicy,
+)
+from ..core.properties import (
+    Alias,
+    Bool,
+    Dict,
+    Either,
+    Enum,
+    Float,
+    Include,
+    Instance,
+    Int,
+    List,
+    Null,
+    Nullable,
+    Override,
+    Readonly,
+    String,
+)
+from ..core.property_mixins import ScalarFillProps, ScalarLineProps
 from ..core.query import find
 from ..core.validation import error, warning
-from ..core.validation.errors import REQUIRED_RANGE, REQUIRED_SCALE, INCOMPATIBLE_SCALE_AND_RANGE
-from ..core.validation.warnings import MISSING_RENDERERS
-from ..util.plot_utils import _list_attr_splat, _select_helper
+from ..core.validation.errors import (
+    BAD_EXTRA_RANGE_NAME,
+    INCOMPATIBLE_SCALE_AND_RANGE,
+    REQUIRED_RANGE,
+    REQUIRED_SCALE,
+)
+from ..core.validation.warnings import (
+    FIXED_HEIGHT_POLICY,
+    FIXED_SIZING_MODE,
+    FIXED_WIDTH_POLICY,
+    MISSING_RENDERERS,
+)
+from ..model import Model
 from ..util.string import nice_join
-
-from .annotations import Legend, Title
+from .annotations import Annotation, Legend, Title
 from .axes import Axis
 from .glyphs import Glyph
 from .grids import Grid
 from .layouts import LayoutDOM
-from .ranges import Range, FactorRange, DataRange1d, Range1d
+from .ranges import (
+    DataRange1d,
+    FactorRange,
+    Range,
+    Range1d,
+)
 from .renderers import GlyphRenderer, Renderer, TileRenderer
-from .scales import Scale, CategoricalScale, LinearScale, LogScale
-from .sources import DataSource, ColumnDataSource
-from .tools import Tool, Toolbar, HoverTool
+from .scales import (
+    CategoricalScale,
+    LinearScale,
+    LogScale,
+    Scale,
+)
+from .sources import ColumnarDataSource, ColumnDataSource, DataSource
+from .tools import HoverTool, Tool, Toolbar
 
-def _check_conflicting_kwargs(a1, a2, kwargs):
-    if a1 in kwargs and a2 in kwargs:
-        raise ValueError("Conflicting properties set on plot: %r and %r" % (a1, a2))
+if TYPE_CHECKING:
+    from .tiles import TileSource
+
+#-----------------------------------------------------------------------------
+# Globals and constants
+#-----------------------------------------------------------------------------
+
+__all__ = (
+    'Plot',
+)
+
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
 
 class Plot(LayoutDOM):
     ''' Model representing a plot, containing glyphs, guides, annotations.
 
     '''
 
-    def __init__(self, **kwargs):
-        '''
-
-        '''
-        _check_conflicting_kwargs("toolbar", "tools", kwargs)
-        _check_conflicting_kwargs("toolbar", "logo", kwargs)
-
-        if "toolbar" not in kwargs:
-            tools = kwargs.pop('tools', [])
-            logo = kwargs.pop('logo', 'normal')
-            kwargs["toolbar"] = Toolbar(tools=tools, logo=logo)
-
-        super(LayoutDOM, self).__init__(**kwargs)
-
     def select(self, *args, **kwargs):
         ''' Query this object and all of its references for objects that
         match the given selector.
@@ -61,6 +129,10 @@ class Plot(LayoutDOM):
         Keyword Arguments:
             kwargs : query dict key/values as keyword arguments
 
+        Additionally, for compatibility with ``Model.select``, a selector
+        dict may be passed as ``selector`` keyword argument, in which case
+        the value of ``kwargs['selector']`` is used for the query.
+
         For convenience, queries on just names can be made by supplying
         the ``name`` string as the single parameter:
 
@@ -80,7 +152,8 @@ class Plot(LayoutDOM):
 
             .. code-block:: python
 
-                # These two are equivalent
+                # These three are equivalent
+                p.select(selector={"type": HoverTool})
                 p.select({"type": HoverTool})
                 p.select(HoverTool)
 
@@ -97,7 +170,7 @@ class Plot(LayoutDOM):
         selector = _select_helper(args, kwargs)
 
         # Want to pass selector that is a dictionary
-        return _list_attr_splat(find(self.references(), selector, {'plot': self}))
+        return _list_attr_splat(find(self.references(), selector))
 
     def row(self, row, gridplot):
         ''' Return whether this plot is in a given row of a GridPlot.
@@ -155,11 +228,12 @@ class Plot(LayoutDOM):
 
     @property
     def legend(self):
-        ''' Splattable list of :class:`~bokeh.models.annotations.Legend` objects.
+        ''' Splattable list of |Legend| objects.
 
         '''
-        legends = [obj for obj in self.renderers if isinstance(obj, Legend)]
-        return _list_attr_splat(legends)
+        panels = self.above + self.below + self.left + self.right + self.center
+        legends = [obj for obj in panels if isinstance(obj, Legend)]
+        return _legend_attr_splat(legends)
 
     @property
     def hover(self):
@@ -169,8 +243,8 @@ class Plot(LayoutDOM):
         hovers = [obj for obj in self.tools if isinstance(obj, HoverTool)]
         return _list_attr_splat(hovers)
 
-    def _grid(self, dimension):
-        grid = [obj for obj in self.renderers if isinstance(obj, Grid) and obj.dimension==dimension]
+    def _grid(self, dimension: Literal[0, 1]):
+        grid = [obj for obj in self.center if isinstance(obj, Grid) and obj.dimension == dimension]
         return _list_attr_splat(grid)
 
     @property
@@ -195,14 +269,14 @@ class Plot(LayoutDOM):
         return _list_attr_splat(self.xgrid + self.ygrid)
 
     @property
-    def tools(self):
+    def tools(self) -> TList[Tool]:
         return self.toolbar.tools
 
     @tools.setter
-    def tools(self, tools):
+    def tools(self, tools: TList[Tool]):
         self.toolbar.tools = tools
 
-    def add_layout(self, obj, place='center'):
+    def add_layout(self, obj: Renderer, place: PlaceType = "center") -> None:
         ''' Adds an object to the plot in a specified place.
 
         Args:
@@ -214,23 +288,14 @@ class Plot(LayoutDOM):
             None
 
         '''
-        valid_places = ['left', 'right', 'above', 'below', 'center']
-        if place not in valid_places:
+        if place not in Place:
             raise ValueError(
-                "Invalid place '%s' specified. Valid place values are: %s" % (place, nice_join(valid_places))
+                f"Invalid place '{place}' specified. Valid place values are: {nice_join(Place)}"
             )
 
-        if hasattr(obj, 'plot'):
-            if obj.plot is not None:
-                raise ValueError("object to be added already has 'plot' attribute set")
-            obj.plot = self
+        getattr(self, place).append(obj)
 
-        self.renderers.append(obj)
-
-        if place is not 'center':
-            getattr(self, place).append(obj)
-
-    def add_tools(self, *tools):
+    def add_tools(self, *tools: Tool) -> None:
         ''' Adds tools to the plot.
 
         Args:
@@ -244,12 +309,14 @@ class Plot(LayoutDOM):
             if not isinstance(tool, Tool):
                 raise ValueError("All arguments to add_tool must be Tool subclasses.")
 
-            if hasattr(tool, 'overlay'):
-                self.renderers.append(tool.overlay)
-
             self.toolbar.tools.append(tool)
 
-    def add_glyph(self, source_or_glyph, glyph=None, **kw):
+    @overload
+    def add_glyph(self, glyph: Glyph, **kwargs: Any) -> GlyphRenderer: ...
+    @overload
+    def add_glyph(self, source: ColumnarDataSource, glyph: Glyph, **kwargs: Any) -> GlyphRenderer: ...
+
+    def add_glyph(self, source_or_glyph: Glyph | ColumnarDataSource, glyph: Glyph | None = None, **kwargs: Any) -> GlyphRenderer:
         ''' Adds a glyph to the plot with associated data sources and ranges.
 
         This function will take care of creating and configuring a Glyph object,
@@ -268,7 +335,7 @@ class Plot(LayoutDOM):
             GlyphRenderer
 
         '''
-        if glyph is not None:
+        if isinstance(source_or_glyph, ColumnarDataSource):
             source = source_or_glyph
         else:
             source, glyph = ColumnDataSource(), source_or_glyph
@@ -279,12 +346,12 @@ class Plot(LayoutDOM):
         if not isinstance(glyph, Glyph):
             raise ValueError("'glyph' argument to add_glyph() must be Glyph subclass")
 
-        g = GlyphRenderer(data_source=source, glyph=glyph, **kw)
+        g = GlyphRenderer(data_source=source, glyph=glyph, **kwargs)
         self.renderers.append(g)
         return g
 
-    def add_tile(self, tile_source, **kw):
-        ''' Adds new TileRenderer into the Plot.renderers
+    def add_tile(self, tile_source: TileSource, **kwargs: Any) -> TileRenderer:
+        ''' Adds new ``TileRenderer`` into ``Plot.renderers``
 
         Args:
             tile_source (TileSource) : a tile source instance which contain tileset configuration
@@ -296,70 +363,91 @@ class Plot(LayoutDOM):
             TileRenderer : TileRenderer
 
         '''
-        tile_renderer = TileRenderer(tile_source=tile_source, **kw)
+        tile_renderer = TileRenderer(tile_source=tile_source, **kwargs)
         self.renderers.append(tile_renderer)
         return tile_renderer
 
     @error(REQUIRED_RANGE)
-    def _check_required_range(self):
-        missing = []
+    def _check_required_range(self) -> str | None:
+        missing: TList[str] = []
         if not self.x_range: missing.append('x_range')
         if not self.y_range: missing.append('y_range')
         if missing:
             return ", ".join(missing) + " [%s]" % self
 
     @error(REQUIRED_SCALE)
-    def _check_required_scale(self):
-        missing = []
+    def _check_required_scale(self) -> str | None:
+        missing: TList[str] = []
         if not self.x_scale: missing.append('x_scale')
         if not self.y_scale: missing.append('y_scale')
         if missing:
             return ", ".join(missing) + " [%s]" % self
 
     @error(INCOMPATIBLE_SCALE_AND_RANGE)
-    def _check_compatible_scale_and_ranges(self):
-        incompatible = []
+    def _check_compatible_scale_and_ranges(self) -> str | None:
+        incompatible: TList[str] = []
         x_ranges = list(self.extra_x_ranges.values())
         if self.x_range: x_ranges.append(self.x_range)
         y_ranges = list(self.extra_y_ranges.values())
         if self.y_range: y_ranges.append(self.y_range)
 
-        for rng in x_ranges:
-            if isinstance(rng, (DataRange1d, Range1d)) and not isinstance(self.x_scale, (LinearScale, LogScale)):
-                incompatible.append("incompatibility on x-dimension: %s, %s" %(rng, self.x_scale))
-            elif isinstance(rng, FactorRange) and not isinstance(self.x_scale, CategoricalScale):
-                incompatible.append("incompatibility on x-dimension: %s/%s" %(rng, self.x_scale))
-            # special case because CategoricalScale is a subclass of LinearScale, should be removed in future
-            if isinstance(rng, (DataRange1d, Range1d)) and isinstance(self.x_scale, CategoricalScale):
-                incompatible.append("incompatibility on x-dimension: %s, %s" %(rng, self.x_scale))
+        if self.x_scale is not None:
+            for rng in x_ranges:
+                if isinstance(rng, (DataRange1d, Range1d)) and not isinstance(self.x_scale, (LinearScale, LogScale)):
+                    incompatible.append("incompatibility on x-dimension: %s, %s" %(rng, self.x_scale))
+                elif isinstance(rng, FactorRange) and not isinstance(self.x_scale, CategoricalScale):
+                    incompatible.append("incompatibility on x-dimension: %s/%s" %(rng, self.x_scale))
+                # special case because CategoricalScale is a subclass of LinearScale, should be removed in future
+                if isinstance(rng, (DataRange1d, Range1d)) and isinstance(self.x_scale, CategoricalScale):
+                    incompatible.append("incompatibility on x-dimension: %s, %s" %(rng, self.x_scale))
 
-        for rng in y_ranges:
-            if isinstance(rng, (DataRange1d, Range1d)) and not isinstance(self.y_scale, (LinearScale, LogScale)):
-                incompatible.append("incompatibility on y-dimension: %s/%s" %(rng, self.y_scale))
-            elif isinstance(rng, FactorRange) and not isinstance(self.y_scale, CategoricalScale):
-                incompatible.append("incompatibility on y-dimension: %s/%s" %(rng, self.y_scale))
-            # special case because CategoricalScale is a subclass of LinearScale, should be removed in future
-            if isinstance(rng, (DataRange1d, Range1d)) and isinstance(self.y_scale, CategoricalScale):
-                incompatible.append("incompatibility on y-dimension: %s, %s" %(rng, self.y_scale))
+        if self.y_scale is not None:
+            for rng in y_ranges:
+                if isinstance(rng, (DataRange1d, Range1d)) and not isinstance(self.y_scale, (LinearScale, LogScale)):
+                    incompatible.append("incompatibility on y-dimension: %s/%s" %(rng, self.y_scale))
+                elif isinstance(rng, FactorRange) and not isinstance(self.y_scale, CategoricalScale):
+                    incompatible.append("incompatibility on y-dimension: %s/%s" %(rng, self.y_scale))
+                # special case because CategoricalScale is a subclass of LinearScale, should be removed in future
+                if isinstance(rng, (DataRange1d, Range1d)) and isinstance(self.y_scale, CategoricalScale):
+                    incompatible.append("incompatibility on y-dimension: %s, %s" %(rng, self.y_scale))
 
         if incompatible:
             return ", ".join(incompatible) + " [%s]" % self
 
     @warning(MISSING_RENDERERS)
-    def _check_missing_renderers(self):
-        if len(self.renderers) == 0:
+    def _check_missing_renderers(self) -> str | None:
+        if len(self.renderers) == 0 and len([x for x in self.center if isinstance(x, Annotation)]) == 0:
             return str(self)
 
-    x_range = Instance(Range, help="""
+    @error(BAD_EXTRA_RANGE_NAME)
+    def _check_bad_extra_range_name(self) -> str | None:
+        msg: str = ""
+        valid = {
+            f'{axis}_name': {'default', *getattr(self, f"extra_{axis}s")}
+            for axis in ("x_range", "y_range")
+        }
+        for place in list(Place) + ['renderers']:
+            for ref in getattr(self, place):
+                bad = ', '.join(
+                    f"{axis}='{getattr(ref, axis)}'"
+                    for axis, keys in valid.items()
+                    if getattr(ref, axis, 'default') not in keys
+                )
+                if bad:
+                    msg += (", " if msg else "") + f"{bad} [{ref}]"
+        if msg:
+            return msg
+
+    x_range = Instance(Range, default=lambda: DataRange1d(), help="""
     The (default) data range of the horizontal dimension of the plot.
     """)
 
-    y_range = Instance(Range, help="""
+    y_range = Instance(Range, default=lambda: DataRange1d(), help="""
     The (default) data range of the vertical dimension of the plot.
     """)
 
     @classmethod
-    def _scale(cls, scale):
+    def _scale(cls, scale: Literal["auto", "linear", "log", "categorical"]) -> Scale:
         if scale in ["auto", "linear"]:
             return LinearScale()
         elif scale == "log":
@@ -367,7 +455,7 @@ class Plot(LayoutDOM):
         if scale == "categorical":
             return CategoricalScale()
         else:
-            raise ValueError("Unknown mapper_type: %s" % scale)
+            raise ValueError(f"Unknown mapper_type: {scale}")
 
     x_scale = Instance(Scale, default=lambda: LinearScale(), help="""
     What kind of scale to use to convert x-coordinates in data space
@@ -391,40 +479,56 @@ class Plot(LayoutDOM):
     This is useful for adding additional axes.
     """)
 
+    extra_x_scales = Dict(String, Instance(Scale), help="""
+    Additional named scales to make available for mapping x-coordinates.
+
+    This is useful for adding additional axes.
+
+    .. note:: This feature is experimental and may change in the short term.
+    """)
+
+    extra_y_scales = Dict(String, Instance(Scale), help="""
+    Additional named scales to make available for mapping y-coordinates.
+
+    This is useful for adding additional axes.
+
+    .. note:: This feature is experimental and may change in the short term.
+    """)
+
     hidpi = Bool(default=True, help="""
     Whether to use HiDPI mode when available.
     """)
 
-    title = Instance(Title, default=lambda: Title(text=""), help="""
+    title = Either(Null, Instance(Title), default=lambda: Title(text=""), help="""
     A title for the plot. Can be a text string or a Title annotation.
-    """)
+    """).accepts(String, lambda text: Title(text=text))
 
-    title_location = Enum(Location, default="above", help="""
+    title_location = Nullable(Enum(Location), default="above", help="""
     Where the title will be located. Titles on the left or right side
     will be rotated.
     """)
 
-    outline_props = Include(LineProps, help="""
-    The %s for the plot border outline.
+    outline_props = Include(ScalarLineProps, prefix="outline", help="""
+    The {prop} for the plot border outline.
     """)
 
     outline_line_color = Override(default="#e5e5e5")
 
     renderers = List(Instance(Renderer), help="""
     A list of all renderers for this plot, including guides and annotations
-    in addition to glyphs and markers.
+    in addition to glyphs.
 
     This property can be manipulated by hand, but the ``add_glyph`` and
     ``add_layout`` methods are recommended to help make sure all necessary
     setup is performed.
     """)
 
-    toolbar = Instance(Toolbar, help="""
+    toolbar = Instance(Toolbar, default=lambda: Toolbar(), help="""
     The toolbar associated with this plot which holds all the tools. It is
     automatically created with the plot if necessary.
     """)
 
-    toolbar_location = Enum(Location, default="right", help="""
+    toolbar_location = Nullable(Enum(Location), default="right", help="""
     Where the toolbar will be located. If set to None, no toolbar
     will be attached to the plot.
     """)
@@ -450,27 +554,33 @@ class Plot(LayoutDOM):
     A list of renderers to occupy the area below of the plot.
     """)
 
-    plot_height = Int(600, help="""
-    Total height of the entire plot (including any axes, titles,
-    border padding, etc.)
-
-    .. note::
-        This corresponds directly to the height of the HTML
-        canvas that will be used.
-
+    center = List(Instance(Renderer), help="""
+    A list of renderers to occupy the center area (frame) of the plot.
     """)
 
-    plot_width = Int(600, help="""
-    Total width of the entire plot (including any axes, titles,
-    border padding, etc.)
+    width: int | None = Override(default=600)
 
-    .. note::
-        This corresponds directly to the width of the HTML
-        canvas that will be used.
+    height: int | None = Override(default=600)
 
+    plot_width: int | None = Alias("width", help="""
+    The outer width of a plot, including any axes, titles, border padding, etc.
     """)
 
-    inner_width = Int(readonly=True, help="""
+    plot_height: int | None = Alias("height", help="""
+    The outer height of a plot, including any axes, titles, border padding, etc.
+    """)
+
+    frame_width = Nullable(Int, help="""
+    The width of a plot frame or the inner width of a plot, excluding any
+    axes, titles, border padding, etc.
+    """)
+
+    frame_height = Nullable(Int, help="""
+    The height of a plot frame or the inner height of a plot, excluding any
+    axes, titles, border padding, etc.
+    """)
+
+    inner_width = Readonly(Int, help="""
     This is the exact width of the plotting canvas, i.e. the width of
     the actual plot, without toolbars etc. Note this is computed in a
     web browser, so this property will work only in backends capable of
@@ -481,7 +591,7 @@ class Plot(LayoutDOM):
 
     """)
 
-    inner_height = Int(readonly=True, help="""
+    inner_height = Readonly(Int, help="""
     This is the exact height of the plotting canvas, i.e. the height of
     the actual plot, without toolbars etc. Note this is computed in a
     web browser, so this property will work only in backends capable of
@@ -492,7 +602,7 @@ class Plot(LayoutDOM):
 
     """)
 
-    layout_width = Int(readonly=True, help="""
+    outer_width = Readonly(Int, help="""
     This is the exact width of the layout, i.e. the height of
     the actual plot, with toolbars etc. Note this is computed in a
     web browser, so this property will work only in backends capable of
@@ -503,7 +613,7 @@ class Plot(LayoutDOM):
 
     """)
 
-    layout_height = Int(readonly=True, help="""
+    outer_height = Readonly(Int, help="""
     This is the exact height of the layout, i.e. the height of
     the actual plot, with toolbars etc. Note this is computed in a
     web browser, so this property will work only in backends capable of
@@ -514,19 +624,19 @@ class Plot(LayoutDOM):
 
     """)
 
-    background_props = Include(FillProps, help="""
-    The %s for the plot background style.
+    background_props = Include(ScalarFillProps, prefix="background", help="""
+    The {prop} for the plot background style.
     """)
 
     background_fill_color = Override(default='#ffffff')
 
-    border_props = Include(FillProps, help="""
-    The %s for the plot border style.
+    border_props = Include(ScalarFillProps, prefix="border", help="""
+    The {prop} for the plot border style.
     """)
 
     border_fill_color = Override(default='#ffffff')
 
-    min_border_top = Int(help="""
+    min_border_top = Nullable(Int, help="""
     Minimum size in pixels of the padding region above the top of the
     central plot region.
 
@@ -536,7 +646,7 @@ class Plot(LayoutDOM):
 
     """)
 
-    min_border_bottom = Int(help="""
+    min_border_bottom = Nullable(Int, help="""
     Minimum size in pixels of the padding region below the bottom of
     the central plot region.
 
@@ -546,7 +656,7 @@ class Plot(LayoutDOM):
 
     """)
 
-    min_border_left = Int(help="""
+    min_border_left = Nullable(Int, help="""
     Minimum size in pixels of the padding region to the left of
     the central plot region.
 
@@ -556,7 +666,7 @@ class Plot(LayoutDOM):
 
     """)
 
-    min_border_right = Int(help="""
+    min_border_right = Nullable(Int, help="""
     Minimum size in pixels of the padding region to the right of
     the central plot region.
 
@@ -566,27 +676,17 @@ class Plot(LayoutDOM):
 
     """)
 
-    min_border = Int(5, help="""
+    min_border = Nullable(Int, default=5, help="""
     A convenience property to set all all the ``min_border_X`` properties
     to the same value. If an individual border property is explicitly set,
     it will override ``min_border``.
     """)
 
-    h_symmetry = Bool(True, help="""
-    Whether the total horizontal padding on both sides of the plot will
-    be made equal (the left or right padding amount, whichever is larger).
-    """)
-
-    v_symmetry = Bool(False, help="""
-    Whether the total vertical padding on both sides of the plot will
-    be made equal (the top or bottom padding amount, whichever is larger).
-    """)
-
     lod_factor = Int(10, help="""
     Decimation factor to use when applying level-of-detail decimation.
     """)
 
-    lod_threshold = Int(2000, help="""
+    lod_threshold = Nullable(Int, default=2000, help="""
     A number of data points, above which level-of-detail downsampling may
     be performed by glyph renderers. Set to ``None`` to disable any
     level-of-detail downsampling.
@@ -615,7 +715,7 @@ class Plot(LayoutDOM):
     match_aspect = Bool(default=False, help="""
     Specify the aspect ratio behavior of the plot. Aspect ratio is defined as
     the ratio of width over height. This property controls whether Bokeh should
-    attempt the match the (width/height) of *data space* to the (width/height)
+    attempt to match the (width/height) of *data space* to the (width/height)
     in pixels of *screen space*.
 
     Default is ``False`` which indicates that the *data* aspect ratio and the
@@ -632,7 +732,7 @@ class Plot(LayoutDOM):
 
     .. warning::
         This setting is incompatible with linking dataranges across multiple
-        plots. Doing so may result in undefined behaviour.
+        plots. Doing so may result in undefined behavior.
     """)
 
     aspect_scale = Float(default=1, help="""
@@ -647,3 +747,112 @@ class Plot(LayoutDOM):
     .. note::
         This setting only takes effect if ``match_aspect`` is set to ``True``.
     """)
+
+    reset_policy = Enum(ResetPolicy, default="standard", help="""
+    How a plot should respond to being reset. By deafult, the standard actions
+    are to clear any tool state history, return plot ranges to their original
+    values, undo all selections, and emit a ``Reset`` event. If customization
+    is desired, this property may be set to ``"event_only"``, which will
+    suppress all of the actions except the Reset event.
+    """)
+
+    # XXX: override LayoutDOM's definitions because of plot_{width,height}.
+    @error(FIXED_SIZING_MODE)
+    def _check_fixed_sizing_mode(self):
+        pass
+
+    @error(FIXED_WIDTH_POLICY)
+    def _check_fixed_width_policy(self):
+        pass
+
+    @error(FIXED_HEIGHT_POLICY)
+    def _check_fixed_height_policy(self):
+        pass
+
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
+def _check_conflicting_kwargs(a1, a2, kwargs):
+    if a1 in kwargs and a2 in kwargs:
+        raise ValueError("Conflicting properties set on plot: %r and %r" % (a1, a2))
+
+class _list_attr_splat(list):
+    def __setattr__(self, attr, value):
+        for x in self:
+            setattr(x, attr, value)
+    def __getattribute__(self, attr):
+        if attr in dir(list):
+            return list.__getattribute__(self, attr)
+        if len(self) == 0:
+            raise AttributeError("Trying to access %r attribute on an empty 'splattable' list" % attr)
+        if len(self) == 1:
+            return getattr(self[0], attr)
+        try:
+            return _list_attr_splat([getattr(x, attr) for x in self])
+        except Exception:
+            raise AttributeError("Trying to access %r attribute on a 'splattable' list, but list items have no %r attribute" % (attr, attr))
+
+    def __dir__(self):
+        if len({type(x) for x in self}) == 1:
+            return dir(self[0])
+        else:
+            return dir(self)
+
+_LEGEND_EMPTY_WARNING = """
+You are attempting to set `plot.legend.%s` on a plot that has zero legends added, this will have no effect.
+
+Before legend properties can be set, you must add a Legend explicitly, or call a glyph method with a legend parameter set.
+"""
+
+class _legend_attr_splat(_list_attr_splat):
+    def __setattr__(self, attr, value):
+        if not len(self):
+            warnings.warn(_LEGEND_EMPTY_WARNING % attr)
+        return super().__setattr__(attr, value)
+
+def _select_helper(args, kwargs):
+    """ Allow flexible selector syntax.
+
+    Returns:
+        dict
+
+    """
+    if len(args) > 1:
+        raise TypeError("select accepts at most ONE positional argument.")
+
+    if len(args) > 0 and len(kwargs) > 0:
+        raise TypeError("select accepts EITHER a positional argument, OR keyword arguments (not both).")
+
+    if len(args) == 0 and len(kwargs) == 0:
+        raise TypeError("select requires EITHER a positional argument, OR keyword arguments.")
+
+    if args:
+        arg = args[0]
+        if isinstance(arg, dict):
+            selector = arg
+        elif isinstance(arg, str):
+            selector = dict(name=arg)
+        elif isinstance(arg, type) and issubclass(arg, Model):
+            selector = {"type": arg}
+        else:
+            raise TypeError("selector must be a dictionary, string or plot object.")
+
+    elif 'selector' in kwargs:
+        if len(kwargs) == 1:
+            selector = kwargs['selector']
+        else:
+            raise TypeError("when passing 'selector' keyword arg, not other keyword args may be present")
+
+    else:
+        selector = kwargs
+
+    return selector
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
