@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -12,26 +12,34 @@ of plots.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
+from __future__ import annotations
 
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
-# Standard library imports
-
-# External imports
-
 # Bokeh imports
 from ..core.enums import LatLon
 from ..core.has_props import abstract
-from ..core.properties import Enum, Float, Instance, Int, Override, Seq
+from ..core.properties import (
+    Auto,
+    Either,
+    Enum,
+    Float,
+    Instance,
+    Int,
+    NonNullable,
+    Nullable,
+    Override,
+    Seq,
+)
 from ..core.validation import error
 from ..core.validation.errors import MISSING_MERCATOR_DIMENSION
 from ..model import Model
+from .mappers import ScanningColorMapper
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -39,6 +47,7 @@ from ..model import Model
 
 __all__ = (
     'Ticker',
+    'BinnedTicker',
     'ContinuousTicker',
     'FixedTicker',
     'AdaptiveTicker',
@@ -124,7 +133,7 @@ class AdaptiveTicker(ContinuousTicker):
     The smallest allowable interval between two adjacent ticks.
     """)
 
-    max_interval = Float(help="""
+    max_interval = Nullable(Float, help="""
     The largest allowable interval between two adjacent ticks.
 
     .. note::
@@ -155,7 +164,7 @@ class SingleIntervalTicker(ContinuousTicker):
 
     '''
 
-    interval = Float(help="""
+    interval = NonNullable(Float, help="""
     The interval between adjacent ticks.
     """)
 
@@ -202,7 +211,7 @@ class MercatorTicker(BasicTicker):
 
     '''
 
-    dimension = Enum(LatLon, default=None, help="""
+    dimension = Nullable(Enum(LatLon), help="""
     Specify whether to generate ticks for Latitude or Longitude.
 
     Projected coordinates are not separable, computing Latitude and Longitude
@@ -279,6 +288,18 @@ class DatetimeTicker(CompositeTicker):
         YearsTicker(),
     ])
 
+class BinnedTicker(Ticker):
+    """Ticker that aligns ticks exactly at bin boundaries of a scanning color mapper. """
+
+    mapper = Instance(ScanningColorMapper, help="""
+    A scanning color mapper (e.g. ``EqHistColorMapper``) to use.
+    """)
+
+    num_major_ticks = Either(Int, Auto, default=8, help="""
+    The number of major tick positions to show or "auto" to use the
+    number of bins provided by the mapper.
+    """)
+
 #-----------------------------------------------------------------------------
 # Dev API
 #-----------------------------------------------------------------------------
