@@ -11,6 +11,8 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -18,12 +20,25 @@ log = logging.getLogger(__name__)
 # Imports
 #-----------------------------------------------------------------------------
 
+# Standard library imports
+from typing import TYPE_CHECKING, Callable, List as TList
+
 # Bokeh imports
 from ...core.has_props import abstract
-from ...core.properties import Bool, Int, List, Nullable, String
+from ...core.properties import (
+    Bool,
+    Enum,
+    Int,
+    List,
+    Nullable,
+    String,
+)
 from .buttons import ButtonLike
 from .widget import Widget
 
+if TYPE_CHECKING:
+    from ..callbacks import Callback
+
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
@@ -52,7 +67,7 @@ class AbstractGroup(Widget):
     List of text labels contained in this group.
     """)
 
-    def on_click(self, handler):
+    def on_click(self, handler: Callable[[int], None] | Callable[[TList[int]], None]) -> None:
         ''' Set up a handler for button check/radio box clicks including
         the selected indices.
 
@@ -65,7 +80,7 @@ class AbstractGroup(Widget):
         '''
         self.on_change('active', lambda attr, old, new: handler(new))
 
-    def js_on_click(self, handler):
+    def js_on_click(self, handler: Callback) -> None:
         ''' Set up a handler for button check/radio box clicks including the selected indices. '''
         self.js_on_change('active', handler)
 
@@ -75,6 +90,10 @@ class ButtonGroup(AbstractGroup, ButtonLike):
 
     '''
 
+    orientation = Enum("horizontal", "vertical", help="""
+    Orient the button group either horizontally (default) or vertically.
+    """)
+
 @abstract
 class Group(AbstractGroup):
     ''' Abstract base class for groups with items rendered as check/radio
