@@ -11,6 +11,8 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -216,7 +218,7 @@ class Spinner(NumericInput):
 
     '''
 
-    def __init__(self, **kwargs):
+    def __init__(self, **kwargs) -> None:
         if "value" in kwargs and "value_throttled" not in kwargs:
             kwargs["value_throttled"] = kwargs["value"]
 
@@ -392,6 +394,11 @@ class MultiChoice(InputWidget):
     The number of choices that will be rendered in the dropdown.
     """)
 
+    search_option_limit = Nullable(Int, help="""
+    The number of choices that will be rendered in the dropdown
+    when search string is entered.
+    """)
+
     placeholder = Nullable(String, help="""
     A string that is displayed if not item is added.
     """)
