@@ -17,6 +17,8 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -26,7 +28,12 @@ log = logging.getLogger(__name__)
 
 # Bokeh imports
 from ...core.has_props import abstract
-from ...core.properties import Any, Bool, Dict, String
+from ...core.properties import (
+    Any,
+    Bool,
+    Dict,
+    String,
+)
 from .widget import Widget
 
 #-----------------------------------------------------------------------------
@@ -50,6 +57,10 @@ class Markup(Widget):
 
     Markups include e.g., ``<div>``, ``<p>``, and ``<pre>``.
 
+    Content can be interpreted as `TeX and LaTeX input`_ when rendering as HTML.
+    TeX/LaTeX processing can be disabled by setting ``disable_math`` to True.
+
+    .. _`TeX and LaTeX input`: https://docs.mathjax.org/en/latest/basic/mathematics.html#tex-and-latex-input
     '''
 
     text = String(default="", help="""
@@ -69,6 +80,10 @@ class Markup(Widget):
     Raw CSS style declaration. Note this may be web browser dependent.
     """)
 
+    disable_math = Bool(False, help="""
+    Whether the contents should not be processed as TeX/LaTeX input.
+    """)
+
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
@@ -86,14 +101,13 @@ class Div(Markup):
     ''' A block (div) of text.
 
     This Bokeh model corresponds to an HTML ``<div>`` element.
-
     '''
 
     __example__ = "sphinx/source/docs/user_guide/examples/interaction_div.py"
 
     render_as_text = Bool(False, help="""
     Whether the contents should be rendered as raw text or as interpreted HTML.
-    The default value is ``False``, meaning contents are rendered as HTML.
+    The default value is False, meaning contents are rendered as HTML.
     """)
 
 class PreText(Paragraph):
