@@ -16,6 +16,8 @@ in the browser,  or with python callbacks that execute on a Bokeh server.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -25,7 +27,7 @@ log = logging.getLogger(__name__)
 
 # Bokeh imports
 from ...core.has_props import abstract
-from ...core.properties import Enum, Int, Override
+from ...core.properties import Int, Override
 from ..layouts import LayoutDOM
 
 #-----------------------------------------------------------------------------
@@ -50,12 +52,6 @@ class Widget(LayoutDOM):
 
     '''
 
-    orientation = Enum("horizontal", "vertical", help="""
-    Orient the widget either horizontally (default) or vertically.
-
-    Note that not all widgets support vertical orientation.
-    """)
-
     default_size = Int(default=300, help="""
     The default size (width or height) in the dominating dimension.
 
