@@ -12,6 +12,8 @@ message objects that can be processed.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -19,8 +21,22 @@ log = logging.getLogger(__name__)
 # Imports
 #-----------------------------------------------------------------------------
 
+# Standard library imports
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    List,
+    Union,
+    cast,
+)
+
 # Bokeh imports
 from .exceptions import ValidationError
+from .message import BufferHeader, Message
+
+if TYPE_CHECKING:
+    from . import Protocol
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -38,6 +54,8 @@ __all__ = (
 # Dev API
 #-----------------------------------------------------------------------------
 
+Fragment = Union[str, bytes]
+
 class Receiver:
     ''' Receive wire message fragments and assemble complete Bokeh server
     message objects.
@@ -83,7 +101,13 @@ class Receiver:
 
     '''
 
-    def __init__(self, protocol):
+    _current_consumer: Callable[[Receiver, Fragment], None]
+    _fragments: List[Fragment]
+    _message: Message[Any] | None
+    _buf_header: BufferHeader | None
+    _partial: Message[Any] | None
+
+    def __init__(self, protocol: Protocol) -> None:
         ''' Configure a Receiver with a specific Bokeh protocol.
 
         Args:
@@ -92,11 +116,12 @@ class Receiver:
                 fragments.
         '''
         self._protocol = protocol
-        self._current_consumer = self._HEADER
+        self._current_consumer = self._HEADER  # type: ignore[assignment] # https://github.com/python/mypy/issues/2427
         self._message = None
+        self._partial = None
         self._buf_header = None
 
-    async def consume(self, fragment):
+    async def consume(self, fragment: Fragment) -> Message[Any]|None:
         ''' Consume individual protocol message fragments.
 
         Args:
@@ -109,53 +134,53 @@ class Receiver:
         self._current_consumer(fragment)
         return self._message
 
-    def _HEADER(self, fragment):
-        self._assume_text(fragment)
+    def _HEADER(self, fragment: Fragment) -> None:
         self._message = None
         self._partial = None
-        self._fragments = [fragment]
-        self._current_consumer = self._METADATA
+        self._fragments = [self._assume_text(fragment)]
+        self._current_consumer = self._METADATA  # type: ignore[assignment] # https://github.com/python/mypy/issues/2427
 
-    def _METADATA(self, fragment):
-        self._assume_text(fragment)
-        self._fragments.append(fragment)
-        self._current_consumer = self._CONTENT
+    def _METADATA(self, fragment: Fragment) -> None:
+        metadata = self._assume_text(fragment)
+        self._fragments.append(metadata)
+        self._current_consumer = self._CONTENT  # type: ignore[assignment] # https://github.com/python/mypy/issues/2427
 
-    def _CONTENT(self, fragment):
-        self._assume_text(fragment)
-        self._fragments.append(fragment)
+    def _CONTENT(self, fragment: Fragment) -> None:
+        content = self._assume_text(fragment)
+        self._fragments.append(content)
 
-        header_json, metadata_json, content_json = self._fragments[:3]
+        header_json, metadata_json, content_json = [self._assume_text(x) for x in self._fragments[:3]]
 
         self._partial = self._protocol.assemble(header_json, metadata_json, content_json)
 
         self._check_complete()
 
-    def _BUFFER_HEADER(self, fragment):
-        self._assume_text(fragment)
-        self._buf_header = fragment
-        self._current_consumer = self._BUFFER_PAYLOAD
+    def _BUFFER_HEADER(self, fragment: Fragment) -> None:
+        self._buf_header = self._assume_text(fragment)
+        self._current_consumer = self._BUFFER_PAYLOAD  # type: ignore[assignment] # https://github.com/python/mypy/issues/2427
 
-    def _BUFFER_PAYLOAD(self, fragment):
-        self._assume_binary(fragment)
-        self._partial.assemble_buffer(self._buf_header, fragment)
+    def _BUFFER_PAYLOAD(self, fragment: Fragment) -> None:
+        payload = self._assume_binary(fragment)
+        cast(Message[Any], self._partial).assemble_buffer(cast(BufferHeader, self._buf_header), payload)
 
         self._check_complete()
 
-    def _check_complete(self):
-        if self._partial.complete:
+    def _check_complete(self) -> None:
+        if self._partial and self._partial.complete:
             self._message = self._partial
-            self._current_consumer = self._HEADER
+            self._current_consumer = self._HEADER  # type: ignore[assignment] # https://github.com/python/mypy/issues/2427
         else:
-            self._current_consumer = self._BUFFER_HEADER
+            self._current_consumer = self._BUFFER_HEADER  # type: ignore[assignment] # https://github.com/python/mypy/issues/2427
 
-    def _assume_text(self, fragment):
+    def _assume_text(self, fragment: Fragment) -> str:
         if not isinstance(fragment, str):
-            raise ValidationError("expected text fragment but received binary fragment for %s" % (self._current_consumer.__name__))
+            raise ValidationError(f"expected text fragment but received binary fragment for {self._current_consumer.__name__}")
+        return fragment
 
-    def _assume_binary(self, fragment):
+    def _assume_binary(self, fragment: Fragment) -> bytes:
         if not isinstance(fragment, bytes):
-            raise ValidationError("expected binary fragment but received text fragment for %s" % (self._current_consumer.__name__))
+            raise ValidationError(f"expected binary fragment but received text fragment for {self._current_consumer.__name__}")
+        return fragment
 
 #-----------------------------------------------------------------------------
 # Private API
