@@ -4,13 +4,21 @@
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
-'''
+''' Route data (including altitude) for a bike race in Eastern Europe.
+
+This module contains one pandas Dataframe: ``obiszow_mtb_xcm``.
+
+.. rubric:: ``obiszow_mtb_xcm``
+
+:bokeh-dataframe:`bokeh.sampledata.mtb.obiszow_mtb_xcm`
 
 '''
 
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
