@@ -4,13 +4,21 @@
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
-'''
+''' Time series of historical average sea surface temperatures.
+
+This module contains one pandas Dataframe: ``sea_surface_temperature``.
+
+.. rubric:: ``sea_surface_temperature``
+
+:bokeh-dataframe:`bokeh.sampledata.sea_surface_temperature.sea_surface_temperature`
 
 '''
 
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
