@@ -11,6 +11,8 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -18,17 +20,37 @@ log = logging.getLogger(__name__)
 # Imports
 #-----------------------------------------------------------------------------
 
+# Standard library imports
+import weakref
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Awaitable,
+    Callable,
+    Dict,
+    Iterable,
+    List,
+)
+
 # External imports
 from tornado import gen
 
+if TYPE_CHECKING:
+    from tornado.httputil import HTTPServerRequest
+    from tornado.ioloop import IOLoop
+
 # Bokeh imports
 from ..application.application import ServerContext, SessionContext
 from ..document import Document
 from ..protocol.exceptions import ProtocolError
 from ..util.token import get_token_payload
-from ..util.tornado import _CallbackGroup
 from .session import ServerSession
 
+if TYPE_CHECKING:
+    from ..application.application import Application
+    from ..core.types import ID
+    from ..util.token import TokenPayload
+
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
@@ -52,40 +74,30 @@ __all__ = (
 #-----------------------------------------------------------------------------
 
 class BokehServerContext(ServerContext):
-    def __init__(self, application_context):
-        self.application_context = application_context
-        self._callbacks = _CallbackGroup(self.application_context.io_loop)
-
-    def _remove_all_callbacks(self):
-        self._callbacks.remove_all_callbacks()
+    def __init__(self, application_context: ApplicationContext) -> None:
+        self._application_context = weakref.ref(application_context)
 
     @property
-    def sessions(self):
-        result = []
-        for session in self.application_context.sessions:
-            result.append(session)
+    def application_context(self) -> ApplicationContext | None:
+        return self._application_context()
+
+    @property
+    def sessions(self) -> List[ServerSession]:
+        result: List[ServerSession] = []
+        context = self.application_context
+        if context:
+            for session in context.sessions:
+                result.append(session)
         return result
 
-    def add_next_tick_callback(self, callback):
-        return self._callbacks.add_next_tick_callback(callback)
-
-    def remove_next_tick_callback(self, callback_id):
-        self._callbacks.remove_next_tick_callback(callback_id)
-
-    def add_timeout_callback(self, callback, timeout_milliseconds):
-        return self._callbacks.add_timeout_callback(callback, timeout_milliseconds)
-
-    def remove_timeout_callback(self, callback_id):
-        self._callbacks.remove_timeout_callback(callback_id)
-
-    def add_periodic_callback(self, callback, period_milliseconds):
-        return self._callbacks.add_periodic_callback(callback, period_milliseconds)
-
-    def remove_periodic_callback(self, callback_id):
-        self._callbacks.remove_periodic_callback(callback_id)
-
 class BokehSessionContext(SessionContext):
-    def __init__(self, session_id, server_context, document, logout_url=None):
+
+    _session: ServerSession | None
+    _request: _RequestProxy | None
+    _token: str | None
+
+    def __init__(self, session_id: ID, server_context: ServerContext,
+            document: Document, logout_url: str | None = None) -> None:
         self._document = document
         self._session = None
         self._logout_url = logout_url
@@ -94,10 +106,10 @@ class BokehSessionContext(SessionContext):
         self._request = None
         self._token = None
 
-    def _set_session(self, session):
+    def _set_session(self, session: ServerSession) -> None:
         self._session = session
 
-    async def with_locked_document(self, func):
+    async def with_locked_document(self, func: Callable[[Document], Awaitable[None]]) -> None:
         if self._session is None:
             # this means we are in on_session_created, so no locking yet,
             # we have exclusive access
@@ -106,7 +118,7 @@ class BokehSessionContext(SessionContext):
             await self._session.with_document_locked(func, self._document)
 
     @property
-    def destroyed(self):
+    def destroyed(self) -> bool:
         if self._session is None:
             # this means we are in on_session_created
             return False
@@ -114,19 +126,20 @@ class BokehSessionContext(SessionContext):
             return self._session.destroyed
 
     @property
-    def logout_url(self):
+    def logout_url(self) -> str | None:
         return self._logout_url
 
     @property
-    def request(self):
+    def request(self) -> _RequestProxy | None:
         return self._request
 
     @property
-    def token_payload(self):
+    def token_payload(self) -> TokenPayload:
+        assert self._token is not None
         return get_token_payload(self._token)
 
     @property
-    def session(self):
+    def session(self) -> ServerSession | None:
         return self._session
 
 
@@ -134,55 +147,61 @@ class ApplicationContext:
     ''' Server-side holder for ``bokeh.application.Application`` plus any associated data.
         This holds data that's global to all sessions, while ``ServerSession`` holds
         data specific to an "instance" of the application.
+
+    .. autoclasstoc::
+
     '''
 
-    def __init__(self, application, io_loop=None, url=None, logout_url=None):
+    _sessions: Dict[ID, ServerSession]
+    _pending_sessions: Dict[ID, gen.Future[ServerSession]]
+    _session_contexts: Dict[ID, SessionContext]
+    _server_context: BokehServerContext
+
+    def __init__(self, application: Application, io_loop: IOLoop | None = None,
+            url: str | None = None, logout_url: str | None = None):
         self._application = application
         self._loop = io_loop
-        self._sessions = dict()
-        self._pending_sessions = dict()
-        self._session_contexts = dict()
-        self._server_context = None
+        self._sessions = {}
+        self._pending_sessions = {}
+        self._session_contexts = {}
+        self._server_context = BokehServerContext(self)
         self._url = url
         self._logout_url = logout_url
 
     @property
-    def io_loop(self):
+    def io_loop(self) -> IOLoop | None:
         return self._loop
 
     @property
-    def application(self):
+    def application(self) -> Application:
         return self._application
 
     @property
-    def url(self):
+    def url(self) -> str | None:
         return self._url
 
     @property
-    def server_context(self):
-        if self._server_context is None:
-            self._server_context = BokehServerContext(self)
+    def server_context(self) -> BokehServerContext:
         return self._server_context
 
     @property
-    def sessions(self):
+    def sessions(self) -> Iterable[ServerSession]:
         return self._sessions.values()
 
-    def run_load_hook(self):
+    def run_load_hook(self) -> None:
         try:
             self._application.on_server_loaded(self.server_context)
         except Exception as e:
-            log.error("Error in server loaded hook %r", e, exc_info=True)
+            log.error(f"Error in server loaded hook {e!r}", exc_info=True)
 
-    def run_unload_hook(self):
+    def run_unload_hook(self) -> None:
         try:
             self._application.on_server_unloaded(self.server_context)
         except Exception as e:
-            log.error("Error in server unloaded hook %r", e, exc_info=True)
+            log.error(f"Error in server unloaded hook {e!r}", exc_info=True)
 
-        self.server_context._remove_all_callbacks()
-
-    async def create_session_if_needed(self, session_id, request=None, token=None):
+    async def create_session_if_needed(self, session_id: ID, request: HTTPServerRequest | None = None,
+            token: str | None = None) -> ServerSession:
         # this is because empty session_ids would be "falsey" and
         # potentially open up a way for clients to confuse us
         if len(session_id) == 0:
@@ -200,6 +219,12 @@ class ApplicationContext:
                                                   logout_url=self._logout_url)
             if request is not None:
                 payload = get_token_payload(token) if token else {}
+                if ('cookies' in payload and 'headers' in payload
+                    and not 'Cookie' in payload['headers']):
+                    # Restore Cookie header from cookies dictionary
+                    payload['headers']['Cookie'] = '; '.join([
+                        f'{k}={v}' for k, v in payload['cookies'].items()
+                    ])
                 # using private attr so users only have access to a read-only property
                 session_context._request = _RequestProxy(request,
                                                          cookies=payload.get('cookies'),
@@ -208,7 +233,7 @@ class ApplicationContext:
 
             # expose the session context to the document
             # use the _attribute to set the public property .session_context
-            doc._session_context = session_context
+            doc._session_context = weakref.ref(session_context)
 
             try:
                 await self._application.on_session_created(session_context)
@@ -235,14 +260,14 @@ class ApplicationContext:
 
         return session
 
-    def get_session(self, session_id):
+    def get_session(self, session_id: ID) -> ServerSession:
         if session_id in self._sessions:
             session = self._sessions[session_id]
             return session
         else:
             raise ProtocolError("No such session " + session_id)
 
-    async def _discard_session(self, session, should_discard):
+    async def _discard_session(self, session: ServerSession, should_discard: Callable[[ServerSession], bool]) -> None:
         if session.connection_count > 0:
             raise RuntimeError("Should not be discarding a session with open connections")
         log.debug("Discarding session %r last in use %r milliseconds ago", session.id, session.milliseconds_since_last_unsubscribe)
@@ -251,7 +276,7 @@ class ApplicationContext:
 
         # session.destroy() wants the document lock so it can shut down the document
         # callbacks.
-        def do_discard():
+        def do_discard() -> None:
             # while we awaited for the document lock, the discard-worthiness of the
             # session may have changed.
             # However, since we have the document lock, our own lock will cause the
@@ -261,9 +286,9 @@ class ApplicationContext:
                 session.destroy()
                 del self._sessions[session.id]
                 del self._session_contexts[session.id]
-                log.trace("Session %r was successfully discarded", session.id)
+                log.trace(f"Session {session.id!r} was successfully discarded")
             else:
-                log.warning("Session %r was scheduled to discard but came back to life", session.id)
+                log.warning(f"Session {session.id!r} was scheduled to discard but came back to life")
         await session.with_document_locked(do_discard)
 
         # session lifecycle hooks are supposed to be called outside the document lock,
@@ -276,13 +301,13 @@ class ApplicationContext:
 
         return None
 
-    async def _cleanup_sessions(self, unused_session_linger_milliseconds):
-        def should_discard_ignoring_block(session):
+    async def _cleanup_sessions(self, unused_session_linger_milliseconds: int) -> None:
+        def should_discard_ignoring_block(session: ServerSession) -> bool:
             return session.connection_count == 0 and \
                 (session.milliseconds_since_last_unsubscribe > unused_session_linger_milliseconds or \
                  session.expiration_requested)
         # build a temp list to avoid trouble from self._sessions changes
-        to_discard = []
+        to_discard: List[ServerSession] = []
         for session in self._sessions.values():
             if should_discard_ignoring_block(session) and not session.expiration_blocked:
                 to_discard.append(session)
@@ -301,18 +326,24 @@ class ApplicationContext:
 #-----------------------------------------------------------------------------
 
 class _RequestProxy:
-    def __init__(self, request, cookies=None, headers=None):
+
+    _arguments: Dict[str, List[bytes]]
+    _cookies: Dict[str, str]
+    _headers: Dict[str, str | List[str]]
+
+    def __init__(self, request: HTTPServerRequest, cookies: Dict[str, str] | None = None,
+            headers: Dict[str, str | List[str]] | None = None) -> None:
         self._request = request
 
         arguments = dict(request.arguments)
-        if 'bokeh-session-id' in arguments: del arguments['bokeh-session-id']
+        if 'bokeh-session-id' in arguments:
+            del arguments['bokeh-session-id']
         self._arguments = arguments
         if cookies is not None:
             self._cookies = cookies
         elif hasattr(request, 'cookies'):
             # Django cookies are plain strings, tornado cookies are objects with a value
-            self._cookies = {k: v if isinstance(v, str) else v.value
-                             for k, v in request.cookies.items()}
+            self._cookies = {k: v if isinstance(v, str) else v.value for k, v in request.cookies.items()}
         else:
             self._cookies = {}
 
@@ -324,18 +355,18 @@ class _RequestProxy:
             self._headers = {}
 
     @property
-    def arguments(self):
+    def arguments(self) -> Dict[str, List[bytes]]:
         return self._arguments
 
     @property
-    def cookies(self):
+    def cookies(self) -> Dict[str, str]:
         return self._cookies
 
     @property
-    def headers(self):
+    def headers(self) -> Dict[str, str | List[str]]:
         return self._headers
 
-    def __getattr__(self, name):
+    def __getattr__(self, name: str) -> Any:
         if not name.startswith("_"):
             val = getattr(self._request, name, None)
             if val is not None:
