@@ -12,6 +12,8 @@ components in ``bokeh.server``.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -19,6 +21,17 @@ log = logging.getLogger(__name__)
 # Imports
 #-----------------------------------------------------------------------------
 
+# Standard library imports
+from typing import (
+    TYPE_CHECKING,
+    List,
+    Sequence,
+    Tuple,
+)
+
+if TYPE_CHECKING:
+    from socket import socket
+
 # External imports
 from tornado import netutil
 
@@ -37,7 +50,7 @@ __all__ = (
 # General API
 #-----------------------------------------------------------------------------
 
-def bind_sockets(address, port):
+def bind_sockets(address: str | None, port: int) -> Tuple[List[socket], int]:
     ''' Bind a socket to a port on an address.
 
     Args:
@@ -66,7 +79,7 @@ def bind_sockets(address, port):
         assert actual_port == port
     return ss, actual_port
 
-def check_allowlist(host, allowlist):
+def check_allowlist(host: str, allowlist: Sequence[str]) -> bool:
     ''' Check a given request host against a allowlist.
 
     Args:
@@ -92,7 +105,7 @@ def check_allowlist(host, allowlist):
 
     return any(match_host(host, pattern) for pattern in allowlist)
 
-def create_hosts_allowlist(host_list, port):
+def create_hosts_allowlist(host_list: Sequence[str] | None, port: int) -> List[str]:
     '''
 
     This allowlist can be used to restrict websocket or other connections to
@@ -126,7 +139,7 @@ def create_hosts_allowlist(host_list, port):
     if not host_list:
         return ['localhost:' + str(port)]
 
-    hosts = []
+    hosts: List[str] = []
     for host in host_list:
         if '*' in host:
             log.warning(
@@ -155,7 +168,7 @@ def create_hosts_allowlist(host_list, port):
             raise ValueError("Invalid host value: %s" % host)
     return hosts
 
-def match_host(host, pattern):
+def match_host(host: str, pattern: str) -> bool:
     ''' Match a host string against a pattern
 
     Args:
@@ -205,28 +218,26 @@ def match_host(host, pattern):
         False
 
     '''
+    host_port: str | None = None
     if ':' in host:
         host, host_port = host.rsplit(':', 1)
-    else:
-        host_port = None
 
+    pattern_port: str | None = None
     if ':' in pattern:
         pattern, pattern_port = pattern.rsplit(':', 1)
         if pattern_port == '*':
             pattern_port = None
-    else:
-        pattern_port = None
 
     if pattern_port is not None and host_port != pattern_port:
         return False
 
-    host = host.split('.')
-    pattern = pattern.split('.')
+    host_parts = host.split('.')
+    pattern_parts = pattern.split('.')
 
-    if len(pattern) > len(host):
+    if len(pattern_parts) > len(host_parts):
         return False
 
-    for h, p in zip(host, pattern):
+    for h, p in zip(host_parts, pattern_parts):
         if h == p or p == '*':
             continue
         else:
