@@ -40,6 +40,8 @@ the above usage yields the output:
 # -----------------------------------------------------------------------------
 # Boilerplate
 # -----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging  # isort:skip
 
 log = logging.getLogger(__name__)
@@ -57,9 +59,11 @@ from docutils.parsers.rst.directives import unchanged
 from sphinx.errors import SphinxError
 
 # Bokeh imports
+from bokeh.core.property._sphinx import type_link
 from bokeh.util.options import Options
 
 # Bokeh imports
+from . import PARALLEL_SAFE
 from .bokeh_directive import BokehDirective, py_sig_re
 from .templates import OPTIONS_DETAIL
 
@@ -118,7 +122,7 @@ class BokehOptionsDirective(BokehDirective):
             opts.append(
                 dict(
                     name=prop_name,
-                    type=descriptor.property._sphinx_type(),
+                    type=type_link(descriptor.property),
                     default=repr(descriptor.instance_default(options_obj)),
                     doc="" if descriptor.__doc__ is None else textwrap.dedent(descriptor.__doc__.rstrip()),
                 )
@@ -126,13 +130,14 @@ class BokehOptionsDirective(BokehDirective):
 
         rst_text = OPTIONS_DETAIL.render(opts=opts)
 
-        return self._parse(rst_text, "<bokeh-options>")
+        return self.parse(rst_text, "<bokeh-options>")
 
 
 def setup(app):
     """ Required Sphinx extension setup function. """
     app.add_directive_to_domain("py", "bokeh-options", BokehOptionsDirective)
 
+    return PARALLEL_SAFE
 
 # -----------------------------------------------------------------------------
 # Private API
