@@ -43,6 +43,8 @@ in conjunction with the :ref:`bokeh.sphinxext.bokeh_autodoc` extension.
 # -----------------------------------------------------------------------------
 # Boilerplate
 # -----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging  # isort:skip
 
 log = logging.getLogger(__name__)
@@ -61,9 +63,11 @@ from docutils.parsers.rst.directives import unchanged
 from sphinx.errors import SphinxError
 
 # Bokeh imports
+from bokeh.core.property._sphinx import type_link
 from bokeh.util.warnings import BokehDeprecationWarning
 
 # Bokeh imports
+from . import PARALLEL_SAFE
 from .bokeh_directive import BokehDirective
 from .templates import PROP_DETAIL
 
@@ -123,17 +127,18 @@ class BokehPropDirective(BokehDirective):
             name=prop_name,
             module=self.options["module"],
             default=repr(descriptor.instance_default(model_obj)),
-            type_info=descriptor.property._sphinx_type(),
+            type_info=type_link(descriptor.property),
             doc="" if descriptor.__doc__ is None else textwrap.dedent(descriptor.__doc__),
         )
 
-        return self._parse(rst_text, "<bokeh-prop>")
+        return self.parse(rst_text, "<bokeh-prop>")
 
 
 def setup(app):
     """ Required Sphinx extension setup function. """
     app.add_directive_to_domain("py", "bokeh-prop", BokehPropDirective)
 
+    return PARALLEL_SAFE
 
 # -----------------------------------------------------------------------------
 # Private API
