@@ -11,8 +11,6 @@ subdirectory, and includes them in *reverse version order*. Typical usage:
 
 .. code-block:: rest
 
-    :tocdepth: 1
-
     .. toctree::
 
     .. bokeh-releases::
@@ -29,6 +27,8 @@ To avoid warnings about orphaned files, add the following to the Sphinx
 # -----------------------------------------------------------------------------
 # Boilerplate
 # -----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging  # isort:skip
 
 log = logging.getLogger(__name__)
@@ -42,6 +42,7 @@ from os import listdir
 from os.path import join
 
 # External imports
+from docutils import nodes
 from packaging.version import Version as V
 
 # Bokeh imports
@@ -49,7 +50,9 @@ from bokeh import __version__
 from bokeh.resources import get_sri_hashes_for_version
 
 # Bokeh imports
+from . import PARALLEL_SAFE
 from .bokeh_directive import BokehDirective
+from .templates import SRI_TABLE
 
 # -----------------------------------------------------------------------------
 # Globals and constants
@@ -65,65 +68,47 @@ __all__ = ("BokehReleases", "setup")
 # Dev API
 # -----------------------------------------------------------------------------
 
-_SRI_SECTION_PRE = """
+class sri_table(nodes.General, nodes.Element):
 
-**Sub-Resource Integrity Hashes**
+    @staticmethod
+    def visit_html(visitor, node):
+        version = node["version"]
+        table = node["table"]
+        visitor.body.append(f'<button type="button" class="bk-collapsible">Table of SRI Hashes for version {version}</button>')
+        visitor.body.append('<div class="bk-collapsible-content">')
+        visitor.body.append(SRI_TABLE.render(table=table))
+        visitor.body.append("</div>")
+        raise nodes.SkipNode
 
-.. raw:: html
-
-    <div class="expander">
-        <a href="javascript:void(0)" class="expander-trigger expander-hidden">Table of SRI Hashes for version %s</a>
-        <div class="expander-content">
-            <div style="font-size: small;">
-
-.. csv-table::
-    :widths: 20, 80
-    :header: "Filename", "Hash"
-
-"""
-
-_SRI_SECTION_POST = """
-.. raw:: html
-
-            </div>
-        </div>
-   </div>
-
-"""
+    html = visit_html.__func__, None
 
 
 class BokehReleases(BokehDirective):
     def run(self):
-        env = self.state.document.settings.env
-        app = env.app
-
         rst = []
 
-        versions = [x.rstrip(".rst") for x in listdir(join(app.srcdir, "docs", "releases")) if x.endswith(".rst")]
+        srcdir = self.env.app.srcdir
+        versions = [x.rstrip(".rst") for x in listdir(join(srcdir, "docs", "releases")) if x.endswith(".rst")]
         versions.sort(key=V, reverse=True)
 
         for v in versions:
-            rst_text = f".. include:: releases/{v}.rst"
+            rst += self.parse(f".. include:: releases/{v}.rst", "<bokeh-releases>")
             try:
                 hashes = get_sri_hashes_for_version(v)
-                rst_text += _SRI_SECTION_PRE % v  # TODO (bev) get rid of old style string substituion
-                for key, val in sorted(hashes.items()):
-                    rst_text += f"    ``{key}``, ``{val}``\n"
-                rst_text += _SRI_SECTION_POST
+                rst += [sri_table(version=v, table=sorted(hashes.items()))]
             except KeyError:
                 if v == __version__:
                     raise RuntimeError(f"Missing SRI Hash for full release version {v!r}")
 
-            entry = self._parse(rst_text, "<bokeh-releases>")
-            rst.extend(entry)
-
         return rst
 
 
 def setup(app):
     """ Required Sphinx extension setup function. """
     app.add_directive("bokeh-releases", BokehReleases)
+    app.add_node(sri_table, html=sri_table.html)
 
+    return PARALLEL_SAFE
 
 # -----------------------------------------------------------------------------
 # Private API
