@@ -21,6 +21,8 @@ This directive takes the name of a module attribute
 # -----------------------------------------------------------------------------
 # Boilerplate
 # -----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging  # isort:skip
 
 log = logging.getLogger(__name__)
@@ -41,6 +43,7 @@ from sphinx.errors import SphinxError
 from bokeh.settings import PrioritizedSetting, _Unset
 
 # Bokeh imports
+from . import PARALLEL_SAFE
 from .bokeh_directive import BokehDirective, py_sig_re
 from .templates import SETTINGS_DETAIL
 
@@ -104,13 +107,14 @@ class BokehSettingsDirective(BokehDirective):
             settings.append(setting)
 
         rst_text = SETTINGS_DETAIL.render(name=obj_name, module_name=module_name, settings=settings)
-        return self._parse(rst_text, "<bokeh-settings>")
+        return self.parse(rst_text, "<bokeh-settings>")
 
 
 def setup(app):
     """ Required Sphinx extension setup function. """
     app.add_directive_to_domain("py", "bokeh-settings", BokehSettingsDirective)
 
+    return PARALLEL_SAFE
 
 # -----------------------------------------------------------------------------
 # Private API
