@@ -39,6 +39,8 @@ The inline example code above produces the following output:
 # -----------------------------------------------------------------------------
 # Boilerplate
 # -----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging  # isort:skip
 
 log = logging.getLogger(__name__)
@@ -56,7 +58,7 @@ from docutils.parsers.rst.directives import unchanged
 from sphinx.directives.code import CodeBlock
 
 # Bokeh imports
-from .templates import CCB_EPILOGUE, CCB_PROLOGUE
+from . import PARALLEL_SAFE
 
 # -----------------------------------------------------------------------------
 # Globals and constants
@@ -65,8 +67,6 @@ from .templates import CCB_EPILOGUE, CCB_PROLOGUE
 __all__ = (
     "collapsible_code_block",
     "CollapsibleCodeBlock",
-    "html_depart_collapsible_code_block",
-    "html_visit_collapsible_code_block",
     "setup",
 )
 
@@ -80,7 +80,17 @@ __all__ = (
 
 
 class collapsible_code_block(nodes.General, nodes.Element):
-    pass
+
+    @staticmethod
+    def visit_html(visitor, node):
+        heading = node["heading"]
+        visitor.body.append(f"<details><summary>{heading}</summary>")
+
+    @staticmethod
+    def depart_html(visitor, _node):
+        visitor.body.append("</details>")
+
+    html = visit_html.__func__, depart_html.__func__
 
 
 class CollapsibleCodeBlock(CodeBlock):
@@ -89,40 +99,28 @@ class CollapsibleCodeBlock(CodeBlock):
     option_spec.update(heading=unchanged)
 
     def run(self):
-        env = self.state.document.settings.env
-
         rst_source = self.state_machine.node.document["source"]
         rst_filename = basename(rst_source)
 
-        serial_no = env.new_serialno("ccb")
-        target_id = f"{rst_filename}.ccb-{serial_no}"
-        target_id = target_id.replace(".", "-")
-        target_node = nodes.target("", "", ids=[target_id])
+        serial_no = self.env.new_serialno("ccb")
+        target_id = f"{rst_filename}.ccb-{serial_no}".replace(".", "-")
+        target = nodes.target("", "", ids=[target_id])
 
-        node = collapsible_code_block()
-        node["target_id"] = target_id
-        node["heading"] = self.options.get("heading", "Code")
+        block = collapsible_code_block(target_id=target_id, heading=self.options.get("heading", "Code"))
 
         cb = CodeBlock.run(self)
-        node.setup_child(cb[0])
-        node.children.append(cb[0])
+        block.setup_child(cb[0])
+        block.children.append(cb[0])
 
-        return [target_node, node]
-
-
-def html_visit_collapsible_code_block(self, node):
-    self.body.append(CCB_PROLOGUE.render(id=node["target_id"], heading=node["heading"]))
-
-
-def html_depart_collapsible_code_block(self, node):
-    self.body.append(CCB_EPILOGUE.render())
+        return [target, block]
 
 
 def setup(app):
     """ Required Sphinx extension setup function. """
-    app.add_node(collapsible_code_block, html=(html_visit_collapsible_code_block, html_depart_collapsible_code_block))
+    app.add_node(collapsible_code_block, html=collapsible_code_block.html)
     app.add_directive("collapsible-code-block", CollapsibleCodeBlock)
 
+    return PARALLEL_SAFE
 
 # -----------------------------------------------------------------------------
 # Private API
