@@ -1,68 +1,74 @@
-#-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
+# -----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
-#-----------------------------------------------------------------------------
-'''
+# -----------------------------------------------------------------------------
+"""
 
-'''
+"""
 
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 # Boilerplate
-#-----------------------------------------------------------------------------
-import logging # isort:skip
+# -----------------------------------------------------------------------------
+from __future__ import annotations
+
+import logging  # isort:skip
 log = logging.getLogger(__name__)
 
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 # Imports
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 
 # Standard library imports
-import sys
+from typing import TYPE_CHECKING
 
 # Bokeh imports
 from bokeh.application.handlers.code_runner import CodeRunner
 from bokeh.application.handlers.handler import Handler
+from bokeh.core.types import PathLike
 from bokeh.io.doc import curdoc, set_curdoc
 
-#-----------------------------------------------------------------------------
+if TYPE_CHECKING:
+    from bokeh.document import Document
+
+# -----------------------------------------------------------------------------
 # Globals and constants
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 
 __all__ = (
-    'ExampleHandler',
+    "ExampleHandler",
 )
 
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 # General API
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 # Dev API
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
+
 
 class ExampleHandler(Handler):
-    """ A stripped-down handler similar to CodeHandler but that does
+    """A stripped-down handler similar to CodeHandler but that does
     some appropriate monkeypatching.
 
     """
 
-    _output_funcs = ['output_notebook', 'output_file', 'reset_output']
-    _io_funcs = ['show', 'save']
+    _output_funcs = ["output_notebook", "output_file", "reset_output"]
+    _io_funcs = ["show", "save"]
 
-    def __init__(self, source, filename):
-        super().__init__(self)
+    def __init__(self, source: str, filename: PathLike) -> None:
+        super().__init__()
         self._runner = CodeRunner(source, filename, [])
 
-    def modify_document(self, doc):
+    def modify_document(self, doc: Document) -> None:
         if self.failed:
             return
 
         module = self._runner.new_module()
 
-        sys.modules[module.__name__] = module
-        doc._modules.append(module)
+        doc.modules.add(module)
 
         orig_curdoc = curdoc()
         set_curdoc(doc)
@@ -76,10 +82,12 @@ class ExampleHandler(Handler):
             set_curdoc(orig_curdoc)
 
     def _monkeypatch(self):
+        def _pass(*args, **kw):
+            pass
 
-        def _pass(*args, **kw): pass
         def _add_root(obj, *args, **kw):
             curdoc().add_root(obj)
+
         def _curdoc(*args, **kw):
             return curdoc()
 
@@ -89,6 +97,7 @@ class ExampleHandler(Handler):
         # from io, and use those as the originals to replace everywhere
         import bokeh.io as io  # lgtm [py/import-and-import-from]
         import bokeh.plotting as p
+
         mods = [io, p]
 
         old_io = {}
@@ -102,14 +111,16 @@ class ExampleHandler(Handler):
                 setattr(mod, f, _add_root)
 
         import bokeh.document as d
+
         old_doc = d.Document
         d.Document = _curdoc
 
         return old_io, old_doc
 
     def _unmonkeypatch(self, old_io, old_doc):
-        import bokeh.io as io   # lgtm [py/import-and-import-from]
+        import bokeh.io as io  # lgtm [py/import-and-import-from]
         import bokeh.plotting as p
+
         mods = [io, p]
 
         for mod in mods:
@@ -117,24 +128,29 @@ class ExampleHandler(Handler):
                 setattr(mod, f, old_io[f])
 
         import bokeh.document as d
+
         d.Document = old_doc
 
     @property
-    def failed(self):
+    def failed(self) -> bool:
         return self._runner.failed
 
     @property
-    def error(self):
+    def error(self) -> str | None:
         return self._runner.error
 
     @property
-    def error_detail(self):
+    def error_detail(self) -> str | None:
         return self._runner.error_detail
 
-#-----------------------------------------------------------------------------
-# Private API
-#-----------------------------------------------------------------------------
+    @property
+    def doc(self) -> str | None:
+        return self._runner.doc
 
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
+# Private API
+# -----------------------------------------------------------------------------
+
+# -----------------------------------------------------------------------------
 # Code
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
