@@ -11,6 +11,8 @@
 # -----------------------------------------------------------------------------
 # Boilerplate
 # -----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging  # isort:skip
 
 log = logging.getLogger(__name__)
@@ -31,9 +33,9 @@ from jinja2 import Environment, FileSystemLoader
 
 __all__ = (
     "BJS_CODEPEN_INIT",
+    "BJS_EPILOGUE",
     "BJS_HTML",
-    "CCB_PROLOGUE",
-    "CCB_EPILOGUE",
+    "BJS_PROLOGUE",
     "COLOR_DETAIL",
     "ENUM_DETAIL",
     "GALLERY_PAGE",
@@ -43,6 +45,8 @@ __all__ = (
     "PALETTE_DETAIL",
     "PALETTE_GROUP_DETAIL",
     "PROP_DETAIL",
+    "SETTINGS_DETAIL",
+    "SRI_TABLE",
 )
 
 # -----------------------------------------------------------------------------
@@ -61,20 +65,18 @@ _env = Environment(loader=FileSystemLoader(_templates_path))
 # Dev API
 # -----------------------------------------------------------------------------
 
-CCB_PROLOGUE = _env.get_template("collapsible_code_block_prologue.html")
-CCB_EPILOGUE = _env.get_template("collapsible_code_block_epilogue.html")
-
 BJS_PROLOGUE = _env.get_template("bokehjs_content_prologue.html")
 BJS_EPILOGUE = _env.get_template("bokehjs_content_epilogue.html")
 
 BJS_CODEPEN_INIT = _env.get_template("bokehjs_codepen_init.html")
 BJS_HTML = _env.get_template("bokehjs_html_template.html")
 
-
 COLOR_DETAIL = _env.get_template("color_detail.html")
 
 ENUM_DETAIL = _env.get_template("enum_detail.rst")
 
+EXAMPLE_METADATA = _env.get_template("example_metadata.rst")
+
 GALLERY_PAGE = _env.get_template("gallery_page.rst")
 GALLERY_DETAIL = _env.get_template("gallery_detail.rst")
 
@@ -92,6 +94,8 @@ PROP_DETAIL = _env.get_template("prop_detail.rst")
 
 SETTINGS_DETAIL = _env.get_template("settings_detail.rst")
 
+SRI_TABLE = _env.get_template("sri_table.html")
+
 # -----------------------------------------------------------------------------
 # Code
 # -----------------------------------------------------------------------------
