@@ -12,6 +12,8 @@ interfaces to classes.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -21,11 +23,29 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 from inspect import signature
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    List,
+    Sequence,
+    Type,
+    Union,
+    cast,
+)
 
 # Bokeh imports
-from ..events import Event
+from ..core.types import Unknown
+from ..events import Event, ModelEvent
 from ..util.functions import get_param_info
 
+if TYPE_CHECKING:
+    from ..core.has_props import Setter
+    from ..core.types import ID
+    from ..document.document import Document
+    from ..document.events import DocumentPatchedEvent
+
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
@@ -39,16 +59,31 @@ __all__ = (
 # General API
 #-----------------------------------------------------------------------------
 
+# TODO (bev) the situation with no-argument Button callbacks is a mess. We
+# should migrate to all callbacks receving the event as the param, even if that
+# means auto-magically wrapping user-supplied callbacks for awhile.
+EventCallbackWithEvent = Callable[[Event], None]
+EventCallbackWithoutEvent = Callable[[], None]
+EventCallback = Union[EventCallbackWithEvent, EventCallbackWithoutEvent]
+
+PropertyCallback = Callable[[str, Unknown, Unknown], None]
+
 class EventCallbackManager:
     ''' A mixin class to provide an interface for registering and
     triggering event callbacks on the Python side.
 
     '''
-    def __init__(self, *args, **kw):
-        super().__init__(*args, **kw)
-        self._event_callbacks = dict()
 
-    def on_event(self, event, *callbacks):
+    document: Document | None
+    id: ID
+    subscribed_events: List[str]
+    _event_callbacks: Dict[str, List[EventCallback]]
+
+    def __init__(self, *args: Any, **kw: Any) -> None:
+        super().__init__(*args, **kw)  # type: ignore[call-arg] # https://github.com/python/mypy/issues/5887
+        self._event_callbacks = {}
+
+    def on_event(self, event: str | Type[Event], *callbacks: EventCallback) -> None:
         ''' Run callbacks when the specified event occurs on this Model
 
         Not all Events are supported for all Models.
@@ -70,35 +105,28 @@ class EventCallbackManager:
         if event not in self.subscribed_events:
             self.subscribed_events.append(event)
 
-    def _trigger_event(self, event):
-        def invoke():
+    def _trigger_event(self, event: ModelEvent) -> None:
+        def invoke() -> None:
             for callback in self._event_callbacks.get(event.event_name,[]):
                 if event._model_id is not None and self.id == event._model_id:
                     if _nargs(callback) == 0:
-                        callback()
+                        cast(EventCallbackWithoutEvent, callback)()
                     else:
-                        callback(event)
+                        cast(EventCallbackWithEvent, callback)(event)
 
-        # TODO: here we might mirror the property callbacks and have something
-        # like Document._notify_event which creates an *internal* Bokeh event
-        # (for the user event, confusing!) that then dispatches in the document
-        # and applies curdoc wrapper there. However, most of that machinery is
-        # to support the bi-directionality of property changes. Currently (user)
-        # events only run from client to server. Would like to see if some of the
-        # internal eventing can be reduced or simplified in general before
-        # plugging more into it. For now, just handle the curdoc bits here.
-        if hasattr(self, '_document') and self._document is not None:
-            self._document._with_self_as_curdoc(invoke)
+        if self.document is not None:
+            from ..model import Model
+            self.document.callbacks.notify_event(cast(Model, self), event, invoke)
         else:
             invoke()
 
-    def _update_event_callbacks(self):
+    def _update_event_callbacks(self) -> None:
         if self.document is None:
             return
 
         for key in self._event_callbacks:
-            self.document._subscribed_models[key].add(self)
-
+            from ..model import Model
+            self.document.callbacks.subscribe(key, cast(Model, self))
 
 class PropertyCallbackManager:
     ''' A mixin class to provide an interface for registering and
@@ -106,11 +134,14 @@ class PropertyCallbackManager:
 
     '''
 
-    def __init__(self, *args, **kw):
-        super().__init__(*args, **kw)
-        self._callbacks = dict()
+    document: Document | None
+    _callbacks: Dict[str, List[PropertyCallback]]
 
-    def on_change(self, attr, *callbacks):
+    def __init__(self, *args: Any, **kw: Any) -> None:
+        super().__init__(*args, **kw)  # type: ignore[call-arg] # https://github.com/python/mypy/issues/5887
+        self._callbacks = {}
+
+    def on_change(self, attr: str, *callbacks: PropertyCallback) -> None:
         ''' Add a callback on this object to trigger when ``attr`` changes.
 
         Args:
@@ -126,15 +157,13 @@ class PropertyCallbackManager:
 
         _callbacks = self._callbacks.setdefault(attr, [])
         for callback in callbacks:
-
             if callback in _callbacks:
                 continue
 
             _check_callback(callback, ('attr', 'old', 'new'))
-
             _callbacks.append(callback)
 
-    def remove_on_change(self, attr, *callbacks):
+    def remove_on_change(self, attr: str, *callbacks: PropertyCallback) -> None:
         ''' Remove a callback from this object '''
         if len(callbacks) == 0:
             raise ValueError("remove_on_change takes an attribute name and one or more callbacks, got only one parameter")
@@ -142,7 +171,8 @@ class PropertyCallbackManager:
         for callback in callbacks:
             _callbacks.remove(callback)
 
-    def trigger(self, attr, old, new, hint=None, setter=None):
+    def trigger(self, attr: str, old: Unknown, new: Unknown,
+            hint: DocumentPatchedEvent | None = None, setter: Setter | None = None) -> None:
         ''' Trigger callbacks for ``attr`` on this object.
 
         Args:
@@ -154,13 +184,14 @@ class PropertyCallbackManager:
             None
 
         '''
-        def invoke():
+        def invoke() -> None:
             callbacks = self._callbacks.get(attr)
             if callbacks:
                 for callback in callbacks:
                     callback(attr, old, new)
-        if hasattr(self, '_document') and self._document is not None:
-            self._document._notify_change(self, attr, old, new, hint, setter, invoke)
+        if self.document is not None:
+            from ..model import Model
+            self.document.callbacks.notify_change(cast(Model, self), attr, old, new, hint, setter, invoke)
         else:
             invoke()
 
@@ -172,12 +203,12 @@ class PropertyCallbackManager:
 # Private API
 #-----------------------------------------------------------------------------
 
-def _nargs(fn):
+def _nargs(fn: Callable[..., Any]) -> int:
     sig = signature(fn)
     all_names, default_values = get_param_info(sig)
     return len(all_names) - len(default_values)
 
-def _check_callback(callback, fargs, what="Callback functions"):
+def _check_callback(callback: Callable[..., Any], fargs: Sequence[str], what: str ="Callback functions") -> None:
     '''Bokeh-internal function to check callback signature'''
     sig = signature(callback)
     formatted_args = str(sig)
