@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -25,6 +25,8 @@ The default logging level is ``none``.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -34,7 +36,7 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 import sys
-from typing import Any, Optional, cast
+from typing import Any, cast
 
 # Bokeh imports
 from ..settings import settings
@@ -47,7 +49,7 @@ __all__ = (
   'basicConfig',
 )
 
-default_handler: Optional[logging.Handler] = None
+default_handler: logging.Handler | None = None
 
 #-----------------------------------------------------------------------------
 # General API
