@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -20,6 +20,9 @@ Functions:
         Return the base version string, without any "dev", "rc" or local build
         information appended.
 
+    is_full_release:
+        Return whether the current installed version is a full release.
+
 .. _versioneer: https://github.com/warner/python-versioneer
 
 '''
@@ -27,20 +30,17 @@ Functions:
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
+from __future__ import annotations
 
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
-# Standard library imports
-
-# External imports
-
 # Bokeh imports
+from .._version import get_versions
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -48,19 +48,21 @@ log = logging.getLogger(__name__)
 
 __all__ = (
     'base_version',
+    'is_full_release',
 )
 
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
 
-def base_version():
+def base_version() -> str:
     return _base_version_helper(__version__)
 
-def _base_version_helper(version):
+def is_full_release(version: str | None = None) -> bool:
     import re
-    VERSION_PAT = re.compile(r"^(\d+\.\d+\.\d+)((?:dev|rc).*)?")
-    return VERSION_PAT.search(version).group(1)
+    version = version or __version__
+    VERSION_PAT = re.compile(r"^(\d+\.\d+\.\d+)$")
+    return bool(VERSION_PAT.match(version))
 
 #-----------------------------------------------------------------------------
 # Dev API
@@ -70,10 +72,16 @@ def _base_version_helper(version):
 # Private API
 #-----------------------------------------------------------------------------
 
+def _base_version_helper(version: str) -> str:
+    import re
+    VERSION_PAT = re.compile(r"^(\d+\.\d+\.\d+)((?:dev|rc).*)?")
+    match = VERSION_PAT.search(version)
+    assert match is not None
+    return match.group(1)
+
 #-----------------------------------------------------------------------------
 # Code
 #-----------------------------------------------------------------------------
 
-from .._version import get_versions
 __version__ = get_versions()['version']
 del get_versions
